package sieve.beanbox;

import sieve.workbench.*;
import java.awt.*;

/**
 * The BeanBoxLink is used to connect arbitrary objects to or from
 * BeanBoxComponents such as PropertyBinders and EventAdapters.
 */
public class BeanBoxLink extends SimpleLink {
    // The source object for this link
    private Object src = null;
    // The destination object for this link
    private Object dest = null;
    // True if our destination object is a BeanBoxComponent
    private boolean bbcIsDest = false;

    /**
     * Registers this link type with Sieve's link factory.
     */
    public static void setup() {
	LinkFactory.getLinkFactory().registerLink(
	    BeanBoxComponent.class, Object.class, BeanBoxLink.class);
	LinkFactory.getLinkFactory().registerLink(
	    Object.class, BeanBoxComponent.class, BeanBoxLink.class);
    }

    public void link(WorkbenchComponent s, WorkbenchComponent d) 
	throws LinkException {

	// Get (and save) the objects that the two WorkbenchComponents represent
	src = s.getModel();
	dest = d.getModel();

	// Figure out which direction we are linking
	if (src instanceof BeanBoxComponent) {
	    try {
	        ((BeanBoxComponent)src).setDest(dest);
		// Remember which direction we are going
		bbcIsDest = false;
	    }
	    catch(Exception e) {
	    	throw new LinkException("Property binding failed");
	    }
	}
	else if (dest instanceof BeanBoxComponent) {
	    try {
	        ((BeanBoxComponent)dest).setSource(src);
		// Remember which direction we are going
		bbcIsDest = true;
	    }
	    catch(Exception e) {
		e.printStackTrace();
	    	throw new LinkException("Property binding failed");
	    }
	}
	else
	    throw new LinkException("Incompatible link type!");
    }

    public void unlink(WorkbenchComponent s, WorkbenchComponent d) { 
	if (bbcIsDest) 
	     ((BeanBoxComponent)dest).setSource(null);
	else 
	     ((BeanBoxComponent)src).setDest(null);
    }

}

