package sieve.beanbox;

import java.beans.*;
import java.awt.*;
import java.util.*;
import java.lang.reflect.*;
import com.sun.java.swing.*;
import com.sun.java.swing.event.*;

public class EventAdapterCustomizer extends Container implements 
	Customizer {
    private static Dimension zero = new Dimension(0,0);

    private JList interfaces = null;
    private JList eventMethods = null;
    private JList methods = null;

    private EventAdapter adapter = null;
    private PropertyChangeSupport listeners = null;
    private PropertyChangeListener spcl = null;

    private EventSetDescriptor[] esd = null;

    private boolean processing = false;

    public EventAdapterCustomizer() {
	setLayout(new GridLayout(1, 3));
	interfaces = new JList();
	interfaces.addListSelectionListener(new EventListUpdater());
	eventMethods = new JList();
	eventMethods.addListSelectionListener(new ListenerInfoUpdater());
	methods = new JList();
	methods.addListSelectionListener(new MethodInfoUpdater());
	JScrollPane iScroll = new JScrollPane(interfaces);
	iScroll.setMaximumSize(new Dimension(50, 50));
	JScrollPane emScroll = new JScrollPane(eventMethods);
	emScroll.setMaximumSize(new Dimension(50, 50));
	JScrollPane mScroll = new JScrollPane(methods);
	mScroll.setMaximumSize(new Dimension(50, 50));
	add(iScroll);
	add(emScroll);
	add(mScroll);
	listeners = new PropertyChangeSupport(this);
	spcl = new Reseter();
	setSize(new Dimension(200, 120));
    }

    private class Reseter implements PropertyChangeListener {
	public void propertyChange(PropertyChangeEvent e) {
	    reset();
	}
    }
    private class EventListUpdater implements ListSelectionListener {
	public void valueChanged(ListSelectionEvent e) {
	    EventSetWrapper esw = (EventSetWrapper) 
		interfaces.getSelectedValue();
	    if (esw != null)
		adapter.setListenerInterface(
		    esw.getEventSetDescriptor().getListenerType().getName());
	    resetEventMethods();
	}
    }
    private class ListenerInfoUpdater implements ListSelectionListener {
        public void valueChanged(ListSelectionEvent e) {
	    MethodDescriptorWrapper method = 
		(MethodDescriptorWrapper) eventMethods.getSelectedValue();
	    if (method != null) 
	        adapter.setListenerMethod(method.getMethodDescriptor());
        }
    }
    private class MethodInfoUpdater implements ListSelectionListener {
        public void valueChanged(ListSelectionEvent e) {
	    MethodDescriptorWrapper method = 
		(MethodDescriptorWrapper) methods.getSelectedValue();
	    if (method != null) 
	        adapter.setMethodToInvoke(method.getMethodDescriptor());
        }
    }

    private synchronized void reset() {
	if (! processing && (adapter != null)) {
	    processing = true;
	    Object source = adapter.getSource();
	    if (source != null) {
            	esd = null;
            	try {
                    esd = Introspector.getBeanInfo(
                    	source.getClass()).getEventSetDescriptors();
            	} 
		catch (IntrospectionException ie) {
                    System.err.println("Failed to introspect");
            	}
		EventSetWrapper[] esdw = new EventSetWrapper[esd.length];
		EventSetDescriptor selected = null;
		String selectedIf = adapter.getListenerInterface();
	    	for (int i = 0; i < esd.length; i++) {
		    esdw[i] = new EventSetWrapper(esd[i]);
		    if ((selectedIf != null) && 
			selectedIf.equals(esd[i].getListenerType().getName()))
			selected = esd[i];
		}
		interfaces.setListData(esdw);
		if (selected != null) {
		    interfaces.setSelectedValue(selected, true);
		    resetEventMethods();
		}
	    }
	    Object dest = adapter.getDest();
	    MethodDescriptor listenerMethod = adapter.getListenerMethod();
	    if ((dest != null) && (listenerMethod != null)) {
		// More code shamelessly stolen from BeanBox
		try {
		    MethodDescriptor[] mds = 
		        Introspector.getBeanInfo(dest.getClass(
			)).getMethodDescriptors();
		    Class eargs[] = 
			listenerMethod.getMethod().getParameterTypes();
		    Vector matchMethods = new Vector();
		    // Vector zeroArgMethods = new Vector();

		    MethodDescriptorWrapper selected = null;
		    MethodDescriptor current = adapter.getMethodToInvoke();

		    for (int i = 0; i < mds.length; i++) {
                	MethodDescriptor md = mds[i];
                	Class margs[] = md.getMethod().getParameterTypes();

 
                	if (margs.length == 0 &&
                            md.getMethod().getReturnType() == Void.TYPE) {
                    	    // zeroArgMethods.addElement(md);
			    // PLI -- putting everything into one vector
			    MethodDescriptorWrapper mdw = 
				new MethodDescriptorWrapper(md);
                	    matchMethods.addElement(mdw);
			    if ((current != null) && (current.equals(md)))
			         selected = mdw;
                    	    continue;
                	}
                	if (eargs.length != margs.length) {
                    	    continue;
                	}
                	boolean match = true;
                	for (int j = 0; j < eargs.length; j++) {
                    	    if (!isSubclass(eargs[j], margs[j])) {
                        	match = false;
                        	break;
                     	    }
                	}
                	if (match) {
			    MethodDescriptorWrapper mdw = 
				new MethodDescriptorWrapper(md);
                	    matchMethods.addElement(mdw);
			    if ((current != null) && (current.equals(md)))
			         selected = mdw;
                	}
		    } 
		    methods.setListData(matchMethods);
		    if (selected != null) 
		        methods.setSelectedValue(selected, true);
		}
		catch(Exception e) {
		    e.printStackTrace();
		}
		
	    }
	    processing = false;
	}
	else {
	    Object[] empty = new Object[0];
	    interfaces.setListData(empty);
	    eventMethods.setListData(empty);
	}
    }

    private void resetEventMethods() {
	// Should we allow multiple selection here?
	EventSetWrapper esw = (EventSetWrapper) interfaces.getSelectedValue();
	if (esw != null) {
	    MethodDescriptor[] methods = 
		esw.getEventSetDescriptor().getListenerMethodDescriptors();
	    MethodDescriptorWrapper[] wrappers = 
		new MethodDescriptorWrapper[methods.length];
	    MethodDescriptorWrapper selected = null;
	    MethodDescriptor selectedMethod = adapter.getListenerMethod();
	    for(int i = 0; i < methods.length; i++) {
		wrappers[i] = new MethodDescriptorWrapper(methods[i]);
		if ((selectedMethod != null) &&
		    selectedMethod.equals(methods[i]))
		    selected = wrappers[i];
	    }
	    eventMethods.setListData(wrappers);
	    if (selected != null)
		eventMethods.setSelectedValue(selected, true);
	    eventMethods.invalidate();
	    getParent().validate();
	    eventMethods.repaint();
	}
    }

    // This is here to get toString() to call getName() (for JList)
    private class EventSetWrapper {
	private EventSetDescriptor esd = null;
	public EventSetWrapper(EventSetDescriptor esd) {
	    this.esd = esd;
	}
	public EventSetDescriptor getEventSetDescriptor() {
	    return esd;
	}
	public String toString() {
	    return esd.getName();
	}
    }
    // Same for this one...
    private class MethodDescriptorWrapper {
	private MethodDescriptor md = null;
	public MethodDescriptorWrapper(MethodDescriptor md) {
	    this.md = md;
	}
	// We could do something with display name here, and then get
	// the "real" name when we go to set the property.
	public String toString() {
	    return md.getName();
	}
	public MethodDescriptor getMethodDescriptor() {
	    return md;
	}
    }

    // ------------------- Customizer methods -------------------
    public void setObject(Object bean) {
	if (adapter != null) 
	    adapter.removePropertyChangeListener(spcl);
	adapter = (EventAdapter)bean;
	if (adapter != null) 
	    adapter.addPropertyChangeListener(spcl);
	reset();
    }

    public void addPropertyChangeListener(PropertyChangeListener l) {
	listeners.addPropertyChangeListener(l);
    }

    public void removePropertyChangeListener(PropertyChangeListener l) {
	listeners.removePropertyChangeListener(l);
    }

    /**
     * Return true if class a is either equivalent to class b, or
     * if class a is a subclass of class b.
     * Note tht either or both "Class" objects may represent interfaces.
     */
    static boolean isSubclass(Class a, Class b) {
        // We rely on the fact that for any given java class or
        // primtitive type there is a unqiue Class object, so
        // we can use object equivalence in the comparisons.
        if (a == b) {
            return true;
        }
        if (a == null || b == null) {
            return false;
        }
        for (Class x = a; x != null; x = x.getSuperclass()) {
            if (x == b) {
                return true;
            }
            if (b.isInterface()) {
                Class interfaces[] = x.getInterfaces();
                for (int i = 0; i < interfaces.length; i++) {
                    if (interfaces[i] == b) {
                        return true;
                    }
                }
            }
        }
        return false;
    }

    public Dimension getPreferredSize() {
        Dimension current = getSize();
        return (current.equals(zero) ? super.getPreferredSize() : current);
    }

}


