
/*
 *
 * Written By:   Chengliang Yang
 * Date:         May 30 1997
 * Description:  Consumer for a channel that recieves messages from
 *		 channel and store them into a local table
 *
 */
 
package sieve.crew;

import sieve.quarry.*;
import SharedData.*;
import java.util.*;
import sieve.sluice.*;
import sieve.sluice.local.*;

/**
 * Class of channel consumer which listens to messages from the channel
 * and stores them into a local table
 *
 * @author 
 * @version 
 *
 */


public class Listener extends RemoteControl {
  public static final String ChannelName = "NuggetControl";
  private Hashtable participants = null;
  private SaveTable table = null;
 

  public Listener(Session session, String channelName) {
    
    super(session, ChannelName);
    participants = new Hashtable();
	try{
	    table = new SaveTable();
	} catch (InterchangeException e){}	
    addTableFields();
    debug("Listener started");
  }
 
  public void addTableFields(){
	try{
   	   FieldDescriptor fd;
		fd = new FieldDescriptor("MsgType");
		fd.setType("java.lang.String");
    		table.addField(fd);
		fd = new FieldDescriptor("Data");
		fd.setType("java.lang.String");
    		table.addField(fd);
	} catch (InterchangeException e){}
  }


  public synchronized void  notifyParticipants(int msgType, Object data) {
    debug("notifyParticipants: "+data);
      if (channel != null) {
	try {
	  channel.uniformSendToOthers(this, 
                  new Data(NuggetMessage.toByteArray(
			     new NuggetMessage(msgType, data))));
	} catch (Exception sdex) {
	  sdex.printStackTrace();
	}
      }
  }


  public synchronized void  sendStoredData(String receiver, int msgType, Object data) {
      debug("sendStoredData");
      if (channel != null) {
	debug(" channel is not null");
	try {
	  channel.sendToClient(this, receiver, new Data(NuggetMessage.toByteArray( new NuggetMessage(msgType, data))));
	debug("send data successfully");

	} catch (SharedDataException sdex) {
	  sdex.printStackTrace();
	}}}

  /* ------- ChannelConsumer Interface methods ------ */
  public void dataReceived(Data data) {
    Message msg = null;
    Object nugget = null;
    String sender = data.getSenderName();

    msg = (Message)Message.messageFromData(data);

    debug("data received");
    debug("data sender = " + data.getSenderName());
    debug("msgtype = "+ msg.type);

    switch(msg.type) {
	case NuggetMessage.REQUEST:{
		debug("REQUEST");
		int msgType, rcdNum = 0;
		Object storedData = null;
		Object [] record;
		String receiver = data.getSenderName();
		
		try {
			for (int i=0; i < this.table.recordCount(); i++){
				record = this.table.getRecord(i);	
						
			msgType = Integer.parseInt(record[0].toString());
			storedData =  record[1]; 		
			sendStoredData(receiver, msgType, storedData);
			}
		}catch (InterchangeException e){}
	}
	break;

    case NuggetMessage.CREATE: {
      	debug("CREATE");
	try{
		Object[] values = {new Integer(NuggetMessage.CREATE), msg.data};
		table.appendRecord(values);

		debug(" table contents ");
		table.printAllRecord();

      } catch (InterchangeException e){	}
     }	
     break;	

      // fall through...
    case NuggetMessage.MODIFY: {
	debug("Module modify");
	Object [] values = {new Integer(NuggetMessage.MODIFY), msg.data};
	try {
		table.appendRecord(values);
		debug("table contents");
		table.printAllRecord();

	} catch (InterchangeException e) {}


	}	break;
    
    case NuggetMessage.MOVE: {

	debug("Module moved");
	Object [] value = {new Integer(NuggetMessage.MOVE), msg.data};
	try {
		table.appendRecord(value);
		debug("table content");
		table.printAllRecord();

	} catch (InterchangeException e){}

	}
    	break;

	// currently only consider save the last modification
	// i.e, overwrite information saved

    case NuggetMessage.REMOVE:
 	// don't consider this case right now
	debug("Module removed ");
	break;
	
    case NuggetMessage.LINK_CREATE:      {
	debug("LINK_CREATE ");
	Object [] value = {new Integer(NuggetMessage.LINK_CREATE), msg.data};
	try {
		table.appendRecord(value);
		debug("table contents");
		table.printAllRecord();

	} catch (InterchangeException e){}

      } break;

    case NuggetMessage.LINK_REMOVE:      {

	debug("LINK_REMOVE ");

	Object [] values = {new Integer(NuggetMessage.LINK_REMOVE), msg.data};
	try {
		table.appendRecord(values);
		debug("table contents");
		table.printAllRecord();
	} catch (InterchangeException e){}
  
      } break;
	
    default:
      System.err.println("Unknown message type received.  Ignoring.");
    }
  }
  /* ------------------------------------------------ */





  /* ----------- Client Interface methods ----------- */
  public String authenticate(String challenge) {
    System.out.println("Sieve: athenticate: challenge="+challenge);
    return null;
  }


  public String getName() {
    debug("getName() "+name);
    return name;
  }
  public void clientChanged(ClientEvent event) {
        System.out.println("Sieve: clientChanged. event="+event);
  }
  /* ------------------------------------------------ */


  private static  final boolean debug = false;
  private void debug(String msg) {
    if (debug)
      System.out.println(getClass().getName()+": "+msg);
  } 
}











