/*
 *
 * Written By:   James "Bo" Begole
 * Date:         Feb 20 1997
 * Description:  Consumer for a channel that recieves block and plane control 
 *                   event messages.
 *                   These messages will be converted to calls to a
 *                   BlockAndPlane object.
 *
 * $Header: /home/nie/cvsroot/prototype/vt/cs/virtSchool/client/RoomView.java,v
1.1 1996/12/13 21:43:54 begolej Exp $
 *
 * Copyright (c) 1996 James Bo Begole
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */
 
package sieve.crew;

import sieve.quarry.*;
import SharedData.*;
import java.beans.*;
import java.lang.reflect.*;

/**
 * Consumer for a channel that recieves control 
 * event messages.
 * These messages will be converted to calls to a
 *  object.
 *
 * @version $Revision  $
 */
public class NuggetControl extends RemoteControl {
    public static final String ChannelName = "NuggetControl";

    private QuarryBox beanBox = null;

    private ParticipantControl participantControl = null;

  public NuggetControl(Session session, QuarryBox beanBox) {
    super(session, ChannelName);
    this.beanBox = beanBox;
  }

  public synchronized void  notifyParticipants(int msgType, Object data) {
    debug("notifyParticipants: "+data);
      if (channel != null) {
	try {
	  channel.uniformSendToOthers(this, 
                  new Data(NuggetMessage.toByteArray(
			     new NuggetMessage(msgType, data))));
	} catch (Exception sdex) {
	  sdex.printStackTrace();
	}
      }
  }

    public void setParticipantControl(ParticipantControl participantControl) {
	this.participantControl = participantControl;
    }

  /* ------- ChannelConsumer Interface methods ------ */
  public synchronized void dataReceived(Data data) {
    Message msg = null;
    
    Object nugget = null;

    msg = (Message)Message.messageFromData(data);
    switch(msg.type) {
    case NuggetMessage.CREATE:
      NuggetInfo ni = (NuggetInfo)(msg.data);
      debug("CREATE"+ni);
      try {
	Class cl = Class.forName(ni.getClassName());
	nugget = cl.newInstance();
      } catch (ClassNotFoundException cnf) {
	System.err.println("Class not found: "+cnf);
	cnf.printStackTrace();
      } catch (InstantiationException ie) {
	System.err.println("Cannot Instantiate: "+ie);
	ie.printStackTrace();
      } catch (IllegalAccessException ia) {
	System.err.println("Illegal Access: "+ia);
	ia.printStackTrace();
      }
      ni.setNugget(nugget);
      NuggetInfo.registerNuggetInfo(nugget, ni);
      debug("pre doInsert");
      beanBox.doRemoteInsert(nugget, ni.getBounds());
      debug("post doInsert");

	String creatorId = ni.getCreatorId();
	if ((creatorId != null) && (participantControl != null)) {
	    Participant creator = participantControl.getParticipant(creatorId);
	    ni.getWrapper().setColorSource(creator);
	    ni.getWrapper().repaint();
	}

      break;
    case NuggetMessage.REMOVE:
      break;
    case NuggetMessage.MODIFY:
      NuggetChangeInfo nci = (NuggetChangeInfo)(msg.data);
      nugget = nci.getNugget();
      Class nuggetClass = nugget.getClass();
      String propertyName = nci.getProperty();
      try {
        BeanInfo bi = Introspector.getBeanInfo(nuggetClass);
        PropertyDescriptor props[] = bi.getPropertyDescriptors();
        for (int i = 0; i < props.length; i++) {
            if (props[i].getName().equals(propertyName)) {
              Object[] args = { nci.getNewValue() };
              Method setter = props[i].getWriteMethod();
              setter.invoke(nugget, args);
              break;
            }
        }
        if (nugget instanceof java.awt.Component) {
          // Need to re-create property sheet if this nugget is currently
          // selected?
        }
        else {
          Wrapper wrapper = nci.getNuggetInfo().getWrapper();
          Object tvp = wrapper.getBean();
          if (tvp instanceof RemotePropertyChangeListener) {
            ((RemotePropertyChangeListener)tvp).remotePropertyChange(
		new PropertyChangeEvent(
                this, propertyName, null, nci.getNewValue()));
          }
        }
      } catch (IntrospectionException ex) {
        System.out.println("Caught introspection exception: " + ex);
        ex.printStackTrace();
      }
      catch (Exception e) {
        e.printStackTrace();
       }

//System.out.println("*** Got Property Change: " + nci + " for nugget: " +nci.getNugget());
      break;
    case NuggetMessage.MOVE:
      NuggetInfo remoteNI = (NuggetInfo)(msg.data);
      debug("MOVE "+remoteNI);
      NuggetInfo localNI = NuggetInfo.getNuggetInfo(remoteNI.getIdentifier());
      nugget = localNI.getNugget();
      localNI.setBounds(remoteNI.getBounds());
      // Do the move
      Wrapper wrapper = localNI.getWrapper();
      beanBox.startMove(wrapper,
			wrapper.location().x,
			wrapper.location().y);
      // PLI -- Adding width and height here. (Need to make a separate resize
      // message.)
      beanBox.remoteFinishMove(localNI.getBounds().x,
			 localNI.getBounds().y, localNI.getBounds().width,
			 localNI.getBounds().height);
      break;
    case NuggetMessage.LINK_CREATE: 
      {
	LinkInfo li = (LinkInfo)(msg.data);
	debug("LINK_CREATE "+li);
	NuggetInfo sourceNI = NuggetInfo.getNuggetInfo(li.getSourceIdentifier());
	NuggetInfo destNI = NuggetInfo.getNuggetInfo(li.getDestIdentifier());
	Wrapper sourceWrapper = sourceNI.getWrapper();
	Wrapper destWrapper = destNI.getWrapper();


	Link link = beanBox.doLinkRemote(sourceWrapper, destWrapper);
	LinkInfo.registerLinkInfo(link, li);
      } break;
    case NuggetMessage.LINK_REMOVE:
      {
	LinkInfo remoteLI = (LinkInfo)(msg.data);

System.out.println("LINK_REMOVE "+remoteLI);

	//LinkInfo localLI = LinkInfo.getLinkInfo(remoteLI.getIdentifier());
//System.out.println("localLI = " + localLI);

	  NuggetInfo sourceNI = 
		NuggetInfo.getNuggetInfo(remoteLI.getSourceIdentifier());
	  NuggetInfo destNI = 
		NuggetInfo.getNuggetInfo(remoteLI.getDestIdentifier());
	  Wrapper sourceWrapper = sourceNI.getWrapper();
	  Wrapper destWrapper = destNI.getWrapper();
	  Link link = 
		Link.findLink(sourceWrapper.getBean(), destWrapper.getBean());

System.out.println("Source wrapper = " + sourceWrapper + " dest wrapper = " + destWrapper + " link = " + link);

	if (link != null) {
	  // PLI Link link = localLI.getLink();

	  link.removeDest();
	  link.removeSource();
	  link.repaint();
	}
      } 
      break;
    default:
      System.err.println("Unknown message type received.  Ignoring.");
    }
  }
  /* ------------------------------------------------ */


  /* ----------- Client Interface methods ----------- */
  public String authenticate(String challenge) {
    System.out.println("Sieve: athenticate: challenge="+challenge);
    return null;
  }


  public String getName() {
    debug("getName() "+name);
    return name;
  }
  public void clientChanged(ClientEvent event) {
        System.out.println("Sieve: clientChanged. event="+event);
  }
  /* ------------------------------------------------ */


  private static  final boolean debug = false;
  private void debug(String msg) {
    if (debug)
      System.out.println(getClass().getName()+": "+msg);
  } 
}

