/*
 * %W% %E%  James Bo Begole
 *
 * Copyright (c) 1997 James Begole,  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * James Begole MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. James Begole 
 * SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */
 
package sieve.crew;

import java.awt.*;

/**
 * A Participant of a collaborative session
 *
 * @version 	$Revision: 1.6 $
 * @author 	James "Bo" Begole
 */

public class Participant 
extends java.util.Observable 
implements java.io.Serializable,   vt.cs.widgets.RadarViewPort,   
    java.util.Observer, sieve.quarry.ColorSource {
  
  String identifier = null;

  /** 
   * The name of the Participant
   */
  private String name = null;
  
  /** 
   * The color of this participant
   */
  private Color color = null;
  
  private Rectangle bounds = null;
  private Point cursorLocation = null;
  
  public Participant(String name, Color color) {
    this.name = name;
    this.color = color;
    this.identifier = "participant-"+System.currentTimeMillis();
  }
  
  public Participant() {
  }
  
  public Participant(Participant toCopy) {
    setParticipant(toCopy);
  }
  
  public  String getIdentifier() {
    return identifier;
  }
  public synchronized void setIdentifier(String identifier) {
    this.identifier = identifier;
  }
  public  String getName() {
    return name;
  }
  
  public  Color getColor() {
    return color;
  }
  
  public synchronized void setName(String name) {
    this.name = name;
    setChanged();
    notifyObservers("Name");
  }
  
  public synchronized void setColor(Color color) {
    this.color = color;
    setChanged();
    notifyObservers("Color");
  }
  
  public synchronized void setBounds(Rectangle bounds) {
    this.bounds = bounds;
    setChanged();
    notifyObservers("Bounds");
  }
  public  Rectangle getBounds() {
    return bounds;
  }
  public synchronized void setLocation(Point location) {
    debug("setLocation "+location);
    this.cursorLocation = location;
    setChanged();
    notifyObservers("Location");
  }
  public  Point getLocation() {
    return cursorLocation;
  }
  public  Point getCursorPosition() {
    return getLocation();
  }
  public   String getLabel() {
    return getName();
  }
  
  public synchronized void setParticipant(Participant participant) {
    this.color = participant.getColor();
    this.bounds = participant.getBounds();
    this.cursorLocation = participant.getLocation();
    this.name = participant.getName();
    this.identifier = participant.getIdentifier();
    setChanged();
    notifyObservers();
  }
  
  public synchronized boolean equals(Object obj) {
    debug("name = "+name+" color = "+color+" obj = "+obj);
    if (!(obj instanceof Participant))
      return false;
    else if (obj == null)
      return false;
    
    if (name == null) {
      if (((Participant)obj).getName() != null)
	return false;
    } else if (!name.equals(((Participant)obj).getName()))
      return false;
    
    if (color == null) {
      if (((Participant)obj).getColor() != null)
	return false;
    } else if (!color.equals(((Participant)obj).getColor()))
      return false;
    
    return true;
  }
  

// --------- Observer Methods ---------------
  public synchronized void update(java.util.Observable o, Object arg){
    debug("update "+o+" :"+arg);
    if (o instanceof Participant) {
      if ("Name".equals(arg)) {
	this.name = ((Participant)o).getName();
	setChanged();
	notifyObservers("Name");
      }
      if ("Color".equals(arg)){
	this.color = ((Participant)o).getColor();
	setChanged();
	notifyObservers("Color");
      }
    }
  }
// --------- end Observer Methods ---------------


  public synchronized String toString() {
    return new String("Participant: name: "+name+" color:"+color);
  }

  private static final boolean debug = false;
  private void debug(String message){
    if (debug)
      System.err.println(getClass().getName()+": "+message);
  }    

  // PLI -- still looking for sync problem
  private synchronized void writeObject(java.io.ObjectOutputStream out) 
    throws java.io.IOException {
      out.defaultWriteObject();
  }
}


