/*
 * @(#)ParticipantDialog.java	1.6 96/11/20   James "Bo" Begole
 *
 * Copyright (c) 1995-1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */
package sieve.crew;

import java.awt.*;

/**
 * A Dialog to collect information from a participant
 *
 * @version 	1.6, 11/20/96
 * @author 	James "Bo" Begole
 */



public class ParticipantDialog extends Dialog {

    private static final String initialInstruction =
        "Enter your name and select a color.";
    private static final String nameRequired =
        "Please enter your name.";
    private static final String colorRequired =
        "Please select a color.";
    TextField nameField = new TextField(20);
    Participant returnParticipant = null;
    Label instructionLabel = new Label(initialInstruction);
    Choice colorChoice = new Choice();

    public ParticipantDialog(Frame parent, Participant returnParticipant) {
	super(parent, "Participant Information", true);
	this.returnParticipant = returnParticipant;
	
	setLayout(new BorderLayout());

	// Set up color Choice
	//	colorChoice.addItem("black");
	colorChoice.addItem("blue");
	colorChoice.addItem("cyan");
	colorChoice.addItem("darkGray");
	colorChoice.addItem("gray");
	colorChoice.addItem("green");
	//colorChoice.addItem("lightGray");
	colorChoice.addItem("magenta");
	colorChoice.addItem("orange");
	colorChoice.addItem("pink");
	colorChoice.addItem("red");
	//	colorChoice.addItem("white");
 	colorChoice.addItem("yellow");

	Panel namePanel = new Panel();
	namePanel.setLayout(new FlowLayout());
	namePanel.add(new Label("Name: "));
	namePanel.add(nameField);

	Panel infoPanel = new Panel();
	infoPanel.setLayout(new BorderLayout());
	infoPanel.add("North", instructionLabel);
	infoPanel.add("Center", namePanel);
	infoPanel.add("South", colorChoice);

	Panel buttonPanel = new Panel();
	buttonPanel.setLayout(new FlowLayout());
	buttonPanel.add(new Button("Apply"));
	buttonPanel.add(new Button("Cancel"));

	//	add("North", new Label("Information about you to share with others."));
	add("Center", infoPanel);
	add("South", buttonPanel);
	restoreParticipant();
	pack();
	// this shouldn't be necessary, pack doesn't work
	// on Win NT
	//resize(300, 175); 
    }

    private boolean saveParticipant() {
	if (nameField.getText() == null) {
	    instructionLabel.setText(nameRequired);
	    return false;
	} else if (parseColor(colorChoice.getSelectedItem()) == null) {
	    instructionLabel.setText(colorRequired);
	    return false;
	} else {
	    returnParticipant.setName(nameField.getText());
	    returnParticipant.setColor(parseColor(colorChoice.getSelectedItem()));
	    return true;
	}
    }

    private void restoreParticipant() {
	if ((returnParticipant.getName() != null) && 
	    (returnParticipant.getColor() != null)) {
	    nameField.setText(returnParticipant.getName());
	    colorChoice.select(parseColor(returnParticipant.getColor()));
	}
    }

    private String parseColor(Color color) {
	if (color != null) {
	    if (color.equals(Color.black))
		return "black";
	    else if (color.equals(Color.blue))
		return "blue";
	    else if (color.equals(Color.cyan))
		return "cyan";
	    else if (color.equals(Color.darkGray))
		return "darkGray";
	    else if (color.equals(Color.gray))
		return "gray";
	    else if (color.equals(Color.green))
		return "green";
	    else if (color.equals(Color.lightGray))
		return "lightGray";
	    else if (color.equals(Color.magenta))
		return "magenta";
	    else if (color.equals(Color.orange))
		return "orange";
	    else if (color.equals(Color.pink))
		return "pink";
	    else if (color.equals(Color.red))
		return "red";
	    else if (color.equals(Color.white))
		return "white";
	    else if (color.equals(Color.yellow))
		return "yellow";
	}
	return null;
    }

    private Color parseColor(String colorName) {
	if (colorName != null) {
	    if (colorName.equalsIgnoreCase("black")) 
		return Color.black;
	    else if (colorName.equalsIgnoreCase("blue"))
		return Color.blue;
	    else if (colorName.equalsIgnoreCase("cyan"))
		return Color.cyan;
	    else if (colorName.equalsIgnoreCase("darkGray"))
		return Color.darkGray;
	    else if (colorName.equalsIgnoreCase("gray"))
	    return Color.gray;
	    else if (colorName.equalsIgnoreCase("green"))
		return Color.green;
	    else if (colorName.equalsIgnoreCase("lightGray"))
		return Color.lightGray;
	    else if (colorName.equalsIgnoreCase("magenta"))
		return Color.magenta;
	    else if (colorName.equalsIgnoreCase("orange"))
		return Color.orange;
	    else if (colorName.equalsIgnoreCase("pink"))
		return Color.pink;
	    else if (colorName.equalsIgnoreCase("red"))
		return Color.red;
	    else if (colorName.equalsIgnoreCase("white"))
		return Color.white;
	    else if (colorName.equalsIgnoreCase("yellow"))
		return Color.yellow;
	}
	return null;
    }


    public boolean action(Event evt, Object arg) {
	if ((arg.equals("Apply")) || (evt.target.equals(nameField))) {
	    if (saveParticipant())
		hide();
	    return true;
	} else if (arg.equals("Cancel")) {
	    restoreParticipant();
	    if (saveParticipant())
		hide();
	    return true;
	} 
	return false;
    }


  public static Color getRandomColor() {
    java.util.Random random = new java.util.Random();
    int colorIndex = random.nextInt() % 10;
    switch (colorIndex) {
    case 0:
       return Color.blue;
    case 1:
      return Color.cyan;
    case 2:
      return Color.gray;
    case 3:
      return Color.green;
    case 4:
      return Color.darkGray;
    case 5:
      return Color.magenta;
    case 6:
      return Color.orange;
    case 7:
      return Color.pink;
    case 8:
      return Color.red;
    case 9:
      return Color.yellow;
    default:
      return Color.orange;
    }
  }
}
