package sieve.nuggets;

import java.awt.*;
import java.awt.event.*;
import com.sun.java.swing.*;
import sieve.util.SwingButtonGroupBox;
import sieve.sluice.*;
import java.util.Enumeration;


/**
 * extends RegressionPlot and allows user to drag points and delete them.
 * Class EditableRegressionPlot includes the radio buttons used to select Move
 * or Delete mode and to select which field. Does NOT propagate move/delete changes
 * to downstream view because the plot data is stored in integer arrays rather than
 * in a TableView (had to duplicate x-values for each y-variable). 
 *
 * @see sieve.sluice.TableView
 * @see sieve.nuggets.RegressionPlot
 * @see sieve.nuggets.EditableRegressionPlot
 *  
 * @version	%I%, %G%
 * @author	Lynn W. Jones
 */

public class EditablePlot extends RegressionPlot 
implements TableView,TableViewListener,MouseListener,MouseMotionListener,ActionListener {
//RegressionPlot extends TableViewJComponent

	private static final int	MOVING = 2;
	private static final int	DELETING = 3;
	private static final int	DELETED = -1;
	public  static final String	moveStr = "Move Points";
	public  static final String	delStr = "Delete Points";

	private Image		imgBuf = null;	//offscreen image
	private int[][]		xCopies = null;	//make separate copies of x vals for each y field
	private int			fieldNo = 1;	//field number selected for add/delete
	private int			recNo;			//record number clicked on
	private boolean		pointSelected = false;
	private int			mode = MOVING;


	/**
	 * source table, initial fields to use, maximum number of records
	 * Calls super(src, selections, max)
	 */
	public EditablePlot(TableView src, String[] selections, int max) {
		super(src,selections,max);
		initLocal();
	}

	/**
	 * source table, initial fields to use -- Calls super(src, selections, 0)
	 */
	public EditablePlot(TableView src, String[] selections) {
		super(src,selections,0);
		initLocal();
	}

	/**
	 * source table -- Calls super(src, null, 0)
	 */
	public EditablePlot(TableView src) {
		super(src,null,0);
		initLocal();
	}

	/**
	 * Calls super(null, null, 0)
	 */
	public EditablePlot() {
		super(null,null,0);
		initLocal();
	}

	/**
	 * called by all constructors
	 */
	private void initLocal() {
		addMouseListener(this);
		addMouseMotionListener(this);
		//drawLabels = false;
		if (plotData != null) {
			int rows = plotData.length-1;
			int cols = plotData[0].length;
			//copy all the x values so if change for one variable, don't change for all
			xCopies = new int[rows][cols];
			for (int r=0;r<rows;r++)
				for (int s=0;s<cols;s++)
					xCopies[r][s] = plotData[0][s];
		}
	}

	/**
	 * returns true if user's mouse over a data point
	 */
	private boolean overGraph(int X,int Y) {
		if ((X > baseX-3) && (X < maxX+3) && (Y < baseY+3) && (Y > maxY-3))
			return true;
		else
			return false;
	}

	/**
	 * returns true if user's mouse click over a data point
	 */
	private boolean pointClicked(int x, int y) {
		int lowX = x-3;		//give some range in clicking on a point
		int highX = x+3;
		int lowY = y+3;
		int highY = y-3;
		int k;
		//fieldNo is set by the user with a radio group
		//this loop looks through the x values to find a match in that field
		for (k=0;k<maxTemp;k++) { //maxTemp is currect number of records 
			if((xCopies[fieldNo-1][k] > lowX) && (xCopies[fieldNo-1][k] < highX)) { //x found
				if((plotData[fieldNo][k] < lowY) && (plotData[fieldNo][k] > highY))	{
					recNo = k;
					return true;
				}
			}
		}
		//if didn't return true already, point not clicked
		return false;
	}

	/**
	 * Calculates new regression line(s) after point moved or deleted
	 */
	private void recalcRegression() {
		//the only change is in plotData[fieldNo][recNo], so we can leave all the other
		//paint data the same and only go through this part of the array
		if (actualFields > 1) { //this shouldn't be called if this fails, but for safety...
			double sumX2,sumX, sumY,sumXY;	//These for calculating the regression line
			double temp;
			double mxX,mxY,mnX,mnY;
			int xTemp,yTemp,intercept;
			int deletedCount = 0;
			double denom, slope;
			Point leftEnd,rightEnd;	
	
			leftEnd = new Point(0,0);
			rightEnd = new Point(0,0);
			sumX2 = sumX = sumY = sumXY = 0.0;
		
			for(int i=0; i<maxTemp; i++) {
			//sum data for regression line
				xTemp = xCopies[fieldNo-1][i];
				yTemp = plotData[fieldNo][i];
				if(yTemp != DELETED) {
					sumX += xTemp;
					sumX2 += xTemp*xTemp;
					sumY += yTemp;
					sumXY += xTemp*yTemp;
				}
				else
					deletedCount++;
			}

			//make the denominator
			denom = (maxTemp - deletedCount)*sumX2 - (sumX * sumX);
			
			if (denom != 0) {	  //just in case...
				//make intercept and slope
				intercept =(int)(((sumX2*sumY) - (sumX*sumXY)) / denom);
				slope =  (((maxTemp - deletedCount)*sumXY) - (sumX*sumY)) / denom;

				//do some checking to draw correctly, only on the graph area
				for (int h=0;h<1;h++) {	//make loop so I can copy corrected
					//left endpoint		//...version from RegressionPlot, using breaks
					xTemp = baseX;
					yTemp = intercept;
					if (intercept <= maxY) {
						if (slope <= 0) {
							leftEnd.x = 0;
							leftEnd.y = 0;
							rightEnd.x = 0;
							rightEnd.y = 0;
							break;
						}
						else {
							xTemp = (int)((maxY-intercept)/slope);
							yTemp = maxY;	/*if left endpoints not right, add test here like at "reference" comment below*/
						}
					}
					else if (intercept >= baseY) {
						if (slope >= 0) {
							leftEnd.x = 0;
							leftEnd.y = 0;
							rightEnd.x = 0;
							rightEnd.y = 0;
							break;
						}
						else {
							xTemp = (int)((baseY-intercept)/slope);
							yTemp = baseY;
							if(xTemp < baseX) { /*reference*/
								xTemp = baseX;
								yTemp = ((int)(slope * xTemp) + intercept);
						}						}
					}
					if (xTemp >= maxX) {	//then the left endpoint starts off the graph
						leftEnd.x = 0;
						leftEnd.y = 0;
						rightEnd.x = 0;
						rightEnd.y = 0;
						break;
					}
					leftEnd.x = xTemp;
					leftEnd.y = yTemp;
					
					//right endpoint
					xTemp = maxX;
					yTemp = (int)(slope*maxX) + intercept;
					if (yTemp <= maxY) {
						xTemp = (int)((maxY-intercept)/slope); 
						yTemp = maxY;
					}
					else if (yTemp >= baseY) {
						xTemp = (int)((baseY-intercept)/slope); 
						yTemp = baseY;
					}
					rightEnd.x = xTemp;
					rightEnd.y = yTemp;
				}//end for
			}//end if denom != 0
			else {	 //if denominator == 0, don't do the division
				slope = 0;						  //draw a horizontal line
				intercept =(int)(sumY / (maxTemp - deletedCount));//at the average y-value
				leftEnd.y = rightEnd.y = intercept;
				leftEnd.x = baseX;
				rightEnd.x = maxX;
			}  
			leftEndpoints[fieldNo - 1] = leftEnd;
			rightEndpoints[fieldNo -1] = rightEnd;
		}//end if actualFields >1
	}

	/**
	 * Override superclass paintComponent to use xCopies[][] values instead of
	 * plotData[][] values and to provide image buffering -- swing
	 * docs say override paintComponent instead of paint
	 */
	public void paintComponent(Graphics gr) {
		int i;
		Graphics g;
		if(imgBuf == null) {
			imgBuf = createImage(size().width,size().height);
			if(imgBuf == null)		//repaint called by some initialization methods
				return;				//this will be null b/c hasn't been added yet
		}
		g = imgBuf.getGraphics(); 
		g.clearRect(0,0,size().width,size().height);
		g.setColor(Color.white);
		g.fillRect(0,0,getWidth(),getHeight());

		g.setColor(Color.black);

		//draw border and axes
		g.drawRect(0,0,size().width-1,size().height-1);
		g.drawLine(baseX,baseY,baseX,maxY);
		g.drawLine(baseX,baseY,maxX,baseY);
    
		//label x-axis and min/max values
		g.drawString("x: " + x_yLabels[0],baseX-5,263);
		g.drawString(dataMinX.toString(),baseX-8,245);
		g.drawString(dataMinY.toString(),10,baseY);
		g.drawString(dataMaxY.toString(),10,baseY-192);
		g.drawString(dataMaxX.toString(),maxX-10,245);
	
		//draw tic marks on axes
		for(i=1;i<5;i++) {
		  g.fillRect(baseX-2+(50*i),baseY-2,2,4);
		  g.fillRect(baseX-2,baseY-2-(50*i),4,2);
		}//end for

		//draw the data points
		if (maxTemp > 200)
			dotSize = 3;
		else
			dotSize = 5;
		int n = colors.length;
		for(i=1;i<actualFields;i++) {
			if(i<=n)
				g.setColor(colors[i-1]);
			else
				g.setColor(Color.black);
			for (int k=0;k<maxTemp;k++) {
				if(plotData[i][k] != DELETED)	//for the subclass EditableRegressionPlot which
												//allows users to delete and move points
				  g.fillOval(xCopies[i-1][k]-dotSize/2, plotData[i][k]-dotSize/2, dotSize, dotSize);
			}
			//draw the regression lines, double width
			g.setXORMode(Color.white);
			g.drawLine(leftEndpoints[i-1].x,leftEndpoints[i-1].y,rightEndpoints[i-1].x,rightEndpoints[i-1].y);
			g.drawLine(leftEndpoints[i-1].x,leftEndpoints[i-1].y-1,rightEndpoints[i-1].x,rightEndpoints[i-1].y-1);
			g.setPaintMode();
			//label it in same color, at top if only 1 y-field, else listing by color
		}
		gr.drawImage(imgBuf,0,0,null);
	} 

	/**
	 * Changing number of records discards any edits user has made and re-fetches data
	 */
	public void setMaxRecords(int toPlot) {
		super.setMaxRecords(toPlot);	//this will also fire propertyChange since
		//copy the x values again		//...listeners have been registered to plot
		if (plotData != null) {
			int rows = plotData.length-1;
			int cols = plotData[0].length;
			xCopies = new int[rows][cols];
			for (int r=0;r<rows;r++)
				for (int s=0;s<cols;s++)
					xCopies[r][s] = plotData[0][s];
		}
		repaint();			
	}			

	//-------------------------------------------------------------
	/**
	 * TableView interface methods
	 */
	public void addSourceView(TableView src) throws TooManySourcesException {
		try { 
			super.addSourceView(src);
			//copy the x values again
			if (plotData != null) {
				int rows = plotData.length-1;
				int cols = plotData[0].length;
				xCopies = new int[rows][cols];
				for (int r=0;r<rows;r++)
					for (int s=0;s<cols;s++)
						xCopies[r][s] = plotData[0][s];
			}
		}
		catch(TooManySourcesException exc) {throw (exc);}
	}

	/** 
	 * other TableView interface methods pass through to parent
	 */
	//addTableViewListener(TableViewListener) 
	//get(String[], int) 
    //get(String[], int, int) 
    //getFieldDescriptor(String) 
    //getFieldNames() 
    //getRecordCount() 
    //getTableDescriptor() 
    //removeSourceView(TableView) 
	//removeTableViewListener(TableViewListener) 
	//table() 


	//-------------------------------------------------------------
	/**
	 * TableViewListener interface method
	 */
	public void tableViewChanged(TableViewEvent t) {
		super.tableViewChanged(t); //calls repaint

		//copy the x values again
		if (plotData != null) {
			int rows = plotData.length-1;
			int cols = plotData[0].length;
			xCopies = new int[rows][cols];
			for (int r=0;r<rows;r++)
				for (int s=0;s<cols;s++)
					xCopies[r][s] = plotData[0][s];
		}
	}


	//-------------------------------------------------------------
	/**
	 * MouseListener interface methods
	 */
	public void mouseClicked(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
   	public void mouseExited(MouseEvent e) {}

   	public void mousePressed(MouseEvent e) 	{	
		int mX = e.getX();
		int mY = e.getY();
		if (overGraph(mX,mY)) { 
			pointSelected = pointClicked(mX,mY);	//sets moving flag if true
			if(pointSelected) {
				if (mode==MOVING)
					setCursor(new Cursor(Cursor.MOVE_CURSOR));
				else if (mode == DELETING)
					setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
			}	
		}
	}

    public void mouseReleased(MouseEvent e) {
		if(pointSelected) {				//got point on mouseDown
			if (mode == DELETING) {		//if mode deleting, only delete if user didn't move off point
				if( (Math.abs(xCopies[fieldNo-1][recNo] - e.getX()) < 3) && (Math.abs(plotData[fieldNo][recNo] - e.getY()) < 3)) {
					plotData[fieldNo][recNo] = DELETED;	//mark it deleted
				}
			}
			pointSelected = false;
			recalcRegression();		//data already scaled; use plot data to redraw this line
			repaint();
		}
		if(overGraph(e.getX(),e.getY()))
			setCursor(new Cursor(Cursor.HAND_CURSOR));
		else
			setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
	}


	//-------------------------------------------------------------
	/**
	 * MouseMotionListener interface methods
	 */
	public void mouseDragged(MouseEvent e) {
		if ((pointSelected) && (mode == MOVING)) {
			int x = e.getX();
			int y = e.getY();
			if((x>=baseX) && (x<=maxX)) 
				xCopies[fieldNo-1][recNo] = x;
			if ((y<=baseY) && (y>=maxY)) 
				plotData[fieldNo][recNo] = y;
			repaint();
		}
	}

	public void mouseMoved(MouseEvent e) {
		if(overGraph(e.getX(),e.getY()))
			setCursor(new Cursor(Cursor.HAND_CURSOR));
		else
			setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
	}


	//-------------------------------------------------------------
	/** 
	 * ActionListener interface method
	 */
	public void actionPerformed(ActionEvent e) {
		String com = e.getActionCommand();
		if (com.compareTo(moveStr)==0) 
			mode = MOVING;
		else if (com.compareTo(delStr)==0) 
			mode = DELETING;
		else {	//action came from field names radio -- find the field
			int num = x_yLabels.length;
			for(int k=1;k<num;k++) {	//compare to y labels only
				if(com.compareTo(x_yLabels[k])==0) {
					fieldNo = k;
					break;
				}
			}
		}
	}

	/**
	 * RegressionPlot superclass also defines add/remove TableViewListeners
	 * by calling table.add / remove...
	 */

	
	public Dimension getPreferredSize() {
		return new Dimension(300,300);
    }
    
    public Dimension getMinimumSize() {
		return new Dimension(300,300);
    }
	
	public Dimension preferredSize() {
		return getPreferredSize();
    }
    
    public Dimension minimumSize() {
		return getMinimumSize();
    }


}

