package sieve.nuggets;

import java.util.*;
import java.awt.*;
import java.applet.Applet;
import sieve.sluice.*;
import sieve.sluice.event.*;
import sieve.sluice.views.LabelView;
import vt.cs.widgets.RadarPane;

class Node {
    double x;
    double y;

    double dx;
    double dy;

    boolean fixed;

    Color color;
    String field;

    String lbl;
}

class Edge {
    int from;
    int to;
    int count = 1;

    double len;
}

class GraphPanel extends Canvas implements Runnable {
    Graph graph;
    int nnodes;
    Node nodes[] = new Node[1000];

    int nedges;
    Edge edges[] = new Edge[2000];

    Thread relaxer;
    boolean stress;
    boolean random;
    boolean secDone;	  

    Date d = new Date();

    long nodeSeconds = d.getTime();

    GraphPanel(Graph graph) {
	this.graph = graph;
    }

    public void reset() {
	nodes = new Node[1000];
	edges = new Edge[2000];
	nedges = 0;
	nnodes = 0;
	stop();
    }

    int findNode(String lbl, Color color, String field) {
	for (int i = 0 ; i < nnodes ; i++) {
	    if (nodes[i].lbl.equals(lbl)) {
// PLI: Merge colors here? (e.g., if a label appears in multiple fields?)
		return i;
	    }
	}
	return addNode(lbl, color, field);
    }

    int addNode(String lbl, Color color, String field) {
        Random rn = new Random(nnodes);

	Node n = new Node();
	n.x = 10 + 380*rn.nextFloat();
	n.y = 10 + 380*rn.nextFloat();
	n.lbl = lbl;
        n.color = color;
        n.field = field;
	nodes[nnodes] = n;
	return nnodes++;
    }

    void addEdge(String from, Color fromColor, String fromField,
	String to, Color toColor, String toField, int len) {
	int count = 0;
	boolean finished = false;
	Edge e = new Edge();
	e.from = findNode(from, fromColor, fromField);
	e.to = findNode(to, toColor, toField);
	e.len = len;
//****************************************************
        while ((count < nedges) && (!finished)) {
	    if ((edges[count].from == e.from) && 
		(edges[count].to == e.to)) {
		   edges[count].count++;
		   finished = true;
	    } 
	    count++;
	}
	if (!finished) {
	    edges[nedges++] = e;
	}
//*****************************************************
    }

    public void run() {
      int counter = 0;
      while (true) {
	relax();
	Random rn = new Random(12345);
	rn.setSeed(12345);
	if (random && (rn.nextFloat() < 0.03)) {
	  Node n = nodes[(rn.nextInt() * nnodes)];
	  if (!n.fixed) {
	    n.x += 100*rn.nextFloat() - 50;
	    n.y += 100*rn.nextFloat() - 50;
	  }
	}
	try {
	  Thread.yield();
	  Thread.sleep(1000);
	} catch (InterruptedException e) {
	  break;
	}
      }
    }
  
  synchronized void relax() {
    Random rn = new Random(12345);
	/*rn.setSeed(12345);*/
	for (int i = 0 ; i < nedges ; i++) {
	    Edge e = edges[i];
	    double vx = nodes[e.to].x - nodes[e.from].x;
	    double vy = nodes[e.to].y - nodes[e.from].y;
	    double len = Math.sqrt(vx * vx + vy * vy);
	    double f = (edges[i].len - len) / (len * 3) ;
	    double dx = f * vx;
	    double dy = f * vy;

	    nodes[e.to].dx += dx;
	    nodes[e.to].dy += dy;
	    nodes[e.from].dx += -dx;
	    nodes[e.from].dy += -dy;
	}

	for (int i = 0 ; i < nnodes ; i++) {
	    Node n1 = nodes[i];
	    double dx = 0;
	    double dy = 0;

	    for (int j = 0 ; j < nnodes ; j++) {
		if (i == j) {
		    continue;
		}
		Node n2 = nodes[j];
		double vx = n1.x - n2.x;
		double vy = n1.y - n2.y;
		double len = vx * vx + vy * vy;
		if (len == 0) {
		    dx += rn.nextFloat();
		    dy += rn.nextFloat();
		} else if (len < 100*100) {
		    dx += vx / len;
		    dy += vy / len;
		}
	    }
	    double dlen = dx * dx + dy * dy;
	    if (dlen > 0) {
		dlen = Math.sqrt(dlen) / 2;
		n1.dx += dx / dlen;
		n1.dy += dy / dlen;
	    }
	}

	Dimension d = size();
	for (int i = 0 ; i < nnodes ; i++) {
	    Node n = nodes[i];
	    if (!n.fixed) {
		n.x += Math.max(-5, Math.min(5, n.dx));
		n.y += Math.max(-5, Math.min(5, n.dy));
		//System.out.println("v= " + n.dx + "," + n.dy);
		if (n.x < 0) {
		    n.x = 0;
		} else if (n.x > d.width) {
		    n.x = d.width;
		}
		if (n.y < 0) {
		    n.y = 0;
		} else if (n.y > d.height) {
		    n.y = d.height;
		}
	    }
	    n.dx /= 2;
	    n.dy /= 2;
	//n.fixed = true;
	}
	repaint();
    }

    Node pick;
    boolean pickfixed;
    Image offscreen;
    Dimension offscreensize;
    Graphics offgraphics;
    

    //final Color fixedColor = Color.red;
    final Color fixedColor = new Color(250, 220, 100);
    final Color selectColor = Color.pink;
    final Color edgeColor = Color.black;
    final Color nodeColor = new Color(250, 220, 100);
    final Color stressColor = Color.gray;
    final Color arcColor1 = Color.black;
    final Color arcColor2 = Color.pink;
    final Color arcColor3 = Color.red;

    public void paintNode(Graphics g, Node n, FontMetrics fm) {
	int x = (int)n.x;
	int y = (int)n.y;
        if (n == pick)
          g.setColor(selectColor);
        else
          g.setColor(n.color);
/*
        else if (n.fixed)
          g.setColor(fixedColor);
*/

	int w = fm.stringWidth(n.lbl) + 10;
	int h = fm.getHeight() + 4;
	g.fillRect(x - w/2, y - h / 2, w, h);
	g.setColor(Color.black);
	g.drawRect(x - w/2, y - h / 2, w-1, h-1);
        if (n.color == Color.black)
          g.setColor(Color.white);

	g.drawString(n.lbl, x - (w-10)/2, (y - (h-4)/2) + fm.getAscent());
    }


    public void paint(Graphics g) {
		super.paint(g);
      	debug("paint");
    	update(g);
    }

    public synchronized void update(Graphics g) {
      debug("update");
	Dimension d = size();
	if ((offscreen == null) || (d.width != offscreensize.width) || 
	  (d.height != offscreensize.height)) {
	  debug("new Offscreen image");
	    offscreen = createImage(d.width, d.height);
	    offscreensize = d;
	    offgraphics = offscreen.getGraphics();
	    offgraphics.setFont(getFont());
	}

	offgraphics.setColor(getBackground());
	offgraphics.fillRect(0, 0, d.width, d.height);


//***********************************************************************************

	
	for (int i = 0 ; i < nedges ; i++) {
	    Edge e = edges[i];

	
	    int x1 = (int)nodes[e.from].x;
	    int y1 = (int)nodes[e.from].y;
	    int x2 = (int)nodes[e.to].x;
	    int y2 = (int)nodes[e.to].y;
	    int len = (int)Math.abs(Math.sqrt((x1-x2)*(x1-x2) + (y1-y2)*(y1-y2)) - e.len);
	    offgraphics.setColor((len < 10) ? arcColor1 : (len < 20 ? arcColor2 : arcColor3)) ;
	    offgraphics.drawLine(x1, y1, x2, y2);

/*
	    //System.out.println(nodeSeconds);
	    
	    Date dd = new Date();
	    long newnodeSeconds = dd.getTime();
	    //System.out.println(newnodeSeconds);

	    // times for ~8 seconds then locks nodes in place
	    if ( (newnodeSeconds > (nodeSeconds + 8000)) && !secDone ){
	      for (int ii = 0 ; ii < nnodes ; ii++) {
	    	Node n = nodes[ii];
		n.fixed = true;
		secDone = true;
	      }
	    }
*/

	    if (stress) {
	     for (int cc = 0; cc < nedges; cc++){  
		String lbl = String.valueOf(e.count);
		offgraphics.setColor(stressColor);
		offgraphics.drawString(lbl, x1 + (x2-x1)/2, y1 + (y2-y1)/2);
		offgraphics.setColor(edgeColor);
	     }
	   
	    }
	}
	
//***********************************************************************************


	FontMetrics fm = offgraphics.getFontMetrics();
	for (int i = 0 ; i < nnodes ; i++) {
	    paintNode(offgraphics, nodes[i], fm);
	}

	g.drawImage(offscreen, 0, 0, null);
    }

    synchronized void changeNodeColors(String fld, Color color) {
	for (int i = 0; i < nnodes; i++) {
	    if (nodes[i].field.equals(fld)) 
		nodes[i].color = color;
	}
	repaint();
    }

    public synchronized boolean mouseDown(Event evt, int x, int y) {
	double bestdist = Double.MAX_VALUE;
	for (int i = 0 ; i < nnodes ; i++) {
	    Node n = nodes[i];
	    double dist = (n.x - x) * (n.x - x) + (n.y - y) * (n.y - y);
	    if (dist < bestdist) {
		pick = n;
		bestdist = dist;
	    }
	}
	if (pick != null) {
	  pickfixed = pick.fixed;
	  pick.fixed = true;
	  pick.x = x;
	  pick.y = y;
	  repaint();
	}
	return true;
    }

    public synchronized boolean mouseDrag(Event evt, int x, int y) {
      if (pick != null) {
	pick.x = x;
	pick.y = y;
	repaint();
      }
	return true;
    }

    public synchronized boolean mouseUp(Event evt, int x, int y) {
      if (pick != null) {
	pick.x = x;
	pick.y = y;
	pick.fixed = pickfixed;
	
	repaint();
      }
	pick = null;

      return true;
    }

    public void start() {
	relaxer = new Thread(this);
	try {
	  relaxer.setPriority(relaxer.getPriority()-1);
	} catch (SecurityException sec) {
	  // do nothing
	}
	relaxer.start();
    }
    public void stop() {
      if (relaxer != null)
	relaxer.stop();
    }

  private static final boolean debug = false;
  private void debug(String message){
    if (debug)
      System.err.println(getClass().getName()+": "+message);
  }
}



//**********************************************************************
// this originally extended TableViewApplet. Changed to extend 
// TableViewJComponent, 1-15-98. All other methods same (lwjones)
//********************************************************************** 
//-------------------------------------------------------------
public class Graph extends TableViewJComponent implements TableView, 
    TableViewListener {
    GraphPanel panel;
    TableView	source = null;
      //    public RadarPane radarPane = null;

    public Graph() {
	setLayout(new BorderLayout());
	setName("Transition Graph");

	//	radarPane = new RadarPane(RadarPane.SCROLLBARS_AS_NEEDED);
	panel = new GraphPanel(this);
	panel.resize(1000, 1000);
	//	radarPane.add(panel);
	//	add("Center", radarPane);
	add("Center", panel);
	Dimension d = getPreferredSize();
	//	radarPane.resize(Math.round(d.width*3/4), d.height);
	//	add("East", radarPane.detachRadarView());

	/* 
           p.add(new Button("Scramble"));
	   p.add(new Button("Shake"));
	   p.add(new Checkbox("Random"));
         */

    }

/*    public void init() {
	String location = getParameter("source");
	try {
	    TableView source = new sieve.sluice.weblog.TableImpl(location);
System.out.println("Got source: " + source + " " + source.getRecordCount());
	    addSourceView(source);
	}
	catch(Exception e) {
	    e.printStackTrace();
	}
    } */

    public void addSourceView(TableView source) {
	this.source = source;
	reset();
    }

    protected void reset() {

	panel.reset();

	if (source == null)
	    return;

	String[]   fields = {"Document", "Referer"};
	Object[][] data = null;
        Color fromColor = null;
        Color toColor = null;

	try {
	  String[] allFields = source.getFieldNames();
	  if (allFields.length < 2)
	    return;  // Not enough to build a graph

          LabelView lv = new LabelView(source);
          data = lv.get(allFields, 0);
          for (int fromFld = 0; fromFld < allFields.length; fromFld++) {
            fromColor = source.getFieldDescriptor(
              allFields[fromFld]).getColor();
            for (int toFld = fromFld + 1; toFld < allFields.length; toFld++) {
                toColor = source.getFieldDescriptor(
                  allFields[toFld]).getColor();
	        for (int rec = 0; rec < data[0].length; rec++) {
	          panel.addEdge(data[fromFld][rec].toString(), fromColor, 
                    allFields[fromFld], data[toFld][rec].toString(), 
		    toColor, allFields[toFld], 100);
                }
            }
          }
        }
	catch (Exception e) {
          // e.printStackTrace();
        }

	Dimension d = size();

	panel.start();
	repaint();
    }

    public void removeSourceView(TableView sourceView) {
	this.source = null;
	reset();
    }

    public void tableViewChanged(TableViewEvent event) {
	if (event instanceof FieldDescriptorChangeEvent) {
	    try {
		String fld = ((FieldDescriptorChangeEvent)event).getField();
	        panel.changeNodeColors(fld, 
		    source.getFieldDescriptor(fld).getColor());
	    }
	    catch (InterchangeException e) {
		e.printStackTrace();
	    }
	}
	else
	    reset();
	super.tableViewChanged(event);
    }

    public void start() {
	panel.start();
    }
    public void stop() {
	panel.stop();
    }

/*
    public boolean action(Event evt, Object arg) {
      // radarPane.setClientChanged(true);
      if (arg instanceof Boolean) {
	    if (((Checkbox)evt.target).getLabel().equals("Accesses")) {
		panel.stress = ((Boolean)arg).booleanValue();
	    } else {
		panel.random = ((Boolean)arg).booleanValue();
	    }
	    return true;
	}
	return false;
    }
*/

    public void setShowWeights(boolean show) {
	panel.stress = show;
    }

    public boolean getShowWeights() {
	return panel.stress;
    }

  public void update(Graphics g) {
    debug("update");
    paint(g);
  }

  public void paint(Graphics g) {
    debug("paint");
    super.paint(g);
  }

    public Dimension getPreferredSize() {
        Dimension size = getSize();
        if (size.width == 0) {
            size = new Dimension(500, 500);
        }
        return size;
    }   

    public Dimension preferredSize() {
        return getPreferredSize();
    }
    public Dimension getMinimumSize() {
        return getPreferredSize();
    }
    public Dimension minimumSize() {
        return getPreferredSize();
    }

  private static final boolean debug = false;
  private void debug(String message){
    if (debug)
      System.err.println(getClass().getName()+": "+message);
  }
}



























