package sieve.nuggets.LineGraph;

import java.awt.*;
import java.util.*;
import sieve.sluice.*;
import sieve.sluice.views.*;
import sieve.sluice.event.*;
import sieve.nuggets.*;


public class LineGraph extends TableViewApplet implements TableView, 
    TableViewListener {


    //graph class that contains many methods, drag box zoom,
    //manually setting the range, etc.
    private G2Dint               graph;

  
    //x axis of graph
    private Axis                 xaxis;

    //y axis of graph
    private Axis                 yaxis;


    public LineGraph() {
      setLayout(new BorderLayout());
    }


    private Hashtable dataSets = new Hashtable();
    private String[] fieldNames = null;
    private String indepVar = null;
    private int indepVarIndex = -1;

    private void reset() {
	if (source == null)
	    return;

        // get source information
    	int numRecords = 0;
    	Object[][] tableArray = null;
    	try {
	  //System.out.println("Source is: " + source);
	    NumberView numericFieldsOnly = new NumberView(source);
      	    fieldNames = numericFieldsOnly.getFieldNames();

	    if (fieldNames.length < 1)
		return;

	    //System.out.println(" Field 0: " + fieldNames[0]);
            numRecords = (int)source.getRecordCount();
	    //System.out.println(" records: " + numRecords);
            tableArray = source.get(fieldNames, 0);
      	    TableDescriptor td = source.getTableDescriptor();
      	    indepVar = td.getIndependentVariableField();
            if (indepVar != null)
	        for (int i = 0; i<fieldNames.length; i++) 
	  	    if (indepVar.equals(fieldNames[i]))
	    	    	indepVarIndex = i;

	    if (indepVarIndex < 0) 
		indepVarIndex = 0;
	    if (indepVar == null)
		indepVar = fieldNames[0];
		     
    	}
	catch(Exception e) {
      	    // no data in source
      	    if (graph != null)
		remove(graph);
      	    graph = null;
      	    if (debug)
		e.printStackTrace();
      	    return;
    	}

    	if (numRecords == 0)
      	     return;

         // setup graph
    	if (graph != null)
      	    remove(graph);
        graph = new G2Dint();
    	graphAttrib(graph);


    	// read the data
    	for (int fieldIndex = 0; fieldIndex<fieldNames.length; fieldIndex++) {
      	    double data[] = new double[2*numRecords];

      	    int i, j;
      	    for( i = j = 0; i < numRecords; i++, j+=2) {
		// independent
		data[j] = ((Number)tableArray[indepVarIndex][i]).doubleValue();
	        // dependent
	        data[j+1] = ((Number)tableArray[fieldIndex][i]).doubleValue();
      	    }

      	    DataSet datain = graph.loadDataSet(data, numRecords);
      	    datain.linestyle = 1;
	    try {
		datain.linecolor = 
	           source.getFieldDescriptor(fieldNames[fieldIndex]).getColor();
/*
      	    switch (fieldIndex) {
      	    case 1: 
		datain.linecolor = Color.red;
		break;
      	    case 2: 
		datain.linecolor = Color.green;
		break;
      	    case 3: 
		datain.linecolor = Color.blue;
		break;
      	    case 0: 
		datain.linecolor = Color.gray;
		break;
      	    default: 
	 	datain.linecolor = Color.cyan;
      	    }
*/
	    }
	    catch (InterchangeException e) {
		e.printStackTrace();
	    }
      	    datain.marker    = 1;
      	    datain.markerscale = 1.5;
      	    datain.markercolor = new Color(0, 0, 255);

      	    dataSets.put(fieldNames[fieldIndex], datain);
        }

        // setup the axes
        xaxis = graph.createXAxis();  //creates X axis at bottom of graph
        xaxis.setTitleText(indepVar);             //label of x axis
        xaxis.setTitleColor(Color.blue);         //text label color
        xaxis.setLabelColor(new Color(170, 0, 255)); //marker label color
	  
        yaxis = graph.createYAxis();   //creates Y axis at the left of the graph
        //    yaxis.setTitleText(fieldNames[1]+" & "+fieldNames[2]);
        yaxis.setTitleColor(Color.red); 
        yaxis.setLabelColor(Color.black);
        DataSet independentDataSet = (DataSet)dataSets.get(indepVar);
        graph.hideDataSet(independentDataSet);

        xaxis.attachDataSet(independentDataSet); //attach appropriate data to x and y
        for (Enumeration e = dataSets.elements(); e.hasMoreElements(); ) {
            DataSet tempDataSet = (DataSet)e.nextElement();
      	    if (tempDataSet != independentDataSet)
	   	yaxis.attachDataSet(tempDataSet);
    	}


    	add("Center", graph);
    	validate();
    	graph.repaint();
    }

    int debugCounter = 0;

    private void appendRecords(Object[][] records) {
        if (graph == null) {
      	    reset();
      	    return;
    	}

    	int numRecords = records[0].length;
	//	System.out.println(" ----  numRecords= "+numRecords+" records[0][0]="+records[0][0]);

	  if ((numRecords > 0) && !(records[0][0] instanceof Number))
	    return; // don't know how to handle records other than Number

    	for (int fieldIndex = 0; fieldIndex < fieldNames.length; fieldIndex++) {
      	    double data[] = new double[2 * numRecords];

	int i, j;
      	for( i = j = 0; i<numRecords; i++, j+=2) {
	    // independent
	    data[j] = ((Number)records[indepVarIndex][i]).doubleValue();
	    // dependent
	    data[j+1] = ((Number)records[fieldIndex][i]).doubleValue();
      	}

      	DataSet dataSet = (DataSet)dataSets.get(fieldNames[fieldIndex]);
      	try {
	    if (dataSet != null)
	  	dataSet.append(data, numRecords);
      	} 
	catch (Exception e) {
	    if (debug) {
		e.printStackTrace();
	    }
        }
    }
    graph.repaint();
  }




    /***** 
  ** sets up the graph attributes
  *****/
  private void graphAttrib(G2Dint gph) {
    gph.borderTop    = 30;
    gph.borderBottom = 10;
    gph.borderRight  = 40;
    gph.setDataBackground(Color.white);
    gph.setGraphBackground(Color.lightGray);
  }
 

    public void addSourceView(TableView source) throws TooManySourcesException{
	if (this.source != null)
	    throw new TooManySourcesException();
	this.source = source;
    	reset();
    }
  
    public void removeSourceView(TableView sourceView) {
	if (this.source == sourceView) {
	    this.source = null;
	    if (graph != null) {
      	        remove(graph);
      	    	graph = null;
     	    }
	}
    }

  public void update(Graphics g) {
    debug("update");
    paint(g);
  }


  //------------ Size Routines --------------
  public Dimension getPreferredSize() {
    Dimension size = size();
    if (size.width == 0) {
      size = new Dimension(300, 300);
    }
    return size;
  }   

  public Dimension preferredSize() {
    return getPreferredSize();
  }
  public Dimension getMinimumSize() {
    return getPreferredSize();
  }
  public Dimension minimumSize() {
    return getPreferredSize();
  }


  //------------- TableViewListener interface -------------------
    public void tableViewChanged(TableViewEvent event) {
        if (event instanceof AppendEvent)
	    appendRecords(((AppendEvent)event).getAppendedRecords());
      	else
	     reset();
      	eventHandler.fireTableViewEvent(event);
    }

  private static final boolean debug = false;
  private void debug(String message){
    if (debug)
      System.err.println(getClass().getName()+": "+message);
  }
}

