package sieve.nuggets.LineGraph;

import java.util.*;
import sieve.sluice.*;
import sieve.sluice.event.*;


public class SensorTable implements TableView, Runnable {
  private Thread life = null;

  private static int numFields = 5;
  TableDescriptor tableDescriptor = null;
  String fieldNames[] = new String[numFields];
  Hashtable fieldDescriptors = new Hashtable();
  TableViewEventSupport	eventHandler	= null;

  Vector records = new Vector();


  public SensorTable(String name) {
    this.eventHandler = new TableViewEventSupport();

    // Setup the table
    tableDescriptor = new TableDescriptor();
    tableDescriptor.setLabelField(name);
    tableDescriptor.setIndependentVariableField("Time");

    fieldNames[0] = "Time";
    fieldNames[1] = "Sensor1";
    fieldNames[2] = "Sensor2";
    fieldNames[3] = "Sensor3";
    fieldNames[4] = "Sensor4";

    fieldDescriptors.put("Time",  new FieldDescriptor("Time"));
    fieldDescriptors.put("Sensor1",  new FieldDescriptor("Sensor1"));
    fieldDescriptors.put("Sensor2",  new FieldDescriptor("Sensor2"));
    fieldDescriptors.put("Sensor3",  new FieldDescriptor("Sensor3"));
    fieldDescriptors.put("Sensor4",  new FieldDescriptor("Sensor4"));
  }

  //-------- Begin TableView interface --------------------------
  public TableView table() throws NoSourceException {
    return this;
  }

    /**
     * Returns the TableDescriptor for this view's representation of
     * the table.
     */
  public TableDescriptor getTableDescriptor() throws NoSourceException {
    return tableDescriptor;
  }

    /**
     * Returns the names of fields in this view. These names should be
     * used to reference fields in the <CODE>get</CODE> methods.
     */
  public String[] getFieldNames() throws NoSourceException {
    return fieldNames;
  }

    /**
     * Returns the FieldDescriptor for the specified field.
     */
    public FieldDescriptor getFieldDescriptor(String field) 
      throws InterchangeException {
	return (FieldDescriptor)fieldDescriptors.get(field);
    }

    /**
     * Returns number of records in the view.
     */
  public int getRecordCount() throws NoSourceException {
    return records.size();
  }

    /**
     * Returns no more than <CODE>numberOfRecords</CODE> records starting
     * at record number <CODE>startingAt</CODE> from 
     * the fields specified in <CODE>fields</CODE>. The resulting array
     * is ordered by <CODE>[field][record]</CODE>.
     */
    public Object[][] get(String[] fields, 
			  int startingAt, 
			  int numberOfRecords) 
      throws InterchangeException {

	debug("get: startingAt="+startingAt+" numberOfRecords="+numberOfRecords);

	Object returnRecords[][] = new Object[(int)numberOfRecords][fields.length];

	int fieldIndeces[] = new int[fields.length];

	for (int i = 0; i<fields.length; i++) {
	  for (int j = 0; j<fieldNames.length; j++) {
	    if (fieldNames[j].equals(fields[i])) {
		fieldIndeces[i] = j;
		break;
	    }
	  }
	}

	// see if we have  the requested records
	if ((records.size()) < (startingAt+numberOfRecords))
	  throw new InterchangeException();

	for (int i = 0; i<numberOfRecords; i++) {
	  Object[] oneRecord = (Object[])records.elementAt((int)startingAt+i);
	  for (int j = 0; j<fieldIndeces.length; j++ ) {
	    returnRecords[i][j] = oneRecord[fieldIndeces[j]];
	  }
	}
	return returnRecords;
    }

    /**
     * Returns all records starting from record number <CODE>startingAt</CODE>
     * from the fields specified in <CODE>fields</CODE>. (I still don't
     * like this because it doesn't add anything. _Every_ implementation
     * will almost certainly just have a call to the three-parm get...)
     */
  public Object[][] get(String[] fields, int startingAt) 
    throws InterchangeException {
      return get(fields, startingAt, records.size()-startingAt);
  }

    /* -------------  Event-related methods ------------- */
    public void addTableViewListener(TableViewListener listener) {
      debug("addTableViewListener "+listener);
	eventHandler.addTableViewListener(listener);
    }

    public void removeTableViewListener(TableViewListener listener) {
	eventHandler.removeTableViewListener(listener);
    }
    public void tableViewChanged(TableViewEvent event) {
      debug("Got tableViewChanged in : " + this);
	eventHandler.fireTableViewEvent(event);
    }

    /* -------------  Methods for link support ------------- */
  public void addSourceView(TableView source) throws TooManySourcesException {
    // no sources allowed for this table
    throw new TooManySourcesException();
  }
  public void removeSourceView(TableView source) {
  }

  //-------- End TableView interface --------------------------




  int time = 0;
  int distance = 0;
  int velocity = 0;
  int distanceIncrement = 3;
  int velocityIncrement = 1;
  
  public void addBogusRecord() {
    Object newRecord[] = new Object[numFields];
    newRecord[0] = new Integer(time++);
    newRecord[1] = new Integer(distance += distanceIncrement);
    newRecord[2] = new Integer(velocity += velocityIncrement);
    newRecord[3] = new Integer(velocity/2);
    newRecord[4] = new Integer(distance/4);
    
    //    newRecord[4] = new Integer((int)Math.round(Math.random()*128));
    velocityIncrement++;

    appendRecord(newRecord);
  }

  public void appendRecord(Object[] record) {
    records.addElement(record);

    //debug("Appended Record: "+record[0]+", "+record[1]+", "+record[2]+
//	  ", "+record[3]+", "+record[4]);
    Object newRecords[][] = new Object[1][];
    newRecords[0] = record;
    eventHandler.fireTableViewEvent(new AppendEvent(this, newRecords));
  }

  // Runnable
  private boolean pleaseRun = false;
  public void run() {
    while (pleaseRun) {
      addBogusRecord();
      try {
	life.sleep(100);
      } catch (InterruptedException ex) {
	ex.printStackTrace();
      }
    }
  }

  public void start() {
    if (life == null)
      life = new Thread(this);
    pleaseRun = true;
    life.start();
  }

  public void stop() {
    pleaseRun = false;
    life.stop();
    life = null;
  }



  private static final boolean debug = false;
  private void debug(String message){
    if (debug)
      System.err.println(getClass().getName()+": "+message);
  }
}
