// Modified version by Christopher Telfer  :: ctelfer@vt.edu 
// Now it is an official changed version.  4/7/98

package sieve.nuggets;

import sieve.sluice.*;
import sieve.sluice.views.*;
import sieve.nuggets.kit.*;
import sieve.util.Arrow;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.*;
import java.beans.*;

//**********************************************************************
// this originally extended TableViewContainer. Changed to extend 
// TableViewJComponent, 1-15-98. All other methods same (lwjones)
//********************************************************************** 

public class NewBarChart extends TableViewJComponent implements TableView {
    private Vector bars = null;
    protected PropertyChangeSupport changes = null;

    private LabeledVerticalAxis vaxis = null;
    private LinearDataDistributor vdist = null;

    private AbbreviatedHorizontalAxis haxis = null;
    private DataDistributor hdist = null;

    private String labelField = null;
    private String freqField = null;

    private Listener listener = null;
    private VerticalBar selectedBar = null;

    // value->label map for horizontal axis
    private Hashtable labelMap = null;

    // If true, the bar under the mouse will be highlighted
    private boolean selectedBarHighlighted = true;
    // The color that the selected bar will be highlighted in
    private Color selectedBarColor = new Color(205, 175, 175);

    // The number of tick marks shown on the vertical axis
    private int numberOfTicks = 0;
    private int tickIncrement = 0;

    // True if mean should be shown
    private boolean showMean = false;
    // True if the data level permits a mean
    private boolean okToShowMean = true;
    private double mean = 0.0;
    private NumberFormat numberFormat = null;

    // True if median should be shown
    private boolean showMedian = false;

    // True if the data level permits a median
    private boolean okToShowMedian = true;
    private Object median = null;
    private double medianOffset = 0.0;

   
    // True if mode should be shown
    private boolean showMode = false;
    private Vector modes = null;
    private int maxFrequency = 0;


    // True if title is defined by the user
    private boolean userTitle = false; 
    private String  titleString = null; 

    // For buffering...
    private Image offScreen = null;

    public NewBarChart() { 
	changes = new PropertyChangeSupport(this);
	numberFormat = NumberFormat.getInstance();
	numberFormat.setMinimumFractionDigits(3);
	numberFormat.setMaximumFractionDigits(3);
    }

    // -------------------- Beans stuff --------------------
    public void addPropertyChangeListener(PropertyChangeListener l) {
	changes.addPropertyChangeListener(l);
    }
    public void removePropertyChangeListener(PropertyChangeListener l) {
	changes.removePropertyChangeListener(l);
    }

    /**
     * If set to true, the bar under the mouse pointer will be highlighted.
     */
    public void setSelectedBarHighlighted(boolean selectedBarHighlighted) {
	if (selectedBarHighlighted != this.selectedBarHighlighted) {
	    this.selectedBarHighlighted = selectedBarHighlighted;
	    repaint();
	    changes.firePropertyChange("selectedBarHighlighted",
		new Boolean(!selectedBarHighlighted),
		new Boolean(selectedBarHighlighted));
	}
    }

    /**
     * If true, the bar under the mouse pointer will be highlighted.
     */
    public boolean isSelectedBarHighlighted() {
	return selectedBarHighlighted;
    }

    /**
     * If set to true the mean will be indicated for continuous data.
     */
    public void setShowMean(boolean showMean) {
	if (showMean != this.showMean) {
	    this.showMean = showMean;
	    repaint();
	    changes.firePropertyChange("showMean", new Boolean(!showMean),
		new Boolean(showMean));
	}
    }

    /**
     * If set to true the mean will be indicated for continuous data.
     */
    public boolean getShowMean() {
	return showMean;
    }

    /**
     * If set to true the median will be indicated for ordered data.
     */
    public void setShowMedian(boolean showMedian) {
	if (showMedian != this.showMedian) {
	    this.showMedian = showMedian;
	    repaint();
	    changes.firePropertyChange("showMedian", new Boolean(!showMedian),
		new Boolean(showMedian));
	}
    }

    /**
     * If set to true the median will be indicated for ordered data.
     */
    public boolean getShowMedian() {
	return showMedian;
    }

    /**
     * If set to true the mode(s) will be indicated.
     */
    public void setShowMode(boolean showMode) {
	if (showMode != this.showMode) {
	    this.showMode = showMode;
	    repaint();
	    changes.firePropertyChange("showMode", new Boolean(!showMode),
		new Boolean(showMode));
	}
    }

    /**
     * If set to true the mode(s) will be indicated.
     */
    public boolean getShowMode() {
	return showMode;
    }


    /**
     * Sets the color that the bar under the mouse pointer will be
     * highlighted in.
     * @see #isSelectedBarHighlighted
     */
    public void setSelectedBarColor(Color c) {
	if ((c != null) && !(c.equals(selectedBarColor))) {
	    Color oldColor = selectedBarColor;
	    this.selectedBarColor = c;
	    repaint();
	    changes.firePropertyChange("selectedBarColor", oldColor, c);
	}
    }

    /**
     * Returns the color that the bar under the mouse pointer will be
     * highlighted in.
     * @see #isSelectedBarHighlighted
     */
    public Color getSelectedBarColor() {
	return selectedBarColor;
    }

    /**
     * Returns the number of tick marks shown on the vertical axis.
     * @see #getTickIncrement
     */
    public int getNumberOfTicks() {
	return numberOfTicks;
    }
  
    /**
     * Sets the number of tick marks shown on the vertical axis.
     * @see #getTickIncrement
     */
    public void setNumberOfTicks(int numberOfTicks) {
	if (numberOfTicks != this.numberOfTicks) {
	    int oldNumber = this.numberOfTicks;
	    this.numberOfTicks = numberOfTicks;
	    reset();
	    changes.firePropertyChange("numberOfTicks", 
		new Integer(oldNumber), new Integer(numberOfTicks));
	}
    }

    /**
     * Returns the increment at which tick marks are shown.
     * If both this and numberOfTicks are zero, then no tick marks will
     * be shown.
     * If tickIncrement is zero and numberOfTicks is greater than zero, 
     * then the range will be divided by numberOfTicks that many tick marks 
     * will be shown.
     * If tickIncrement is greater than zero and numberOfTicks is zero,
     * then as many tick marks as will fit will be shown at multiples of 
     * tickIncrement.
     * If both are greater than zero, then numberOfTicks tick marks will
     * be shown at tickIncrement.
     */
    public int getTickIncrement() {
	return tickIncrement;
    }
  
    /**
     * Sets the number of tick marks shown on the vertical axis.
     */
    public void setTickIncrement(int tickIncrement) {
	if (tickIncrement != this.tickIncrement) {
	    int oldNumber = this.tickIncrement;
	    this.tickIncrement = tickIncrement;
	    reset();
	    changes.firePropertyChange("tickIncrement", 
		new Integer(oldNumber), new Integer(tickIncrement));
	}
    }


    // -------------------- TableView stuff --------------------

    public void addSourceView(TableView source) throws TooManySourcesException {
	if (source == this)
	    throw new TooManySourcesException("Duplicate Source");
	else if (this.source != null)
            throw new TooManySourcesException();
	this.source = source;
	try {
	    if (source.getFieldNames().length > 1) 
                setFields(source.getFieldNames()[0], source.getFieldNames()[1]);
	}
	catch (NoSourceException e) {
            // WHAT TO DO HERE?
            System.out.println("Barchart not drawn: No Source");
        }
    }

    public void removeSourceView(TableView source) {
        if (this.source == source) {
            this.source = null;
            repaint();
            this.tableViewChanged(new TableViewEvent(this));
        }
    }

    private void setupCategoricalAxis(FieldDescriptor fd, Object[] labels) {
	CategoricalDataDistributor cdd = new CategoricalDataDistributor();
	cdd.setCategories(labels);
	hdist = cdd;
	haxis = new AbbreviatedHorizontalAxis(this, hdist);
	labelMap = fd.getLabels();
	haxis.setLabelMap(labelMap);
	haxis.setLabelObjects(labels);
    }

    private void setupContinuousAxis(DescriptiveView dv) 
	throws InterchangeException {
	labelMap = null; // No labels for continuous data
	LinearDataDistributor ldd = new LinearDataDistributor();
	int min = 0;
	int max = ((Number)dv.getMax()).intValue();
	ldd.setMin(min);
	ldd.setMax(max);
	ldd.setConstrainedToInt(true);
	hdist = ldd;
	haxis = new AbbreviatedHorizontalAxis(this, hdist);
	Object[] labels = new Object[max - min + 1];
	for (int i = min; i <= max - min; i++) {
	    labels[i - min] = new Integer(i);
	}
	haxis.setLabelObjects(labels);
    }

    public void setFields(String labelField, String freqField) {
        this.labelField = labelField;
        this.freqField = freqField;
	reset();
    }

    public void reset() {
	if (listener != null)
	    removeMouseMotionListener(listener);

	Object[] labels = null;
	Object[] values = null;

	bars = new Vector();
	try {
	    String[] flds = {labelField, freqField};
	    Object[][] data = source.get(flds, 0);
	    labels = data[0];
	    values = data[1];

	    FieldDescriptor labelFd = source.getFieldDescriptor(labelField);
	    setName(labelFd.getDescription());
	  
	    if ((labelFd.getLevel() == FieldDescriptor.LEVEL_INTERVAL) ||
		(labelFd.getLevel() == FieldDescriptor.LEVEL_RATIO)) {
		setupContinuousAxis(new DescriptiveView(source, labelField));
		okToShowMean = true;
		okToShowMedian = true;
	    }
	    else {
	        setupCategoricalAxis(labelFd, labels);
		okToShowMean = false;
		okToShowMedian = 
		    (labelFd.getLevel() == FieldDescriptor.LEVEL_ORDINAL);
	    }

	    listener = new Listener(haxis, this);
	    addMouseMotionListener(listener);

	    DescriptiveView dv = new DescriptiveView(source, freqField);
	    vdist = new LinearDataDistributor();
	    vdist.setMin(0); // Or use dv.min();
	    double dataMax = ((Number)dv.getMax()).doubleValue();
	    if (tickIncrement > 0 && numberOfTicks > 0)
	        vdist.setMax(Math.max(dataMax, tickIncrement * numberOfTicks));
	    else if (tickIncrement > 0 && numberOfTicks <= 0) 
	        vdist.setMax(Math.max(dataMax, 
		    tickIncrement * (((int)dataMax)/tickIncrement + 1)));
	    else 
	        vdist.setMax(dataMax);
	    vdist.setConstrainedToInt(false);

	    vaxis = new LabeledVerticalAxis(this, vdist);
	    Integer[] vlabels = null;
	    if ((numberOfTicks > 0) && (tickIncrement <= 0)) {
		if (numberOfTicks == 1) {
		    Integer[] tmp = { new 
			Integer(((Number)dv.getMax()).intValue()) };
		    vlabels = tmp;
		}
		else if (numberOfTicks == 2) {
		    Integer[] tmp = { new Integer(0),
			new Integer(((Number)dv.getMax()).intValue()) };
		    vlabels = tmp;
		}
		else {
		    vlabels = new Integer[numberOfTicks];
		    vlabels[0] = new Integer(0);
		    vlabels[numberOfTicks - 1] = 
			new Integer(((Number)dv.getMax()).intValue());
		    int step = 
			((Number)dv.getMax()).intValue()/(numberOfTicks - 1);
		    for (int i = 1; i < numberOfTicks - 1; i++) {
			vlabels[i] = new Integer(step * i);
		    }
		}
		vaxis.setLabelObjects(vlabels);
	    }
	    // Show a tick every tickIncrement, with as many ticks as possible
	    else if ((numberOfTicks <= 0) && (tickIncrement > 0)) {
		int tmpTicks = (((int)dataMax)/tickIncrement) + 1;
		vlabels = new Integer[tmpTicks + 1];
		for (int t = 0; t <= tmpTicks; t++)
		    vlabels[t] = new Integer(t * tickIncrement); 
		vaxis.setLabelObjects(vlabels);
	    }
	    else if ((numberOfTicks > 0) && (tickIncrement > 0)) {
		vlabels = new Integer[numberOfTicks];
		for (int t = 0; t < numberOfTicks; t++)
		    vlabels[t] = new Integer(t * tickIncrement); 
		vaxis.setLabelObjects(vlabels);
            }

	    double weightedSum = 0.0;
	    int totalObservations = 0;
	    maxFrequency = 0;
	    modes = new Vector();
	    for (int i = 0; i < labels.length; i++) {
		VerticalBar b = new VerticalBar(labels[i], values[i]);
		b.setVerticalAxis(vaxis);
		b.setHorizontalAxis(haxis);
		b.setFilled(true);
		b.setBarColor(Color.white);
		bars.addElement(b);
		int observations = ((Number)values[i]).intValue();
		if (okToShowMean) 
		    weightedSum += ((Number)labels[i]).doubleValue() *
			observations;
		totalObservations += observations;
		if (observations > maxFrequency) {
		    maxFrequency = observations;
	    	    modes = new Vector();
		    modes.addElement(labels[i]);
		}
		else if (observations == maxFrequency)
		    modes.addElement(labels[i]);
	    }
	    mean = weightedSum/(double)totalObservations;
	    median = null;
	    if (okToShowMedian) {
	        int observations = 0;
	        int medianIdx = 0;
	        while (observations <= totalObservations/2)
		    observations += ((Number)values[medianIdx++]).intValue();
	        medianIdx--;	
	        median = labels[medianIdx];
	        medianOffset = 1-((double)(observations - totalObservations/2)/
		    ((Number)values[medianIdx]).doubleValue());
	    }
	}
	catch (Exception e) {
	    e.printStackTrace();
	}
	sizeAxes();
	repaint();
    }

    private int getHAxisWidth() {
        Dimension vPref = vaxis.getPreferredSize();
	return getSize().width - 15 - vPref.width;
    }
    private int getHAxisX() {
        Dimension vPref = vaxis.getPreferredSize();
	return 10 + vPref.width;
    }
    private int getHAxisY() {
	Dimension hPref = haxis.getPreferredSize();
	FontMetrics fm = 
	    Toolkit.getDefaultToolkit().getFontMetrics(getFont());
	int fontHeight = fm.getHeight();
	int y = getSize().height - 10 - hPref.height - fontHeight;
	if (showMean && okToShowMean)
	    y -= 2 * fontHeight;
	return y;
    }

    /**
     * Returns the height of all of the stuff at the top of the chart.
     */
    private int getTopHeight(int fontHeight) {
	int topHeight = 5 + fontHeight; // For the title
	if (showMedian && okToShowMedian)
	    topHeight += fontHeight;
	if (showMode)
	    topHeight += fontHeight;
	return topHeight;
    }

    // What we really need is a LayoutManager...
    private void sizeAxes() {
	if (getFont() != null) {
	    Dimension size = getSize();
	    FontMetrics fm = 
		Toolkit.getDefaultToolkit().getFontMetrics(getFont());
	    int fontHeight = fm.getHeight();

	    Dimension hPref = haxis.getPreferredSize();
            Dimension vPref = vaxis.getPreferredSize();

            haxis.setBounds(new Rectangle(getHAxisX(), getHAxisY(),
                getHAxisWidth(), hPref.height));
            vaxis.setBounds(new Rectangle(10 + vPref.width, 
		getTopHeight(fontHeight),
                vPref.width, getHAxisY() - getTopHeight(fontHeight)));
	}
    }

    public void update(Graphics g) {
        paint(g);
    }

    public synchronized void paint(Graphics appG) {
	Dimension size = getSize();
	if ((offScreen == null) || (offScreen.getWidth(null) != size.width) ||
            (offScreen.getHeight(null) != size.height)) {
	    offScreen = createImage(size.width, size.height);
	}

	Graphics g = offScreen.getGraphics();

        g.setColor(getBackground());
	g.setClip(0, 0, size.width, size.height);
        g.fillRect(0, 0, size.width, size.height);

	if ((source != null) && (bars != null) && (bars.size() != 0)) {
            g.setColor(getForeground());
            g.setFont(getFont());
	    FontMetrics fm = g.getFontMetrics();
	    int fontHeight = fm.getHeight();

	    Dimension hPref = haxis.getPreferredSize();
            Dimension vPref = vaxis.getPreferredSize();
 
	    // Draw the horizontal and vertical axes
            haxis.setBounds(new Rectangle(getHAxisX(), getHAxisY(),
                getHAxisWidth(), hPref.height));
            haxis.paint(g);
 
            Rectangle vaxisBounds = new Rectangle(10 + vPref.width, 
		getTopHeight(fontHeight),
                vPref.width, getHAxisY() - getTopHeight(fontHeight));
            vaxis.setBounds(vaxisBounds);
            vaxis.paint(g);
 
	    // Then draw the bars (and any other pieces) on the graph
            for (Enumeration e = bars.elements(); e.hasMoreElements();)
                ((VisualizationElement)e.nextElement()).paint(g);

	    // Draw the title at the top 
            String drawnTitle = (userTitle) ? titleString : getName(); 
	    g.drawString(drawnTitle, 
		getHAxisX() + (getHAxisWidth() - fm.stringWidth(getName()))/2, 
		2 + fontHeight);
		//size.height - fontHeight + 10);

	    // Draw a marker for the mean
	    if (showMean && okToShowMean) {
		int meanX = haxis.getCentralPixel(new Double(mean));
		int topOfMeanMark = getHAxisY() + hPref.height + 1;
		g.setXORMode(Color.white);
		Arrow.paintArrow(g, 
		    new Point(meanX, topOfMeanMark + fontHeight - 1),
		    new Point(meanX, getHAxisY() + 2),
		    4, 9); // length, width
		g.setPaintMode();
/*
		g.drawLine(meanX, topOfMeanMark, meanX, 
		    topOfMeanMark + fontHeight - 1);
		Polygon p = new Polygon();
		p.addPoint(meanX, topOfMeanMark + fontHeight/2);
		p.addPoint(meanX - 5, topOfMeanMark + fontHeight);
		p.addPoint(meanX + 5, topOfMeanMark + fontHeight);
		p.addPoint(meanX, topOfMeanMark + fontHeight/2);
		g.fillPolygon(p);
*/
		String meanString = new String("Mean = " + 
		    numberFormat.format(mean));
		g.drawString(meanString, meanX - fm.stringWidth(meanString)/2,
		    topOfMeanMark + 2 * fontHeight + 1);
	    }

	    // Draw markers for the modes
	    if (showMode && (modes != null) && (modes.size() > 0)) {
		int modeY = vaxis.getCentralPixel(new Integer(maxFrequency));

		int modeLabelX = haxis.getCentralPixel(
			modes.elementAt(modes.size()/2));

		// If there are an even number, put the label between the
		// two middle-most modes.
		if (modes.size() % 2 == 0) {
		    modeLabelX += haxis.getCentralPixel(
			modes.elementAt(modes.size()/2 - 1));
		    modeLabelX = modeLabelX/2;
		}

		for (Enumeration e = modes.elements(); e.hasMoreElements();) {
		    int modeX = haxis.getCentralPixel(e.nextElement());
		    Arrow.paintArrow(g, new Point(modeLabelX, 2 * fontHeight),
			new Point(modeX, modeY), 10, 7);
		}
		String modeString = null;
		if (modes.size() == 1)
		    modeString = "Mode";
		else
		    modeString = "Modes";
		g.drawString(modeString, 
		    modeLabelX - fm.stringWidth(modeString)/2, 2 * fontHeight);
	    }

	    // Draw a marker for the median
	    if (showMedian && okToShowMedian) {
		int medianSize = haxis.getMaxSize(median);
		int medianX = (int)(haxis.getCentralPixel(median) -
		    (medianSize)/2 + (medianSize * medianOffset));
		int startY = vaxis.getFirstPixel() - fontHeight;
		int endY = vaxis.getLastPixel();
		g.drawLine(medianX-1, startY+1, medianX-1, endY-1);
		g.drawLine(medianX, startY, medianX, endY);
		g.drawLine(medianX+1, startY+1, medianX+1, endY-1);

		String medianString = null;
		if ((labelMap != null) && (labelMap.get(median) != null))
		    medianString = new String(
			"Median = " + labelMap.get(median));
		else
		    medianString = new String("Median = " + median);

		int medianLabelOffset = 2;
		if (medianX + fm.stringWidth(medianString) > size.width)
		    medianLabelOffset = - 2 - fm.stringWidth(medianString);

                g.setColor(getBackground());
		g.fillRect(medianX + medianLabelOffset, 
		    getTopHeight(fontHeight) - 5 - fontHeight, 
		    fm.stringWidth(medianString), fontHeight + 2);
                g.setColor(getForeground());
		g.drawString(medianString, medianX + medianLabelOffset, 
		    getTopHeight(fontHeight) - 5);
	    }
	    // g.drawString("n", vaxisBounds.x + 1, vaxisBounds.y + fontHeight);
	}
	super.paint(g);
	appG.drawImage(offScreen, 0, 0, null);
	g.dispose();
    }

    public void tableViewChanged(TableViewEvent event) {
	if (source != null) {
	    try {
	        if (source.getFieldNames().length > 1) 
                    setFields(source.getFieldNames()[0], 
		    source.getFieldNames()[1]);
	    }
	    catch (NoSourceException e) {
                System.out.println("Barchart not drawn: No Source");
            }
	}
	super.tableViewChanged(event);
    }

    public void expandLabel(Object labelObject) {
	if (labelObject instanceof Number)
	    labelObject = new Integer(((Number)labelObject).intValue());
	if (selectedBar != null) 
	    selectedBar.setBarColor(Color.white);
	selectedBar = null;
	if (selectedBarHighlighted && (labelObject != null)) {
	    int i = 0;
	    while ((selectedBar == null) && (i < bars.size())) {
		VerticalBar tmp = (VerticalBar)bars.elementAt(i++);
		if (tmp.getDataObject().equals(labelObject)) {
		    selectedBar = tmp;
		    selectedBar.setBarColor(selectedBarColor);
		}
	    }
	}
	haxis.setExpandedLabel(labelObject);
	repaint();
    }

    /**
     * Returns the label object which corresponds to the given point, or
     * null if the point is outside of the graph.
     */
    public Object getLabel(Point p) {
	return haxis.getDataPoint(p.x);
    }

    /**
     * Returns the frequency object which corresponds to the given point, or
     * null if the point is outside of the graph.
     */
    public Object getFrequency(Point p) {
	return vaxis.getDataPoint(p.y);
    }

    private class Listener extends MouseMotionAdapter {
	private AbbreviatedHorizontalAxis haxis = null;
	private NewBarChart parent = null;
	public Listener(AbbreviatedHorizontalAxis haxis, NewBarChart parent) {
	    this.haxis = haxis;
	    this.parent = parent;
	}
	public void mouseMoved(MouseEvent e) {
	    parent.expandLabel(haxis.getDataPoint(e.getX()));
	}
    }

    /**
     * Returns the mean value.
     */
    public double getMean() {
	return mean;
    }

    /**
     * Returns the median value.
     */
    public Object getMedian() {
	return median;
    }

    /**
     * Returns the mode values.
     */
    public Vector getModes() {
	Vector copy = (Vector) modes.clone();
	return copy;
    }

    public Dimension getMinimumSize() {
	return new Dimension(150, 150);
    }

    // Yes, point is abusive, but... 
    public Point getMeanLine(double x) 
    { 

      Dimension hPref = haxis.getPreferredSize(); 
      int meanX = haxis.getCentralPixel(new Double(x));
      int topOfMeanMark = getHAxisY() + hPref.height + 1;
      return new Point(meanX, topOfMeanMark); 

    }  /* getMeanLine */


    public void setTitle(String inString) { userTitle = true; 
                                            titleString = inString; }
    public void setTitleDefault()         { userTitle = false; } 


} /* New Bar Chart */

