package sieve.nuggets;

//import java.applet.*;
import com.sun.java.swing.*;
import sieve.sluice.*;
import java.util.*;
import java.awt.*;

/**
 * 
 */
public abstract class TableViewApplet extends JApplet {
    protected TableView	source 		= null;

    protected TableViewEventSupport	eventHandler	= null;

    public TableViewApplet() {
		super();
		eventHandler = new TableViewEventSupport();
    }

    protected TableView outputView() throws NoSourceException {
	if (source == this) {
	    System.err.println("WARNING: Recursive source in: " + this);
	    throw new NoSourceException();
	}
	if (source == null)
	    throw new NoSourceException();
	return source;
    }

    public synchronized TableView table() throws NoSourceException {
	return outputView().table();
    }

    public synchronized TableDescriptor getTableDescriptor() throws
        InterchangeException {
        return outputView().getTableDescriptor();
    }

    public synchronized String[] getFieldNames() throws NoSourceException {
	return outputView().getFieldNames();
    }

    public synchronized FieldDescriptor getFieldDescriptor(String field) throws
        InterchangeException {
        return outputView().getFieldDescriptor(field);
    }

    /**
     * Returns number of records in the view.
     */
    public synchronized int getRecordCount() throws NoSourceException {
	return outputView().getRecordCount();
    }

    /**
     * Returns no more than <CODE>numberOfRecords</CODE> records starting
     * at record number <CODE>startingAt</CODE> from 
     * the fields specified in <CODE>fields</CODE>.
     */
    public synchronized Object[][] get(String[] fields, int startingAt, 
	int numberOfRecords) throws InterchangeException {
	return outputView().get(fields, startingAt, numberOfRecords);
    }

    /**
     * Returns all records starting from record number <CODE>startingAt</CODE>
     * from the fields specified in <CODE>fields</CODE>.
     */
    public synchronized Object[][] get(String[] fields, int startingAt) 
	throws InterchangeException { 
	return outputView().get(fields, startingAt); 
    } 

    /* -------------  Event-related methods ------------- */
    public synchronized void addTableViewListener(
	TableViewListener listener) {
	eventHandler.addTableViewListener(listener);
    }

    public synchronized void removeTableViewListener(
	TableViewListener listener) {
	eventHandler.removeTableViewListener(listener);
    }

    /**
     * Propagates event to all listeners with no filtering.
     */
    public void tableViewChanged(TableViewEvent event) {
	eventHandler.fireTableViewEvent(event);
    }

    public Dimension getPreferredSize() {
        Dimension size = getSize();
        if (size.width == 0) {
            size = new Dimension(200, 200);
        }
        return size;
    }
 
    public Dimension preferredSize() {
        return getPreferredSize();
    }
    public Dimension getMinimumSize() {
        return getPreferredSize();
    }
    public Dimension minimumSize() {
        return getPreferredSize();
    }

	/**
	 * TableViewApplet changed from extending Applet to extending JApplet.
	 * Here I am overriding add, remove, setLayout, and getLayout methods
	 * to pass along the methods to the contentPane to avoid modifying
	 * each TableViewApplet subclass. If TableViewApplet subclasses
	 * have quirky behavior in other cases (doLayout, getComponents, etc), 
	 * you may need to access the contentPane directly. 
	 * These changes by Lynn W. Jones, 1/14/98
	 *
	 *	@see com.sun.java.swing.JApplet
	 *
	 **/
/*	public Component add(Component c) { 
		return getContentPane().add(c);
	}

  	public Component add(Component c, int i) { 
		return getContentPane().add(c,i);
	}

  	public void add(Component c, Object constraints) { 
		getContentPane().add(c,constraints);
	}

  	public void add(Component c, Object constraints, int i) { 
		getContentPane().add(c,constraints,i);
	}

  	public Component add(String s, Component c) { 
		return getContentPane().add(s,c);
	}

	public void remove(Component comp) {
		getContentPane().remove(comp);
	}

	public void remove(int index) {
		getContentPane().remove(index);
	}

	public void removeAll() {
		getContentPane().removeAll();
	}

	public LayoutManager getLayout() {
		return getContentPane().getLayout();
	}

	public void setLayout(LayoutManager mgr) {
		getContentPane().setLayout(mgr);
	} */
}


