package sieve.nuggets.kit;

import java.awt.*;

/**
 * An Axis both paints an axis and also controls the layout of other
 * visualization elements by providing bound information for data points.
 * In the descriptions below, a pixel represents the appropriate part
 * of a point based on the orientation of the Axis: For a horizontal axis,
 * a pixel value is equivalent to an x-coordinate, and for a vertical 
 * axis a pixel value is equivalent to a y-coordinate.
 * <P>
 * <I>Does this need a setOrigin(int pixel) method as well?</I>
 */
public interface Axis {

    /**
     * Returns the first pixel on which data may appear. The representation
     * of the lowest data point on this axis must not be draw itself on
     * a pixel lower than this. Note that for horizontal axes, high and
     * low are backwards -- the first pixel would normally be at the 
     * bottom of the display, and will therefore have a higher pixel value.
     */
    public int getFirstPixel();

    /**
     * Returns the last pixel on which data may appear. The representation
     * of the highest data point on this axis must not be draw itself on
     * a pixel higher than this. Note that for horizontal axes, high and
     * low are backwards -- the last pixel would normally be at the 
     * top of the display, and will therefore have a lower pixel value.
     */
    public int getLastPixel();

    /**
     * Return the pixel in the center of the space available for representation
     * of the given datapoint. 
     */
    public int getCentralPixel(Object dataPoint);

    /**
     * Returns the object which would be mapped to the specified pixel.
     */
    public Object getDataPoint(int pixel);

    /**
     * Return the size (in pixels) available for representation of the given
     * datapoint.
     */
    public int getMaxSize(Object dataPoint);

    /**
     * Returns the preferred size of the Axis. The actual size allotted to
     * the Axis may or may not match these dimensions.
     */
    public Dimension getPreferredSize();

    /**
     * Sets the bounds in which the axis is allowed to draw itself.
     */
    public void setBounds(Rectangle bounds);
    
    /**
     * Returns the bounds in which the axis is allowed to draw itself.
     */
    public Rectangle getBounds();
    
    /**
     * Paints the Axis on the given Graphics object. The Axis must obey
     * the bounds specified by a setBounds() call when painting itself.
     */
    public void paint(Graphics g);

}

