package sieve.nuggets.kit;

/**
 * Distributes a set of categorical data on an axis. This 
 * is really simple: each data point gets the same amount of
 * space.
 */
public class CategoricalDataDistributor implements DataDistributor {
    private Object[] categories = null;
    private int size = 0;
    private int pixelsPerCategory = 0;

    public CategoricalDataDistributor() { }

    /**
     * Sets the categories to be distributed.
     */
    public void setCategories(Object[] categories) {
	this.categories = categories;
	if ((categories != null) && (categories.length != 0))
	    pixelsPerCategory = size/categories.length;
    }

    /**
     * Returns the size (in pixels) available for representation of the given
     * datapoint.
     */
    public int getMaxSize(Object dataPoint) {
	if ((size != 0) && (categories != null)) 
	    return pixelsPerCategory;
	else
	    return -Integer.MAX_VALUE;
    }

    /**
     * Return the pixel in the center of the space available for representation
     * of the given datapoint.
     */
    public int getCentralPixel(Object dataPoint) {
	if ((size != 0) && (categories != null)) {
	    // should hash these...
	    for (int i = 0; i < categories.length; i++) {
		if (categories[i].equals(dataPoint))
		    return (i * pixelsPerCategory) + (pixelsPerCategory/2);
	    }
	}
	return -Integer.MAX_VALUE;
    }

    public Object getDataPoint(int pixel) {
	int idx = pixel/pixelsPerCategory;
	if ((idx >= 0) && (idx < categories.length))
	    return categories[idx];
	return null;
    }
 
    /**
     * Sets the number of pixels over which data points may be distributed.
     */
    public void setSize(int pixels) {
	this.size = pixels;
	if ((categories != null) && (categories.length != 0))
	    pixelsPerCategory = size/categories.length;
    }

}

