package sieve.nuggets.kit;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 * A Chart Point is a VisualizationElement that draws an oval on an (x, y) 
 * point.
 */
public class ChartPoint implements VisualizationElement {
    private Object vObj = null;
    private Object hObj = null;
    private Axis vAxis = null;
    private Axis hAxis = null;

    private int maxRadius = 3;
    private boolean pointIsCircular = true;

    private Point p = null;
    private Color pointColor = Color.black;
    private Color lineColor = Color.black;

    private ChartPoint prev = null;

    public ChartPoint(Object hObj, Object vObj) {
	this.hObj = hObj;
	this.vObj = vObj;
    }

    /**
     * Returns the maximum radius of a data point.
     */
    public int getMaxRadius() {
	return maxRadius;
    }
    /**
     * Sets the maximum radius of a data point. Points will be represented
     * by ovals with width and height which are the lesser of this value
     * and the maximum allowed size returned by the appropriate axis object.
     */
    public void setMaxRadius(int maxRadius) {
	this.maxRadius = maxRadius;
    }

    /**
     * Returns the point "before" this point.
     */
    public ChartPoint getPreviousPoint() {
	return prev;
    }

    /**
     * Sets the point "before" this point.
     */
    public void setPreviousPoint(ChartPoint p) {
	prev = p;
    }

    /**
     * Returns true if the point will always be drawn as a circle.
     */
    public boolean isCircular() {
	return pointIsCircular;
    }

    /**
     * Sets whether the point will always be drawn as a circle.
     */
    public void setCircular(boolean isCircular) {
	this.pointIsCircular = isCircular;
    }

    /**
     * Returns the color in which points are drawn.
     */
    public Color getPointColor() {
	return pointColor;
    }

    /**
     * Sets the color in which points are drawn.
     */
    public void setPointColor(Color c) {
	pointColor = c;
    }

    /**
     * Returns the color in which lines are drawn.
     */
    public Color getLineColor() {
	return lineColor;
    }

    /**
     * Sets the color in which lines are drawn.
     */
    public void setLineColor(Color c) {
	lineColor = c;
    }

    // ---------------    VisualizationElement methods    ---------------
    public void setVerticalAxis(Axis a) {
	vAxis = a;
	reset();
    }
    public void setHorizontalAxis(Axis a) {
	hAxis = a;
	reset();
    }

    private void reset() {
	if ((hAxis != null) && (vAxis != null))
	    p = new Point(hAxis.getCentralPixel(hObj),
	        vAxis.getCentralPixel(vObj));
    }

    public void paint(Graphics g) {
	reset();
	if (p != null) {
	    if ((prev != null) && (prev.p != null)) {
// Need some kind of "Connectable" interface here...
		g.setColor(lineColor);
		g.drawLine(prev.p.x, prev.p.y, p.x, p.y);
		if (! lineColor.equals(prev.pointColor))
		    prev.paintPoint(g);
            }
	    paintPoint(g);
	}
    }

    private void paintPoint(Graphics g) {
	int width = Math.min(maxRadius, hAxis.getMaxSize(hObj));
	int height = Math.min(maxRadius, vAxis.getMaxSize(vObj));
	if (pointIsCircular) {
	    width = Math.min(width, height);
	    height = width;
	}
	g.setColor(pointColor);
	g.fillOval(p.x - width/2, p.y - height/2, width, height);
    }

    public boolean contains(int x, int y) {
	return (p != null) && (p.x == x) && (p.y == y);
    }

}


