package sieve.nuggets.kit;

import java.awt.*;
import java.util.*;

public class LabeledHorizontalAxis extends SimpleHorizontalAxis implements 
	Axis {
    private Object[] labelObjects = null;
    private Hashtable map = null;

    public LabeledHorizontalAxis(Component parent, 
	DataDistributor distributor) {
	super(parent, distributor);
    }

    public Object[] getLabelObjects() {
	return labelObjects;
    }
    public void setLabelObjects(Object[] labelObjects) {
	this.labelObjects = labelObjects;
    }

    public void setLabelMap(Hashtable map) {
        this.map = map;
    }

    protected String getPrintableLabel(Object labelObject) {
	if (labelObject == null)
	    return "";
	String label = null;
	if (map != null)
	    label = (String)map.get(labelObject);
	if (label == null)
	    label = labelObject.toString();
	return label;
    }

    public Dimension getPreferredSize() {
	int fontSize = 0;
	Component p = getParent();
	if (labelObjects != null) {
  	    if (p.getFont() != null) {
	        FontMetrics fm = p.getFontMetrics(p.getFont());
	        fontSize = fm.getHeight();
	    }
	    else
		fontSize = 12; 
	}
	return new Dimension(getParent().getSize().width, 5 + fontSize);
    }

    public void paint(Graphics g) {
	super.paint(g);
	Rectangle bounds = getBounds();
	if (bounds != null) {
	    FontMetrics fm = g.getFontMetrics();
	    int height = fm.getHeight();
	    int first = getFirstPixel();
	    DataDistributor dist = getDataDistributor();
	    if ((labelObjects != null) && (bounds.height >= 2 + height)) {
	        for (int i = 0; i < labelObjects.length; i++) {
		     String label = getPrintableLabel(labelObjects[i]);
		     int pos = dist.getCentralPixel(labelObjects[i]);
		     int width = fm.stringWidth(label);
		     g.drawString(label, first + pos - width/2, 
			bounds.y + 2 + height);
		}
	    }
	}
    }

}



