package sieve.nuggets.kit;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 * A VerticalBar is VisualizationElement that draws a vertical 
 * barchart/histogram-style bar.
 */ 
public class VerticalBar implements VisualizationElement {
    protected Object data = null;
    protected Object freq = null;
    protected Axis vAxis = null;
    protected Axis hAxis = null;

    protected Rectangle rect = null;
    protected Color borderColor = Color.black;
    protected Color barColor = Color.black;

    protected boolean filled = false;

    protected double scale = 1.0;

    public VerticalBar(Object data, Object freq) {
	this.data = data;
	this.freq = freq;
    }

    public Object getDataObject() {
	return data;
    }
    public Object getFrequencyObject() {
	return freq;
    }

    public boolean isFilled() {
	return filled;
    }
    public void setFilled(boolean filled) {
	this.filled = filled;
    }

    public void setScale(double scale) {
	this.scale = scale;
    }
    public double getScale() {
	return scale;
    }

    public Color getBorderColor() {
	return borderColor;
    }
    public void setBorderColor(Color borderColor) {
	this.borderColor = borderColor;
    }
    public Color getBarColor() {
	return barColor;
    }
    public void setBarColor(Color barColor) {
	this.barColor = barColor;
    }

    // ---------------    VisualizationElement methods    ---------------
    public void setVerticalAxis(Axis a) {
        vAxis = a;
    }
    public void setHorizontalAxis(Axis a) {
        hAxis = a;
    }

    public void paint(Graphics g) {
	if ((hAxis != null) && (vAxis != null)) {
	     int width = (int)(hAxis.getMaxSize(data) * scale);
	     int x = hAxis.getCentralPixel(data) - width/2;
	     int y = vAxis.getCentralPixel(freq);
	     rect = new Rectangle(x, y, width, vAxis.getLastPixel() - y);
	     if (filled) {
	         g.setColor(barColor);
	         g.fillRect(rect.x, rect.y, rect.width, rect.height);
	     }
	     g.setColor(borderColor);
	     g.drawRect(rect.x, rect.y, rect.width, rect.height);
	}
    }

    public boolean contains(int x, int y) {
        return (rect != null) && rect.contains(x, y);;
    }

}

