package sieve.nuggets.kit;
 
import java.awt.*;
import java.awt.event.*;

/**
 * A VisualizationElement is an object which displays itself on
 * a visualization. There are probably two kinds of these:
 * <OL>
 * <LI> First-order visualization elements, which typically represent
 *      a specific (x, y) data point. The position of these is 
 *      determined by the Axis objects. The VisualizationElement
 *      should query each Axis for position information appropriate
 *      for its data point or points.
 * <LI> Second-order visualization elements, which typically represent
 *      things not directly present in the data -- annotations
 *      of mean, median, and mode, for example.
 * </OL>
 */
public interface VisualizationElement {

    public void setVerticalAxis(Axis a);

    public void setHorizontalAxis(Axis a);

    public void paint(Graphics g);

    public boolean contains(int x, int y);
}

