/*
 *
 * Written By:		James "Bo" Begole
 * Date:			18 June 1996
 * Description:	Block and Plane Simulation Applet
 *
 * Usage Notes:
 *
 * 
 */ 
 
/*
 *
 * Copyright (c) 1996 James "Bo" Begole, All Rights Reserved.
 *
 * 
 */
package sieve.phys.blockandplane; 

import java.applet.Applet;
import java.awt.*;
import java.util.*;
import vt.cs.widgets.*;
import vt.cs.virtSchool.util.*;
//import vt.cs.db.*;
import vt.cs.shareablePrimitive.*;
import vt.cs.util.*;
import SharedData.*;

import sieve.sluice.*;
import sieve.sluice.event.*;
import sieve.nuggets.LineGraph.*;
 
public class BlockAndPlane extends Applet implements TableView  {
  private static boolean inAnApplet = true;  
  public SimCanvas simCanvas = null;
  //  public float t_incr = 0.10f;
  protected ShareableFloat t_incr = null;

  public  int   RunCount = 0;
  SimulationThread thread = null;
  boolean Started = false; // Has "Run" been pushed?
  public  ButtonCanvas BC = null;
  public  Plane plane1 = null;
  public  Plane plane2 = null;
  public  Plane floor = null;
  public  MovingBody MB = null;
  public  PlaneDialog Plane1Dialog = null;
  public  PlaneDialog Plane2Dialog = null;
  public  FloorDialog floorDialog = null;		
  public  MovingBodyDialog MBDialog = null;
  public  ConstantDialog CDialog = null;
  public  DataCollectDialog DataCDialog = null;
  Button Button_plane1,Button_plane2,Button_floor,
    Button_config,Button_Data_collect,Button_const,
    Button_run,Button_pause,Button_reset;
  TextField RunCounter = null; 
  TextArea OutputArea1 = null;
  TextArea OutputArea2 = null;

/*
  private DatabaseSet	dbSet   = null;
  private Database    	db  	= null;
  private vt.cs.db.Table       	table	= null;
*/

  private Buffer tableBuffer = null;
  private static final String[] columnNames = 
      {"Distance", "Velocity", "Acceleration", "Time"};
  private static final String[] columnTypes = 
      {"java.lang.Long","java.lang.Long","java.lang.Long","java.lang.Long"};
  private static final String[] columnDescs = 
      {"meters","meters/millisec","meters/millisec/millisec","millisec"};

  // Shared Data variables
  private Session session = null;
  private SimulationControl simulationControl = null;
  private ParameterControl parameterControl = null;


  public void init() {
    // Check if we are working collaboratively and initiate, if so
    createSession(getParameter("SessionURL"));

    // init canvas objects
    plane1 = new Plane(0.5f, 1.0f, 0.0f, 0.00f, 0.02f, 0, true, 
		       session, parameterControl, "Plane1");
    plane2 = new Plane(1.0f, 2.0f, 0.0f, 0.00f, 0.02f, 500, false,
		       session, parameterControl, "Plane2");
    floor  = new Plane(0f, 2.0f, 0.0f, 0.0f, 0f, 100, true,
		       session, parameterControl, "Floor");
    MB = (MovingBody)(new Block(0.15f,0.15f, this,
				session, parameterControl, "Block"));
    t_incr = new ShareableFloat("BlockAndPlaneTimeStep", 
				session, parameterControl, 0.10f);

    simCanvas = new SimCanvas(plane1, plane2, floor, MB);

    // Set Applet layout
    setLayout(new BorderLayout());
    BC = new ButtonCanvas();
    Panel ButtonCanvasPanel = new Panel();
    ButtonCanvasPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    ButtonCanvasPanel.add(BC);
    Panel PlanePanel = new Panel();
    PlanePanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    Button_plane1 = new Button("Plane 1");
    PlanePanel.add(Button_plane1);
    Button_floor = new Button("Floor");
    PlanePanel.add(Button_floor);
    Button_plane2 = new Button("Plane 2");
    PlanePanel.add(Button_plane2);
    Panel MBPanel = new Panel(); 
    MBPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    Button_config = new Button("Block");
    MBPanel.add(Button_config);
    Panel DataPanel = new Panel();
    DataPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    Button_Data_collect = new Button("Data Collection");
    DataPanel.add(Button_Data_collect);
    Panel ConstPanel = new Panel();
    ConstPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    Button_const = new Button("Constants");
    ConstPanel.add(Button_const);
    Panel TopPanel1 = new Panel();
    TopPanel1.setLayout(new GridLayout(0,1));
    Label l = new Label("Configure:",Label.LEFT);

    TopPanel1.add(l); 
    TopPanel1.add(ButtonCanvasPanel);
    TopPanel1.add(PlanePanel);
    TopPanel1.add(MBPanel);
    TopPanel1.add(ConstPanel);
    TopPanel1.add(DataPanel);		

    Panel TopPanel2 = new Panel();
    TopPanel2.setLayout(new BorderLayout());
    OutputArea1 = new TextArea(5,20);
    OutputArea1.setEditable(false);
    TopPanel2.add("North", new Label("Data"));
    TopPanel2.add("Center", OutputArea1);
          
    Panel TopPanel3 = new Panel();
    TopPanel3.setLayout(new BorderLayout());
    OutputArea2 = new TextArea(5,20);
    OutputArea2.setEditable(false);
    TopPanel3.add("North", new Label("Parameters"));
    TopPanel3.add("Center", OutputArea2);
		
    Panel TopPanel = new Panel();
    TopPanel.setLayout(new GridLayout(1,3));
    TopPanel.add(TopPanel1);
    TopPanel.add(TopPanel2);
    TopPanel.add(TopPanel3);
		
    add("North",TopPanel);
		
    // Create Simulation Canvas
    simCanvas.setBackground(Color.black);
		
    add("Center",simCanvas);
		
    Panel BottomPanel = new Panel();
    BottomPanel.setLayout(new FlowLayout(FlowLayout.CENTER));
    RunCounter = new TextField(new Integer(RunCount).toString(),3);
    RunCounter.setEditable(false);
    BottomPanel.add(RunCounter);
    Button_run = new Button("Run");
    BottomPanel.add(Button_run);
    Button_pause = new Button("Pause");
    BottomPanel.add(Button_pause);
    Button_reset = new Button("Reset");
    BottomPanel.add(Button_reset);
		
    add("South",BottomPanel);
		
    refresh();

    /*
     * We should have gotten strings identifying our DatabaseSet
     * and Database as applet parms. We'll get references to those
     * here, but won't create a table until the Run button
     * is pressed.
    String dbSetLocation = getParameter("DatabaseSet");
    debug("dbSetLocation="+dbSetLocation);
    if (dbSetLocation != null) {
      try {
	dbSet = Manager.getDatabaseSet(dbSetLocation);
	db = dbSet.getDatabase(getParameter("Database"));
	if (db != null) {
	  String[] tables = db.getTables();
	  RunCount = tables.length;
	  RunCounter.setText(new Integer(RunCount).toString());
	  debug("RunCount="+RunCount);
	}
      }
      catch (Exception e) {
	e.printStackTrace();
      }
    }
     */

    setupTable(); // part of quick hack 05/23/97 -Bo

  }


  private void createSession(String sessionURL) {
    /*
     * See if we have a sessionURL.  If so, we are in a collaborative
     * session and need to behave accordingly.
     */
    if (sessionURL != null) {
      debug("sessionURL "+sessionURL);

      try {
	session = SessionFactory.createSession(sessionURL);
	debug("session created: "+sessionURL);
      } catch (Exception e) {
	System.out.println("Error creating session: "+e);
	e.printStackTrace();
      }

      debug("after createSession");
      if (session != null) {
	debug("Session successfully created, opening channels...");
	simulationControl = new SimulationControl(session, this);
	debug("SimulationControl channel successfully joined");
	parameterControl = new ParameterControl(session, this);
 	debug("ParameterControl channel successfully joined");
     }
    }
  }

  private Frame getParentFrame() {
    Component comp = this;
    while (!(comp instanceof Frame)) {
      comp = comp.getParent();
    }
    return (Frame)comp;
  }

  public  void  refresh() {
    debug("refresh()");
    // Set the block redraw flag
    MB.reset();
    // Recompute the relative plane positions
    debug("refresh() setfloors and planes");
    floor.setLeftEnd(plane1.getLength()+plane1.getLeftEnd());
    plane2.setLeftEnd(floor.getLength()+floor.getLeftEnd());
    BC.repaint();	   
    simCanvas.repaint();
  }

  public  float getTimeStep() {
    return t_incr.floatValue();
  }
  public  void setTimeStep(float t) {
      t_incr.setValue(t);
  }
  public  Plane getPlane1() {
    return (Plane)plane1;
  }
  public  Plane getPlane2() {
    return (Plane)plane2;
  }
  public  Plane getFloor() {
    return (Plane)floor;
  }

  public boolean action(Event e, Object arg) {
    Object target = e.target;
    if (target == Button_run) {
      debug("run pre notifyParticipants");
      if (simulationControl != null)
	simulationControl.notifyParticipants(BnPMessage.RUN, null);
      debug("run post notifyParticipants");
      runSimulation(true);
      debug("run post pauseSimulation");
      return true;
    }
    if (target == Button_plane1) {
      Plane1Dialog = new PlaneDialog(plane1, getParentFrame(), this);
      Plane1Dialog.show();
      return(true);
    }
    if (target == Button_plane2) {
      Plane2Dialog = new PlaneDialog(plane2, getParentFrame(), this);
      Plane2Dialog.show();
      return(true);
    }
    if (target == Button_config) {
      MBDialog = new MovingBodyDialog((Block)MB, getParentFrame(), this);
      MBDialog.show();
      return(true);
    }
    if (target == Button_Data_collect) {
      DataCDialog = new DataCollectDialog(getParentFrame(), this);
      DataCDialog.show();
      return(true);
    }
    if (target == Button_const) {
      CDialog = new ConstantDialog(MB, getParentFrame(), this);
      CDialog.show();
      return(true);
    }
    if (target == Button_floor) {
      floorDialog = new FloorDialog(floor, getParentFrame(), this);
      floorDialog.show();
      return(true);
    }
    if (target == Button_pause) {
      debug("pause pre notifyParticipants");
      if (simulationControl != null)
	simulationControl.notifyParticipants(BnPMessage.PAUSE, null);
      debug("pause post notifyParticipants");
      pauseSimulation();
      debug("pause post pauseSimulation");
      return(true);
    }

    if (target == Button_reset) {
      debug("reset pre notifyParticipants (resetSimulation)");
      if (simulationControl != null)
	simulationControl.notifyParticipants(BnPMessage.RESET, null);
      debug("reset post notifyParticipants(resetSimulation)");
      resetSimulation();
      debug("reset post resetSimulation");      return(true);
    }
    return false;
  }

  public void runSimulation(boolean iPressedRun) {
    debug("runSimulation begin");
      Started = true;
      Button_run.disable();
      resetSimVars();
      if (thread == null) {
	  RunCount+=1;
          startNewTable(RunCount);   // part of quick hack 5/23/97 - Bo

	RunCounter.setText(new Integer(RunCount).toString());

/* PLI
	if ((db != null) && (iPressedRun)){
	  table = db.newTable("Run " + 
			      RunCounter.getText(),
			      columnNames, columnTypes, columnDescs,
			      "Block and Plane Applet");
	  //	  tableBuffer = new BoundedBufferVST(100);
	  tableBuffer = new VectorBuffer();
	  //	  TableWriter tableWriter = new TableWriter(table, tableBuffer);
	}
*/

	RecordUserSettings();
	thread = new SimulationThread(simCanvas, MB, this, getTimeStep());
	thread.start();
	//appendNewTable(); // part of quick hack 5/23/97 - Bo
      }
      else {
	thread.resume();			   
      }
    debug("runSimulation end");
  }

  public void pauseSimulation() {
    debug("pauseSimulation begin");
      if (thread != null) {
	thread.suspend();
	Button_run.setLabel("Resume");
	Button_run.enable();
	Button_run.resize(Button_run.preferredSize());
	validate();
      }
    debug("pauseSimulation end");
  }

  public void resetSimulation() {
    debug("resetSimulation begin");
      if (thread != null) {
	Button_run.enable();
	Button_run.setLabel("Run");
	Button_run.resize(Button_run.preferredSize());
	validate();
	Started = false;
	thread = null;
/* PLI
	if ((tableBuffer != null) && (table == null)){
	  debug("Opening table number "+RunCounter.getText());
	  table = db.newTable("Run " + 
			      RunCounter.getText(),
			      columnNames, columnTypes, columnDescs,
			      "Block and Plane Applet");
      	  TableWriter tableWriter = new TableWriter(table, tableBuffer);
	}
*/
	//PLI table = null;

	tableBuffer = null;
	MB.reset();
	simCanvas.repaint();
      }
    debug("resetSimulation end");
  }


  public void RecordUserSettings() {
    OutputArea1.appendText("\n");   
    OutputArea1.appendText("RUN "+RunCount+"\n");   
    OutputArea1.appendText("----------\n"); 	
    OutputArea2.appendText("RUN "+RunCount+"\n");   
    OutputArea2.appendText("----------\n"); 
    addHeader("Plane1 Length",new Float(plane1.getLength()).toString());
    addHeader("Plane1 Height",new Float(plane1.getHeight()).toString());
    addHeader("Plane1 Angle",new Float(plane1.getAngle()).toString());
    addHeader("Plane1 MuS",new Float(plane1.getMuS()).toString());
    addHeader("Plane1 MuK",new Float(plane1.getMuK()).toString());
    addHeader("Plane2 Length",new Float(plane2.getLength()).toString());
    addHeader("Plane2 Height",new Float(plane2.getHeight()).toString());
    addHeader("Plane2 Angle",new Float(plane2.getAngle()).toString());
    addHeader("Plane2 MuS",new Float(plane2.getMuS()).toString());
    addHeader("Plane2 MuK",new Float(plane2.getMuK()).toString());
    addHeader("Floor Length",new Float(floor.getLength()).toString());
    addHeader("Floor MuS  ",new Float(floor.getMuS()).toString());
    addHeader("Floor MuK",new Float(floor.getMuK()).toString());
    addHeader("Block Length",new Float(((Block)MB).getLength()).toString());
    addHeader("Block Height",new Float(((Block)MB).getHeight()).toString());
    addHeader("Time Step    ",new Float(getTimeStep()).toString());
    addHeader("Gravity      ",new Float(MB.getGravity()).toString());
    OutputArea2.appendText("\n");   	   
  }         
  public void addHeader(String ParamName, String Value) {
    OutputArea2.appendText(ParamName+"\t"+Value+"\n");
/*
    if (tableBuffer != null) {
      tableBuffer.put(new Attribute(ParamName, Value));
      //      table.setAttribute(ParamName, Value);
    }
*/
	if (currentTable != null) {
	    currentTable.setAttribute(ParamName, Value);
	}
  }

  float oldX = 0.0f;   //previous position
  float oldY = 0.0f;   // previous position
  float oldTime = 0.0f;
  float deltaTime = 0.0f;
  double deltaDistance = 0.0f;
  double distance = 0.0f;
  double oldDistance = 0.0f;
  double velocity = 0.0f;
  double deltaVelocity = 0.0f;
  double oldVelocity = 0.0f;
  double acceleration = 0.0f;
  int timeCounter = 0;

  private void resetSimVars() {
    oldX = 0.0f;   //previous position
    oldY = 0.0f;   // previous position
    oldTime = 0.0f;
    deltaTime = 0.0f;
    deltaDistance = 0.0f;
    distance = 0.0f;
    velocity = 0.0f;
    deltaVelocity = 0.0f;
    oldVelocity = 0.0f;
    acceleration = 0.0f;
    timeCounter = 0;
  }

  public void addRecord(float x,float y,float time) {
    debug("addRecord");
    timeCounter++;
    
    deltaDistance = Math.sqrt((x-oldX)*(x-oldX) + (y-oldY)*(y-oldY));
    //deltaDistance = distance-oldDistance;
    //oldDistance = distance;
    oldX = x;
    oldY = y;

    distance += Math.abs(deltaDistance);

    deltaTime = getTimeStep();
    if (timeCounter > 1) 
      velocity = deltaDistance/deltaTime;
    
    deltaVelocity = velocity-oldVelocity;
    oldVelocity = velocity;
    if (timeCounter > 2)
      acceleration = (deltaVelocity)/deltaTime;
    
    /* debug(" distance="+distance+
		       " velocity="+velocity+
		       " acceleration="+acceleration+
		       " time="+time);
     */

    if (time == 0)
      OutputArea1.appendText("Time\t Dist\t \tTime\tVel\t \tTime\tAcc\n");

    OutputArea1.appendText(
			   //RunCount + "\t" +
	        Fmt.fmt(time,3,3) + "\t"+ Fmt.fmt(distance,3,3) + "\t \t" +
	        Fmt.fmt(time,3,3) + "\t" + Fmt.fmt(velocity,3,3) + "\t \t" +
	         Fmt.fmt(time,3,3) + "\t" + Fmt.fmt(acceleration,3,3) + "\n");
    if (tableBuffer != null) {
      Object[] row = new Object[4];
      row[0] = new Long((long)(distance*1000));
      row[1] = new Long((long)(velocity*1000));
      row[2] = new Long((long)(acceleration*1000));
      row[3] = new Long((long)(time*1000));
      tableBuffer.put(row);
    }
      
/*
    debug("checking sensorTable != null");
    if (sensorTable != null) {
      debug("fact: sensorTable != null");
      Object[] row = new Object[4];
      row[0] = new Long((long)(time*1000));   // time
      row[1] = new Long((long)(distance*1000));  // distance
      row[2] = new Long((long)(velocity*1000));  // velocity
      row[3] = new Long((long)(acceleration*1000));  // acceleration
      sensorTable.appendRecord(row);
    }
*/
	if (currentTable != null) {
	    try {
	        currentTable.addObservation(time, distance, velocity,
		    acceleration);
	    }
	    catch(InterchangeException e) {
		e.printStackTrace();
	    }
	}
  }

  public void start() {
    super.start();
  }

  public void stop() {
    super.stop();
  }

  public void destroy() {
    disconnect();
  }

  private void disconnect() {
    if (simulationControl != null)
      simulationControl.shutdown();
  }

  public Dimension minimumSize() {
    return new Dimension(450,400);
  }

  public Dimension preferredSize() {
    return minimumSize();
  }


  protected void finalize() throws Throwable{
    destroy();
    super.finalize();
  }

  
  public static void main(String args[]) {
    inAnApplet = false;
    AppletFrame.startApplet("BlockAndPlane", 
			    "Block and Plane Simulator Frame", 
			    args);
  }

    //--------------------------------------

    Table tableOfTables = null; 
    SimTable currentTable  = null;

    void setupTable() {
      	// Setup the table
	tableOfTables = new sieve.sluice.local.TableImpl();
	FieldDescriptor fd = new FieldDescriptor("Simulation Results");
	fd.setType("sieve.sluice.TableView");
	try {
	    tableOfTables.addField(fd);
	}
	catch(InterchangeException e) {
	    e.printStackTrace();
	}
    }

    void startNewTable(int runNumber) {
	try {
	    currentTable = new SimTable(runNumber);
	    Object[][] record= {{currentTable}};
	    tableOfTables.append(record);
	}
	catch(InterchangeException e) {
	    e.printStackTrace();
	}
    }

    //------ TableView interface -----------------
    public TableView table() throws NoSourceException {
	return tableOfTables.table();
    }

    /**
     * Returns the TableDescriptor for this view's representation of
     * the table.
     */
    public TableDescriptor getTableDescriptor() throws NoSourceException {
	return tableOfTables.getTableDescriptor();
    }

    /**
     * Returns the names of fields in this view. These names should be
     * used to reference fields in the <CODE>get</CODE> methods.
     */
    public String[] getFieldNames() throws NoSourceException {
	return tableOfTables.getFieldNames();
    }

    /**
     * Returns the FieldDescriptor for the specified field.
     */
    public FieldDescriptor getFieldDescriptor(String field) 
        throws InterchangeException {
	return tableOfTables.getFieldDescriptor(field);
    }

    /**
     * Returns number of records in the view.
     */
    public int getRecordCount() throws NoSourceException {
	return tableOfTables.getRecordCount();
    }

    /**
     * Returns no more than <CODE>numberOfRecords</CODE> records starting
     * at record number <CODE>startingAt</CODE> from 
     * the fields specified in <CODE>fields</CODE>. The resulting array
     * is ordered by <CODE>[field][record]</CODE>.
     */
    public Object[][] get(String[] fields, int startingAt, int numberOfRecords) 
        throws InterchangeException {
	return tableOfTables.get(fields, startingAt, numberOfRecords);
    }

    /**
     * Returns all records starting from record number <CODE>startingAt</CODE>
     * from the fields specified in <CODE>fields</CODE>. (I still don't
     * like this because it doesn't add anything. _Every_ implementation
     * will almost certainly just have a call to the three-parm get...)
     */
    public Object[][] get(String[] fields, int startingAt) 
        throws InterchangeException {
	return tableOfTables.get(fields, startingAt);
    }
   
    /* -------------  Event-related methods ------------- */
    public void addTableViewListener(TableViewListener listener) {
	tableOfTables.addTableViewListener(listener);
    }

    public void removeTableViewListener(TableViewListener listener) {
	tableOfTables.removeTableViewListener(listener);
    }
    public void tableViewChanged(TableViewEvent event) {
	// We should never actually listen to anyone
	System.err.println("Block and Plane is (and shouldn't be) listening " +
	    " to " + event.getSource());
    }

    /* -------------  Methods for link support ------------- */
    public void addSourceView(TableView source) throws TooManySourcesException {
        throw new TooManySourcesException();
    }
    public void removeSourceView(TableView source) {
    }

  //------ End TableView interface -----------------


  //--------------------------------------
  // End quick hack
  //--------------------------------------

  private static  final boolean debug = false;
  private void debug(String msg) {
    if (debug)
      System.out.println(getClass().getName()+": "+msg);
  }
}



/**
  SimultaionThread handles running the simulation when Run is pressed.
 */
class SimulationThread extends Thread {
  private SimCanvas simCanvas = null;
  private  MovingBody MB = null;
  private BlockAndPlane bnp = null;
  private float timeStep = 0.0f;

  SimulationThread(SimCanvas simCanvas, MovingBody MB, 
		   BlockAndPlane bnp, float timeStep) {
    this.simCanvas = simCanvas;
    this.MB = MB;
    this.bnp = bnp;
    this.timeStep = timeStep;
  }

  public void run() {
    float t = 0;
    while ((SimCanvas.ScaleFactor*MB.getX_CM() < simCanvas.size().width) && 
	   bnp.Started ) {
      MB.move(t);				     
      bnp.addRecord(MB.getX_CM(), MB.getY_CM(), t);
      simCanvas.repaint();
      try {
	sleep((long)Math.round(timeStep*1000));
      } catch(InterruptedException e) {
	System.out.println(e);
      }
      t += timeStep;
    }
  }

}
