package sieve.phys.circuit;

public class AGraph{
        private int capacity = 100;
  	private Edge[][] GraphMatrix = new Edge[capacity][capacity];
        private Node[] NodeVector = new Node[capacity];
	private int NumVertex = 0;
	private int NumEdge = 0;
        private int localEID = 0;
        private int Variable = 1;
        private int orgDirection = -1;
        private boolean redComponents = false;
	
	AGraph(){
	   for(int i = 0; i < capacity; i++)
	   {
              //allocate room for the nodeVector
              NodeVector[i] = new Node(i);
	      //allocate room for each row
                    GraphMatrix[i] = new Edge[capacity];
	      for (int j = 0; j < capacity; j++)
	      {
		//now create an int for each cell
		GraphMatrix[i][j] = new Edge();
                GraphMatrix[i][j].positionx = i;
                GraphMatrix[i][j].positiony = j;
	      }
	   } 
	}

      public int Capacity(){
         return capacity;
      }

      public int NumberOfCurrents(){
         return Variable;
      }

      /* This procedure created new current values for all of the 
         edges and then assigns them new current values.
         Called by the Overlord class
      */

      public void AssignCurrentValues(){
         int StartAt = 0;
         NewCurrents();
         AssignCurrents(StartAt);
      }

     public void PrintCurrents(){
         System.out.println("CurrentValues: ");
         for(int i = 0; i < 15; i++){
            for(int j = 0; j < 15; j++){
               System.out.print(" " + GraphMatrix[i][j].CurrentName + " ");
            }
            System.out.println(" ");
         }
         System.out.println(" ");
      }
 
      public void PrintDirections(){
         CircuitComponent Temp;
         System.out.println("Directions: ");
         System.out.print(" ");
         for(int i = 0; i < 15; i++){
            System.out.print(" " + i + " ");
         }
         System.out.println(" "); 
         for(int i = 0; i < 15; i++){
            System.out.print(" " + "_" + " ");
         }
         System.out.println(" ");
         for(int i = 0; i < 15; i++){
            System.out.print(i + ":");
            for(int j = 0; j < 15; j++){
               Temp = GraphMatrix[i][j].GetComponent();
               if (Temp != null){
                  Temp.direction = GraphMatrix[i][j].Direction;
                  GraphMatrix[i][j].AddComponent(Temp);
               }
               System.out.print(" " + GraphMatrix[i][j].Direction + " ");
            }
            System.out.println(" ");
         }
      }

      private void NewCurrents(){
         Variable = 1;
         for(int i = 0; i < capacity; i++){
            for(int j = 0; j < capacity; j++){
               GraphMatrix[i][j].CurrentName = 0;
               GraphMatrix[i][j].Direction = orgDirection;
            }
         }
      }

      public boolean hasRedComponents(){
         return redComponents;
      }

      public void RedComponents(){
         redComponents = true;
         for (int i = 0; i<capacity; i++){
            for(int j =0; j < capacity; j++){
               if (!GraphMatrix[i][j].Components.isEmpty())
                  GraphMatrix[i][j].paintRed();
            }
         }
      }
  
      public void BlackComponents(){
         redComponents = false;
         for (int i = 0; i<capacity; i++){
            for(int j =0; j < capacity; j++){
               if (!GraphMatrix[i][j].Components.isEmpty())
                  GraphMatrix[i][j].paintBlack();
            }
         }
      }


      public int nodeEdges(int i){
         return NodeVector[i].returnEdges();
      }

/*This procedure assigns current values and directions to all of the 
  edges in the graph. 

  Called by: AssignCurrentValues in this class

*/
      private void AssignCurrents(int StartingPoint){
         /* This function gets all nodes with two or more edges and assigns 
            current variable names on those edges.
         */

         Variable = 1;
         for(int i = StartingPoint; i < capacity; i++){
            if (NodeVector[i].returnEdges() == 2){
               WireCurrentValues(i);
            }
         }
         for(int i = StartingPoint; i < capacity; i++){
            if (NodeVector[i].returnEdges() > 2){
               CurrentValues(i);
            }
         }
         for(int i = StartingPoint; i < capacity; i++){
            if (NodeVector[i].returnEdges() > 2){
               NodeDirections(i);
            }
         }
         for(int i = StartingPoint; i < capacity; i++){
            if (NodeVector[i].returnEdges() == 2){
               WireDirections(i, 0);
            }
         }
     }


    private void AssignNodeDirections(int StartingPoint, int comingFrom){
       int i = StartingPoint;
 
       if (NodeVector[i].returnEdges() == 2){
            WireDirections(i, comingFrom);
       }
       if (NodeVector[i].returnEdges() > 2){
            NodeDirections(i);
       }
    }


    private void AssignNodeCurrent(int StartingPoint){
       /* This function gets all nodes with two or more edges and assigns
            current variable names on those edges.
        */

       int i = StartingPoint;

       if (NodeVector[i].returnEdges() == 2){
            WireCurrentValues(i);
       }
       if (NodeVector[i].returnEdges() > 2){
            CurrentValues(i);
       }
    }


     private boolean BadWireDirections(int i){
        int firstDirection = -2;
        int secondDirection = -1;
        boolean notDone = true;
        int j;

        for(j = 0; (j < capacity) && notDone; j++){
           if (GraphMatrix[i][j].On == 1){
              firstDirection = GraphMatrix[i][j].Direction;
              notDone = false;
           }
        }
        notDone = true;
        int l = j;
        for(int k = l; (k < capacity) && notDone; k++){
           if (GraphMatrix[i][k].On == 1){
              secondDirection = GraphMatrix[i][k].Direction;
              notDone = false;
           }
        }
        
        if ((firstDirection != i) && (secondDirection != i)){
           return true; //this is a bad wire direction
        }
        else
           if ((firstDirection == i) && (secondDirection == i)){
              return true; //this is a bad wire direction
           }
           else
              return false;  //the wire directions are legitimate
     }
           

     private void WireDirections(int i, int comingFrom){
        int Direction;
        boolean FoundIncoming = false;

        for(int j = 0; ((j < capacity) && (FoundIncoming == false)); j++){
           if (GraphMatrix[i][j].On == 1 && GraphMatrix[i][j].Direction == orgDirection){
                 /* There is no direction for the first edge, find other
                    edge and see if it has a direction
                 */
                 int l = j;
                 for(int k = l+1; (k < capacity); k++){
                    if (GraphMatrix[i][k].On == 1){
                       if (GraphMatrix[i][k].Direction == orgDirection){
                          //other edge does not have a direction
                          GraphMatrix[i][j].Direction = i;
                          GraphMatrix[j][i].Direction = i;
                          //Then no other edge has direction
                          GraphMatrix[k][i].Direction = k;
                          GraphMatrix[i][k].Direction = k;
                          FoundIncoming = true;
                          if (NodeVector[k].returnEdges() == 2)
                             AssignNodeDirections(k, i);
                          return;
                       }
                       else
                          if (GraphMatrix[i][k].Direction == i){
                             /*Here we know that edge [i][k] has a direction and 
                               edge [i][j] does not have a direction in this wire.
                             */
                             GraphMatrix[i][j].Direction = j;
                             GraphMatrix[j][i].Direction = j;
                             FoundIncoming = true;
                          }
                          else
                             if (GraphMatrix[i][k].Direction == k){
                                //then other edge has outgoing direction
                                // make this edge incoming
                                GraphMatrix[i][j].Direction = i;
                                GraphMatrix[j][i].Direction = i;
                                FoundIncoming = true;
                             }
                    }//end if
                 } //end for
              } //end if
           else
             if (GraphMatrix[i][j].Direction == i){
                //this is an incoming edge, make other one outgoing
                for(int k = j+1; (k < capacity); k++){
                   if (GraphMatrix[i][k].On == 1){
                      if (GraphMatrix[i][k].Direction == orgDirection){
                         GraphMatrix[i][k].Direction = k;
                         GraphMatrix[k][i].Direction = k; 
                         FoundIncoming = true;
                         if (NodeVector[k].returnEdges() == 2)
                            AssignNodeDirections(k, i);
                         return;
                      }
                      else {
                         /********************************************* 
                            both edges in wire have directions. Check to make sure
                            that the edge directions aren't pointing toward each other
                            or away from each other
                         *********************************************/
                         if (BadWireDirections(i)){
                            int l;
                            if (k == comingFrom){
                               l = j;
                            }
                            else
                               l = k;
                            if (GraphMatrix[i][l].Direction == i){
                               GraphMatrix[i][l].Direction = l;
                               GraphMatrix[l][i].Direction = l;
                               if (NodeVector[l].returnEdges() == 2)
                                  AssignNodeDirections(l, i);
                               return;
                            }
                            else
                               if (GraphMatrix[i][l].Direction == l){
                                  GraphMatrix[i][l].Direction = i;
                                  GraphMatrix[l][i].Direction = i;
                                  if (NodeVector[l].returnEdges() == 2)
                                     AssignNodeDirections(k, i);
                                  return;
                               }
                         }   //end if
                      } //end else
                   }  //end if
                }  //end for
             }  //end if i
           else
             if (GraphMatrix[i][j].Direction == j){
                //this is an outgoing edge, make other edge incoming
                for(int k = j+1; (k < capacity); k++){
                   if (GraphMatrix[i][k].On == 1){
                      if (GraphMatrix[i][k].Direction == orgDirection){
                         GraphMatrix[i][k].Direction = i;
                         GraphMatrix[k][i].Direction = i;
                         FoundIncoming = true;
                        // System.out.println("This is directions for " + i);
                         if (NodeVector[k].returnEdges() == 2)
                            AssignNodeDirections(k, i);
                         return;
                      }
                      else{     
                         /*********************************************
                            both edges in wire have directions. Check to make sure
                            that the edge directions aren't pointing toward each other
                            or away from each other
                         *********************************************/
                          
                         if (BadWireDirections(i)){
                            int l;
                            if (k == comingFrom){
                                l = j;
                            }
                            else
                                l = k;
                            if (GraphMatrix[i][l].Direction == i){
                               GraphMatrix[i][l].Direction = l;
                               GraphMatrix[l][i].Direction = l;
                               if (NodeVector[l].returnEdges() == 2)
                                  AssignNodeDirections(l, i);
                               return;
                            }
                            else
                               if (GraphMatrix[i][l].Direction == l){
                                  GraphMatrix[i][l].Direction = i;
                                  GraphMatrix[l][i].Direction = i;
                                  if (NodeVector[l].returnEdges() == 2)
                                     AssignNodeDirections(l, i);
                                  return;
                               }
                         }   //end if bad
                      }   //end else
                   }   //end if on
                } //end for 
             } //end if j
         } //end for
   } //end procedure


/*Procedure: Node Directions
  Purpose: To give the edges connected to a node a pre-set direction for computational
           purposes
  Method:  This procedure assigns all un-assigned edges connected to a node an incoming
           direction. That is all un-assigned edges connected to edge i will have direction
           pointing toward i.
  Written by: Chad Hawthorne
*/

     private void NodeDirections(int i){
        boolean FirstFound = false;

      if(nodeEdges(i) > 2){         //make sure that this is a "node" with 3 or more wires
        for(int j = 0; (j < capacity); j++){
           if (GraphMatrix[i][j].On == 1 && GraphMatrix[i][j].Direction == orgDirection){
             //if first is zero check next to find possible incoming edges
                GraphMatrix[i][j].Direction = i;
                GraphMatrix[j][i].Direction = i; 
           } //end if 
        } //end for
      } //end if
      } //end
                
/*Procedure: CurrentValues
  CalledBy: AssignCurrents
  Purpose: To assign current names to the edges in nodes with more than two 
           edges connected to it.
*/

     private void CurrentValues(int i){
         for(int j = 0; j < capacity; j++){     
           if (GraphMatrix[i][j].On == 1){
               if (GraphMatrix[j][i].CurrentName == 0){
                  GraphMatrix[j][i].CurrentName = Variable;
                  GraphMatrix[i][j].CurrentName = Variable++;
               }
           }
         } 
     }

/* This procedure is called when there are only two edges 
coming into a node. it gives all of the edges current values */

     private void WireCurrentValues(int i){
        for(int j = 0; j < capacity; j++){
           if (GraphMatrix[i][j].On == 1 && GraphMatrix[i][j].CurrentName != 0){
           /* if this edge is on and already has a current value
            * then find the other edge from this node and give it that Current
            * Value
            */
              int l = j;
              for(int k = ++l; k < capacity; k++){
                 if (GraphMatrix[i][k].On == 1){ 
                    if ((GraphMatrix[i][k].CurrentName == 0)){
                       GraphMatrix[k][i].CurrentName = GraphMatrix[i][j].CurrentName;
                       GraphMatrix[i][k].CurrentName = GraphMatrix[i][j].CurrentName;
                       if(NodeVector[k].returnEdges() == 2){
                          AssignNodeCurrent(k);
                       }
                    }
                 }
              }
           }
           else
              if (GraphMatrix[i][j].On == 1 && GraphMatrix[i][j].CurrentName == 0){
              /* if this edge is on and does not have a current value yet then find the other
               * edge from this node and see if it has a current value
               */
                 int l = j;
                 for(int k = ++l; k < capacity; k++){
                     if (GraphMatrix[i][k].On == 1){
                        /* the second edge has no value*/
                        if ((GraphMatrix[i][k].CurrentName == 0)){
                           GraphMatrix[k][i].CurrentName = Variable;
                           GraphMatrix[i][k].CurrentName = Variable;
                           GraphMatrix[i][j].CurrentName = Variable;
                           GraphMatrix[j][i].CurrentName = Variable++;
                           if(NodeVector[k].returnEdges() == 2){
                              AssignNodeCurrent(k);
                           }
                        }
                        else{
                           if ((GraphMatrix[i][j].CurrentName == 0)){
                              /*the second edge has a value and we give it to the first*/
                              GraphMatrix[i][j].CurrentName = GraphMatrix[k][i].CurrentName;
                              GraphMatrix[j][i].CurrentName = GraphMatrix[k][i].CurrentName;
                              if(NodeVector[j].returnEdges() == 2){
                                 AssignNodeCurrent(j);
                              }
                           }
                        }
                     }
                 }
               }
      }
}

   public boolean EdgesInLoops(Search SearchGraph){
      for (int i = 1; i < capacity; i++){
          for(int k = i; k < capacity; k++){
            if (GraphMatrix[i][k].On == 1)
               if (!(SearchGraph.edgeIsInLoop(GraphMatrix[i][k].EID))){
                  System.out.println("Edge not in loop: " + i + " " + k);
                  return false;
               }
          }
      }
      return true;
   }



      public void PrintGraph(){
         for(int i = 0; i < 10; i++){
             for(int j = 0; j < 10; j++){
                System.out.print(GraphMatrix[i][j].On + "  ");
             }
          System.out.println(" ");
         }
       }
  
      public void PrintNodes(){
         System.out.println("Node List ");
         for(int i = 0; i < capacity; i++){
             System.out.print(i+ ": "  + NodeVector[i].returnEdges() + " ");
         }
         System.out.println(" ");
      }

	public int Vertices(){
	   return NumVertex;
	}

	public int nEdges(){
 	   return NumEdge;
	}

        public void AddComponent(int Vertex1, int Vertex2, 
	   CircuitComponent newComp){
           GraphMatrix[Vertex1][Vertex2].AddComponent(newComp);
           GraphMatrix[Vertex2][Vertex1].AddComponent(newComp);
        }

        public void ConnectNodes(int Vertex1, int Vertex2){
           NumEdge++;
           localEID++;
           //System.out.println("LocalEID: " + localEID);
           GraphMatrix[Vertex1][Vertex2].EID = localEID;
           GraphMatrix[Vertex2][Vertex1].EID = localEID;
           GraphMatrix[Vertex1][Vertex2].MakeConnection();
           GraphMatrix[Vertex2][Vertex1].MakeConnection();
           NodeVector[Vertex1].addEdge();
           NodeVector[Vertex2].addEdge();
        }

        public Edge getEdge(int Vertex1, int Vertex2){
           return GraphMatrix[Vertex1][Vertex2];
        }

        /*This function returns an edge that has been modified after
          a call to getEdge
        */ 

        public void returnEdge(Edge retEdge, int Vertex1, int Vertex2){
           GraphMatrix[Vertex1][Vertex2] = retEdge;
        }


        /*
         Function: RemoveNodes
         Purpose: This procedure removes an edge from the adjacency 
                  matrix graph given the number of the nodes the edge 
                  connects. It also removes the edgeID from the list of 
                  edge ID's kept in SearchGraph.
         SideEffects: this procedure decreased the number of edge counter 
        */

        public void RemoveNodes(int Vertex1, int Vertex2, Search SearchGraph){
           NumEdge--;
           System.out.println("Remove Nodes from Graph: " + Vertex1 + Vertex2);
           SearchGraph.removeEdge(GraphMatrix[Vertex1][Vertex2].EID);
           GraphMatrix[Vertex1][Vertex2].RemoveConnection();
           GraphMatrix[Vertex2][Vertex1].RemoveConnection();
           NodeVector[Vertex1].removeEdge();
           NodeVector[Vertex2].removeEdge();
        }
	
	/* This function returns the first edge of a Vertex*/
	public Edge First(int VertexNum){
                  Edge TempEdge = new Edge();
        for (int k = VertexNum; k <capacity; k++){
	   for (int j = 0; j < capacity; j++){
		TempEdge = GraphMatrix[k][j];
		 if (TempEdge.On != 0)
			return TempEdge;
	   }
        }
         return TempEdge; //returns null if there is no Edge  
  	}


	/* This function returns true if the Vertex (VertexNum) has 
	     an edge that goes to "Conntects To", false otherwise. */

	public boolean IsEdge(int VertexNum, int ConnectsTo){
                 Edge TempEdge = new Edge();
	   TempEdge = GraphMatrix[VertexNum][ConnectsTo];
	   if(TempEdge.On != 0)
		return true;
	    else
		return false;
	}

	/* This function returns the Edge from the Vertex "VertexNum"
	     That comes after "ConnectsTo" If this operation is called on the 
      last vertex the first edge will be returned */

	public Edge Next(int VertexNum, int ConnectsTo){
	   int Start = ConnectsTo;
	   Edge TempEdge = new Edge();
	   for (int j = ++ConnectsTo; j < capacity; j++){
		TempEdge = GraphMatrix[VertexNum][j];	
		if (TempEdge.On != 0)
			return TempEdge;
	   }
         /*
	   for (int i = 0; i < ConnectsTo; i++){
		TempEdge = GraphMatrix[VertexNum][i];
		if(TempEdge.On != 0)
			return TempEdge;
	   }
         */
	   return TempEdge;   //return null if Next is not found
	}

	/* This function returns the weight of the edge at position VertexNum
	   and connectsTo */

	public int EID(int VertexNum, int ConnectsTo){
	   return GraphMatrix[VertexNum][ConnectsTo].EID;
}


}
