
package sieve.phys.circuit;
import java.awt.*;
import java.awt.event.*;
import sieve.workbench.*;

public class CircuitComponent extends  Container implements
        WorkbenchComponent {
   static {
        ToolFactory.getToolFactory().registerTool(
            CircuitComponent.class, CircuitComponentTool.class);
    }
    private UniqueID id = null;
    private boolean BlownCircuit = false;
 
    protected Terminal t1 = null;
    protected Terminal t2 = null;
    protected Terminal selectedTerminal = null;
    protected boolean selected = false;
    protected boolean red = false;
    private Workbench wb = null;
        public boolean Orientation=true; 
	public int ComponentID = 0;
        public int Value = 0;
        private double Amps = 0;
        public int node1 = 0;
        public int node2 = 0;
        public int direction = 0;

public CircuitComponent() {
        id = new UniqueID(this);
        t1 = new Terminal(this);
        t2 = new Terminal(this);
        setLayout(null);
        add(t1);
        add(t2);
        Listener l = new Listener(this);
        addMouseListener(l);
        addMouseMotionListener(l);
        setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
      

}
 void setWorkbench(Workbench wb) {
        this.wb = wb;
    }
 
  public boolean getOrientation(){
  return Orientation;
  }
  public void setOrientation(boolean orient){
  Orientation=orient;
  
  layout();
  wb.validate();
  }
  
  public void layout() {
       if (Orientation)
        {
        Dimension pref = t1.getPreferredSize();
        t1.setBounds(2, (40 - pref.height)/2, pref.width, pref.height);
        pref = t2.getPreferredSize();
        t2.setBounds(68 - pref.width, (40 - pref.height)/2, pref.width,
            pref.height);
         setSize( 70,70);
       }
      else 
        {
         Dimension pref = t1.getPreferredSize();
        t1.setBounds((40 - pref.width)/2, 2, pref.width, pref.height);
        pref = t2.getPreferredSize();
        t2.setBounds((40-pref.width)/2 , 68 - pref.height, pref.width, pref.height);
         setSize( 40,70);
        }

    }

    public Dimension getPreferredSize() {
       if (Orientation)
        return new Dimension(70,70);
        else
        return new Dimension(40,70);
 

    }
 
    public Dimension getMinimumSize() {
        if (Orientation)
        return new Dimension(70,70);
        else
        return new Dimension(40,70);
 

    }

 public boolean contains(int x, int y) {
        Rectangle t1Bounds = t1.getBounds();
        Rectangle t2Bounds = t2.getBounds();
        if (Orientation)
         return (x >= t1Bounds.x) && (x <= t2Bounds.x + t2Bounds.width) &&
            (y >= t1Bounds.y) && (y <= t1Bounds.y + t1Bounds.height);
        else 
         return (x >= t1Bounds.x) && (x <= t1Bounds.x + t1Bounds.width) &&
            (y >= t1Bounds.y) && (y <= t2Bounds.y + t2Bounds.height);
       }
 
    public void terminalSelected(Terminal t) {
        CircuitComponentLink link = new CircuitComponentLink(wb, this, t);
        selectedTerminal = t;
        if (wb.requestLinkStart(this, link))  {
            link.setBounds(0, 0, wb.getSize().width, wb.getSize().height);
            wb.add(link);
        }
        else
            {
             wb.requestSelection(this);
           }
    }
 
    public Terminal getTerminalWithID(Integer id) {
        if (t1.getID().equals(id))
            return t1;
        if (t2.getID().equals(id))
            return t2;
        return null;
    }
 
    public Terminal[] getTerminals() {
        Terminal[] terminals = {t1, t2};
        return terminals;
    }
 
   Terminal getSelectedTerminal() {
        return selectedTerminal;
    }
 
 private void clearSelectedTerminal() {
        selectedTerminal = null;
    }

 // --------------------  WorkbenchComponent methods --------------------
    public synchronized UniqueID getUniqueID() {
        return id;
    }
 
    public synchronized void setUniqueID(UniqueID id) {
        this.id = id;
    }
 
    // Use for drawing handles only?
    public void setColorSource(ColorSource cs) {
    }
 
    public Object getModel() {
        return this;
    }
 
    public Component getView() {
        return this;
    }
 
    public void remove() {
        wb.remove(this);
        wb.componentRemoved(this);
    }
 
    public boolean isSelected() {
        return selected;
    }
 
  public void setSelected(boolean selected) {
        if (selected != this.selected) {
            this.selected = selected;
            if (selected) {
                if (wb.getCurrentSelection() != this)
                    wb.setCurrentSelection(this);
                setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
                wb.requestEditor(this);
                setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            }
            repaint();
        }
    }

  public void paintRed(boolean red){
        if (red != this.red) {
           this.red = red;
           repaint();
        }
  }

	
 // -------------------- Mouse stuff --------------------
    private class Listener extends MouseAdapter implements MouseMotionListener
{
        private CircuitComponent cc = null;
        private Point pressedPoint = null;
        private boolean wasPressed = false;
        private boolean wasDragged = false;
 
        public Listener(CircuitComponent cc) {
            this.cc = cc;
        }
        public void mousePressed(MouseEvent e) {
            cc.clearSelectedTerminal();
            wasPressed = true;
            pressedPoint = e.getPoint();
        }
        public void mouseReleased(MouseEvent e) {
            cc.clearSelectedTerminal();
            if (!cc.isSelected() && wasPressed && !wasDragged) {
                if (wb.requestSelection(cc)) {
                    wb.setCurrentSelection(cc);
                }
            }
            wasPressed = false;
            wasDragged = false;
        }
 
        public void mouseMoved(MouseEvent e) {
        }
 
      public void mouseDragged(MouseEvent e) {
            if (wasPressed) {
                Point loc = cc.getLocation();
                Point p = e.getPoint();
                cc.setLocation(new Point(loc.x + p.x - pressedPoint.x,
                    loc.y + p.y - pressedPoint.y));
                wasDragged = true;
                wb.componentBoundsChanged(cc);
            }
        }
 
    }
 
 


CircuitComponent(int SetID){
   ComponentID = SetID;
}

public int GetValue(){
   return Value;
}

public void SetBatteryDirection(Integer Override){
}

public int batteryDirection(){
   return 0;
}

public void setAmps(double Amp){
          Amps = Amp;
}


public void paint(Graphics g) {}

public void badCircuit(){
   BlownCircuit = true;
}

public void goodCircuit(){
   BlownCircuit = false;
}

}
