package sieve.phys.circuit;

import java.awt.*;
import java.io.IOException;
import java.util.*;


public class Overlord{

   Vector PathVector;
   AGraph AdjMatrixGraph;
   Search SearchGraph;
   Equation EquationCreator;



   public Overlord(){
      AdjMatrixGraph = new AGraph();
      SearchGraph = new Search();
      EquationCreator = new Equation();
      PathVector = new Vector(); 
   }

   public void addNodes(Integer SelectedNode, Integer SelectedNode2){
      AdjMatrixGraph.ConnectNodes(SelectedNode.intValue(), SelectedNode2.intValue());
      System.out.println("Added Nodes: " + SelectedNode.intValue() + " " + SelectedNode2.intValue());
      PathVector = SearchGraph.findLoops(AdjMatrixGraph);
      AdjMatrixGraph.AssignCurrentValues();
   }

   public void addComponent(CircuitComponent NewComp, Integer SelectedNode, Integer SelectedNode2){

      addNodes(SelectedNode, SelectedNode2);
      NewComp.node1 = SelectedNode.intValue();
      NewComp.node2 = SelectedNode2.intValue();
      System.out.println("Added Nodes: " + SelectedNode.intValue() + " " + SelectedNode2.intValue());
      /*****************************************************
       This sets the direction of the battery to the first integer passed into the 
       addComponent function. This means that the positive direction of the battery
       goes toward the direction of the node SelectedNode
      *****************************************************/
      if(NewComp instanceof Battery){
         NewComp.SetBatteryDirection(SelectedNode);
      }
      AdjMatrixGraph.AddComponent(SelectedNode.intValue(), SelectedNode2.intValue(), NewComp);
   }

   public void removeNodes(Integer SelectedNode, Integer SelectedNode2){
      AdjMatrixGraph.RemoveNodes(SelectedNode.intValue(), SelectedNode2.intValue(), SearchGraph);
      PathVector = SearchGraph.findLoops(AdjMatrixGraph);
      AdjMatrixGraph.AssignCurrentValues();
    }

/*
  Function: findLoops

  Purpose:
   this function solves the circuit. If the circuit has loops in it then 
   hasLoops is set to True and the Gaussian Elimination solver is 
   run on the system of equations. Otherwise nothing is done.
*/


   public void findLoops(){
      try {
         if (AdjMatrixGraph.hasRedComponents()){
            System.out.println("Change to black");
            AdjMatrixGraph.BlackComponents();
         }
         PathVector = new Vector();
         AdjMatrixGraph.AssignCurrentValues();
//       AdjMatrixGraph.PrintCurrents();
//       AdjMatrixGraph.PrintDirections();
//       AdjMatrixGraph.PrintGraph();
         PathVector = SearchGraph.findLoops(AdjMatrixGraph);
         Vector CopyPathVector = (Vector)PathVector.clone();

         /************************
           EquationCreator.FindLoopEquations must come before EquationCreator.FindJunctionEquations
           otherwise the error checking for the correct number of variables will not work correctly
         *************************/
         boolean edgesInLoops = AdjMatrixGraph.EdgesInLoops(SearchGraph);
         boolean hasLoops = EquationCreator.FindLoopEquations(AdjMatrixGraph, PathVector);
         boolean enoughLoops = EquationCreator.FindJunctionEquations(AdjMatrixGraph);
//         EquationCreator.PrintLoopEquations();
//         EquationCreator.PrintJunctionEquations();
//         System.out.println("hasLoops: " + hasLoops + " enoughLoops: " + enoughLoops + " edgesInLoops: " 
//                           + edgesInLoops);
         if (hasLoops && enoughLoops && edgesInLoops){
            EquationCreator.SolveEquations(AdjMatrixGraph, CopyPathVector);
         }
      }
 
      catch (BadCircuitException b){
           System.out.println("Change components to red");
           AdjMatrixGraph.RedComponents();
      }
}

   private void BuildLoopVector(){
      /* This function creates a vector of vectors. The vector objects
         will be the equations from the loop rule. 

         The positions in the Vector represent the variable names of the Currents
      */

      Vector Equations;             //a vector of equations returned by BuildLoops
      Equations = new Vector();
      EquationCreator.FindLoopEquations(AdjMatrixGraph, PathVector);
   }
   private void printVector(){
      Vector TempVector;
      Integer TempInt;
      System.out.println("This is a loop:");
      for (int i = 0; i<PathVector.size(); i++){
        TempVector =(Vector)PathVector.elementAt(i);
        for (int j = 0; j<TempVector.size(); j++){
           TempInt = (Integer)TempVector.elementAt(j);
           System.out.println(TempInt.intValue());
        }
        System.out.println(" ");
      }
   }

}

