package sieve.phys.demo;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.util.*;
import java.lang.reflect.*;
import sieve.quarry.*;

public class DemoToolPanel extends Panel {

    ListPanel listPanel = null;

    public DemoToolPanel() {
	setLayout(new BorderLayout());
	add("North", new Label("Tools", Label.CENTER));
	Button button = new Button(" ");
	add("South", button);
	listPanel = new ListPanel(this, button);
	add("Center", listPanel);
	

	ToolSet simulations = new ToolSet("Simulations");
	simulations.addBeanClass("sieve.phys.lwblockandplane.BlockAndPlane");
	simulations.addBeanClass("sieve.phys.blockandplane.BlockAndPlane");

	ToolSet whiteboard = new ToolSet("Annotation Tools");
	whiteboard.addBeanClass("sieve.whiteboard.PostItNote");
	whiteboard.addBeanClass("sieve.whiteboard.Box");
	whiteboard.addBeanClass("sieve.whiteboard.Oval");
	whiteboard.addBeanClass("sieve.whiteboard.Label");


	ToolSet selectors = new ToolSet("Data Selectors");
	selectors.addBeanClass("sieve.sluice.views.TableSelectionView");

	ToolSet viewers = new ToolSet("Data Viewers");
	viewers.addBeanClass("sieve.nuggets.LineGraph.LineGraph");
	viewers.addBeanClass("sieve.nuggets.TextTable");
	viewers.addBeanClass("sieve.nuggets.AttributeViewer");
	viewers.addBeanClass("sieve.nuggets.BarChart");
	viewers.addBeanClass("sieve.nuggets.Histogram");
	viewers.addBeanClass("sieve.nuggets.Graph");

	ToolSet other = new ToolSet("Other Tools");
	other.addBeanClass("sieve.sluice.views.CriteriaView");
	other.addBeanClass("sieve.sluice.views.FieldNameSelectionView");
	other.addBeanClass("sieve.sluice.views.FrequencyTableView");
	other.addBeanClass("sieve.sluice.views.ColorChangeView");
	other.addBeanClass("sieve.sluice.weblog.TableImpl");
	other.addBeanClass("sieve.sluice.views.CacheView");

	ToolSet sosci = new ToolSet("SocSci Tools");
	sosci.addBeanClass("sieve.sluice.codebook.CodeBookTable");
	sosci.addBeanClass("sieve.sluice.gss.GSSTable");
	sosci.addBeanClass("sieve.nuggets.StretchableHistogram");
	
	listPanel.addToolSet(simulations);
	listPanel.addToolSet(whiteboard);
	listPanel.addToolSet(selectors);
	listPanel.addToolSet(viewers);
	listPanel.addToolSet(sosci);
	listPanel.addToolSet(other);

	listPanel.selectToolSet("Simulations");
	listPanel.selectTool(0);
    }
    
}

class ListPanel extends Panel implements ActionListener, Runnable {
    DemoToolPanel toolPanel = null;
    Hashtable setMap = null;
    Hashtable setIdxMap = null;
    String lastSet = null;
    String currentTool = null;

    List sets = null;
    List tools = null;
    Button button = null;

    Thread life = null;
    Object pendingBean = null;

    public ListPanel(DemoToolPanel toolPanel, Button button) {
	this.toolPanel = toolPanel;
	this.button = button;
	button.addActionListener(this);
	setLayout(new GridLayout(1, 2, 2, 2));
	sets = new List();
	sets.addItemListener(new SetListListener(this));
	tools = new List();
	tools.addItemListener(new ToolListListener(this));
	add(sets);
	add(tools);
	setMap = new Hashtable();
	setIdxMap = new Hashtable();

	life = new Thread(this);
	life.start();
    }

    public void addToolSet(ToolSet ts) {
	setMap.put(ts.getName(), ts);
	setIdxMap.put(ts.getName(), new Integer(sets.getItemCount()));
	sets.add(ts.getName());
    }

    public synchronized void selectToolSet(String setName) {
	ToolSet ts = (ToolSet)setMap.get(setName);

	if (setName.equals(lastSet))
	    return;

	if (ts != null ) {
	    lastSet = setName;
	    int idx = ((Integer)setIdxMap.get(setName)).intValue();
	    if (idx != sets.getSelectedIndex())
		sets.select(idx);
	    String[] toolNames = ts.getLabels();
	    tools.removeAll();
	    for (int t = 0; t < toolNames.length; t++)
		tools.addItem(toolNames[t]);
	    tools.select(0);
	    selectTool(0);
	}
    }

    public synchronized void selectToolSet(int idx) {
	selectToolSet(sets.getItem(idx));
    }

    public synchronized void selectTool(int idx) {
	currentTool = tools.getItem(idx);
	button.setLabel("Add New \"" + currentTool + "\"");
    }

    public void actionPerformed(ActionEvent e) {
	if (button.getLabel().startsWith("Click"))
	    return;
	button.setLabel("Click on workspace to add component");
	button.disable();
	sets.disable();
	tools.disable();

 
        // Change the cursor to indicate an "insert".
	setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
 
	// Find the current tool
	ToolSet ts = (ToolSet)setMap.get(lastSet);
	synchronized(this) {
	    pendingBean = ts.getBeanLabeled(currentTool);
	    notify();
	}
    }

    public synchronized void run() {
	// We run an internal thread to handle insertions, one at a time.
        for (;;) {
            Object bean;
            synchronized (this) {
                // Wait for a bean to be ready for insertion.
                while (pendingBean == null) {
                    try {
                        wait();
                    } catch (InterruptedException ix) {
                        System.err.println("ToolBox.run: unexpected interrupt");
                    }
                }
                bean = pendingBean;
		pendingBean = null;
            }

            // Find our the current BeanBox.
            BeanBox box = BeanBoxFrame.getCurrentBeanBox();

            // do the insertion.
            box.doInsert(bean);
 
	    button.setLabel("Add New \"" + currentTool + "\"");
	    button.enable();
	    sets.enable();
	    tools.enable();
	    setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	}
    }
}


class SetListListener implements ItemListener {
    ListPanel listPanel = null;

    public SetListListener(ListPanel listPanel) {
	this.listPanel = listPanel;
    }

    public void itemStateChanged(ItemEvent e) {
	if (e.getStateChange() == ItemEvent.SELECTED) {
	    listPanel.selectToolSet(((Integer)e.getItem()).intValue());
	}
    }
}

class ToolListListener implements ItemListener {
    ListPanel listPanel = null;

    public ToolListListener(ListPanel listPanel) {
	this.listPanel = listPanel;
    }

    public void itemStateChanged(ItemEvent e) {
	if (e.getStateChange() == ItemEvent.SELECTED) {
	    listPanel.selectTool(((Integer)e.getItem()).intValue());
	}
    }
}

// A set of tools, e.g., visualizations, filters, or simulations
class ToolSet {
    Vector beanLabels = new Vector();
    Vector beanNames = new Vector();
    Vector beanIcons = new Vector();
    Vector beanJars = new Vector();
    Hashtable labelMap = new Hashtable();
    String name = null;

    public ToolSet(String name) {
	this.name = name;
    }

    public String getName() {
	return name;
    }

    public String[] getLabels() {
	String[] labels = new String[beanLabels.size()];
	beanLabels.copyInto(labels);
	return labels;
    }

    public void addBeanClass(String beanClassName) {
        Class beanClass;
        try {
            beanClass = Class.forName(beanClassName);
        } catch (Exception ex) {
            System.err.println("Toolbox: couldn't instantiate " +
                                                             beanClassName);
            return;
        }
        BeanInfo bi;
        try {
            bi = Introspector.getBeanInfo(beanClass);
        } catch (Exception ex) {
            System.err.println("Toolbox: couldn't find BeanInfo for " +
                                                                beanClassName);
            ex.printStackTrace();
            return;
        }

        String label = bi.getBeanDescriptor().getDisplayName();
        label = getUniqueName(beanLabels, label);
	beanLabels.addElement(label);

        Image img = bi.getIcon(BeanInfo.ICON_COLOR_16x16);
	labelMap.put(label, beanClassName);
        beanNames.addElement(beanClassName);
        beanIcons.addElement(img);
        //beanJars.addElement(null);
    }

    public synchronized Object getBeanLabeled(String label) {
	String beanName = (String)labelMap.get(label);
//System.out.println("Got bean name: " + beanName);
	if (beanName != null) {
            Object bean = null;
 
            if (beanName.equals("sieve.quarry.BeanBox")) {
		// should pass in BeanBoxFrame.atts -Bo
                bean = new BeanBox(new java.util.Hashtable());  
            } 
	    else {
                try {
//System.out.println("Trying to load: " + beanName);
                   Class cl = Class.forName(beanName);
//System.out.println("Got class: " + cl);
                   bean = cl.newInstance();
//System.out.println("Got instance: " + bean);
	           return bean;
                } 
		catch (Exception exe) {
	 	    exe.printStackTrace();
                }
            }
	}
	return null;
    }

    String getUniqueName(Vector v, String s) {
        if (! v.contains(s)) {
            return s;
        }
        int count = 2;
        while (v.contains(s+"<"+count+">")) {
            count += 1;
        }
	return s+"<"+count+">";
    }
}

