/*
 *
 * Written By: 	Michael Parks
 * Date:			6/23/96
 * Description:	Surface SuperClass
 *
 * Usage Notes:
 *
 * 
 */
  
/*
 *
 * Copyright (c) 1996 Michael Parks. All Rights Reserved.
 *
 * 
 */
package sieve.phys.lwblockandplane;

import java.awt.*;


public abstract class Surface {
  // all units of length in meters
    protected float height = 0;	
    protected float length = 0;
   protected float SurfaceLength = 0;
   protected float MuS = 0; 		// Coefficient of Static Friction
   protected float MuK = 0; 		// Coefficient of Kinetic Friction
   protected float MuR = 0; 		// Coeffienent of Static Rolling Friction
   protected float angle = 0; 		// Angle in Radians


   protected int   LeftEnd = 0; 		// Ends of plane in pixels
   protected int   RightEnd = 0;
   protected Polygon PlanePoly = null;

    public Surface(float ht,
		   float lngth,
		   float NewMuS,
		   float NewMuK,
		   float NewMuR,
		   int StartPos) {
      this(ht, lngth, NewMuS, NewMuK, NewMuR, StartPos, "NoName");
    }
   
    public Surface(float ht,
		   float lngth,
		   float NewMuS,
		   float NewMuK,
		   float NewMuR,
		   int StartPos,
		   String name) {
      //StartPos is the leftmost end of the plane, in pixels
	height = ht;
	length = lngth;
	MuS = NewMuS;
	MuK = NewMuK;
	MuR = NewMuR;

      LeftEnd  = StartPos;      
      RightEnd = StartPos + (int)(lngth * SimCanvas.ScaleFactor);
      init();
   }
   protected void init() {

      float HypSqrd;
      HypSqrd = (getLength()*getLength()) + 
	(getHeight()*getHeight());
      SurfaceLength = (float)Math.sqrt(HypSqrd);
      angle = (float)Math.atan((double)getHeight()/getLength());
      PlanePoly = null;
      RightEnd = LeftEnd + Math.round(getLength()*SimCanvas.ScaleFactor);
   }

   public abstract void paint(Graphics g,Dimension d);  
 
   public void setHeight(float NewHeight) {
	height = NewHeight;
      init();
   }
   public float getHeight() {
     return height;
   }

   public void setLength(float NewLength) {
       length = (NewLength);
   }

   public float getLength() {
     return length;
   }

   public void setAngle(float NewAngle) {
      angle = NewAngle;
      height = ((float)(getLength() * Math.tan((double)angle)));
   }

   public float getAngle() {
     return angle;
   } 

   public float getSurfaceLength() {
     return SurfaceLength;
   }

   public void setMuS(float NewMuS) {
     MuS = (NewMuS);
   }

   public float getMuS() {
     return MuS;
   }

   public void setMuK(float NewMuK) {
     MuK = (NewMuK);
   }

   public float getMuK() {
     return MuK;
   }

   public void setMuR(float NewMuR) {
     MuR = (NewMuR);
   }

   public float getMuR() {
     return MuR;
   }

   public void setLeftEnd(int NewLeftEnd) { 
     //Set in pixels
     LeftEnd = NewLeftEnd;
     RightEnd =(int)(LeftEnd + (getLength()*SimCanvas.ScaleFactor));
   }

   public void setLeftEnd(float NewLeftEnd) { //Set in meters
      LeftEnd = (int)(NewLeftEnd*SimCanvas.ScaleFactor);
      RightEnd = (int)(LeftEnd + (getLength()*SimCanvas.ScaleFactor));
   }

   public float getLeftEnd() { // Get Left End in meters
      return((float)((float)LeftEnd/(float)SimCanvas.ScaleFactor));
   }

  public float getRightEnd() { // Get Right End in meters
      return((float)((float)RightEnd/(float)SimCanvas.ScaleFactor));
  }

   public boolean inside(float position) {
      if (((position*SimCanvas.ScaleFactor) >= LeftEnd) 
      &&  ((position*SimCanvas.ScaleFactor) < RightEnd))
         return true;
      else 
         return false;
   }
}
