/*
 * Same as PropertySheet, only we're a panel instead of a frame...
 */

package sieve.quarry;

import java.beans.*;
import java.lang.reflect.*;
import java.awt.*;
import java.util.Hashtable;
import java.util.Vector;
import sieve.crew.*;

public class PropertyPanel extends Panel implements PropertyChangeListener {
    BeanBox box = null;
    boolean targetHasCustomizer = false;
    Hashtable propertyMap = null;

    PropertyPanel() {
	setLayout(null);
	setBackground(Color.lightGray);	
        box = BeanBox.findBeanBox(this);
    }

    PropertyPanel(Object target) {
        this();
	setTarget(target);
    }

    public boolean getTargetHasCustomizer() {
	return targetHasCustomizer;
    }

    public void setBeanBox(BeanBox box) {
      this.box = box;
    }

    synchronized void setTarget(Object targ) {

	if ((targ != target) && (target != null)) {
	    // Remove ourselves as a listener from our old target
	    try {
		Class[] args = { PropertyChangeListener.class };
	    	Method registerMethod = 
		    target.getClass().getMethod("removePropertyChangeListener", 
		    args);
		Object[] me = { this };
		registerMethod.invoke(targ, me);
	    }
	    catch(Exception e){}
	}

	target = targ; 
	removeAll();

	// If the target accepts property change listeners, register
	// ourselves
	try {
		Class[] args = { PropertyChangeListener.class };
	    	Method registerMethod = 
		    targ.getClass().getMethod("addPropertyChangeListener", 
		    args);
		Object[] me = { this };
		registerMethod.invoke(targ, me);
	}
	catch(Exception e){}

        try {
	    BeanInfo bi = Introspector.getBeanInfo(target.getClass());

	    // Short-circuit if the bean has a customizer
	    BeanDescriptor bd = bi.getBeanDescriptor();
	    targetHasCustomizer = (bd.getCustomizerClass() != null);
	    if (targetHasCustomizer) {
		setLayout(new BorderLayout());
		add(new Label(" "));
		doLayout();
		return;
	    }
	    properties = bi.getPropertyDescriptors();
	} catch (IntrospectionException ex) {
	    error("PropertyPanel: Couldn't introspect", ex);
	    return;
	}

	setLayout(null);

	editors = new PropertyEditor[properties.length];
	values = new Object[properties.length];
	views = new Component[properties.length];
	labels = new Label[properties.length];

/*
	labelWidth = 100;
	viewWidth = 120;
	height = 35;
*/
	labelWidth = 0;
	viewWidth = 0;
	height = 0;

	// PLI Do a bone-headed sort
	Vector sorted = new Vector(properties.length);
	for (int i = 0; i < properties.length; i++) {
	    int j = 0;
	    for(j = 0; j < sorted.size() && 
		properties[i].getDisplayName().compareTo(
		((PropertyDescriptor)sorted.elementAt(j)).getDisplayName()) > 0;
		j++);
	    sorted.insertElementAt(properties[i], j);	
	}
	sorted.copyInto(properties);

	propertyMap = new Hashtable(properties.length);

	for (int i = 0; i < properties.length; i++) {

	    // Don't display hidden or expert properties.
	    if (properties[i].isHidden() || properties[i].isExpert()) {
		continue;
	    }

	    String name = properties[i].getDisplayName();
	    Class type = properties[i].getPropertyType();
	    Method getter = properties[i].getReadMethod();
	    Method setter = properties[i].getWriteMethod();

	    propertyMap.put(name, new Integer(i));

            // PLI: Hack up a slightly better looking display name
            // if the display name was the same as the prop name
            // and the name isn't something in all caps (like "URL").
            if (name.equals(properties[i].getName()) &&
                (! name.toUpperCase().equals(name))) {
                StringBuffer buf = new StringBuffer(name);
 
                // Capitalize the first character
                if (Character.isLowerCase(buf.charAt(0))) {
                    buf.setCharAt(0, Character.toUpperCase(buf.charAt(0)));
                }
 
                // Add Spaces before any other capital letters.
                int len = name.length();
                for (int ch = 1; ch < len; ch++) {
                    if (Character.isUpperCase(buf.charAt(ch))) {
                        buf.insert(ch, ' ');
                        ch++;
                        len++;
                    }
                }
                name = buf.toString();
            }


	    // Only display read/write properties.
	    if (getter == null || setter == null) {
		continue;
	    }
	
	    Component view = null;

	    try {
		Object args[] = { };
		Object value = getter.invoke(target, args);
	        values[i] = value;

	        PropertyEditor editor = null;
	        Class pec = properties[i].getPropertyEditorClass();
		if (pec != null) {
		    try {
			editor = (PropertyEditor)pec.newInstance();
		    } catch (Exception ex) {
			// Drop through.
		    }
		}
		if (editor == null) {
		    editor = PropertyEditorManager.findEditor(type);
		}
	        editors[i] = editor;

	        // If we can't edit this component, skip it.
	        if (editor == null) {
		    continue;
	        }

	        editor.setValue(value);
	        editor.addPropertyChangeListener(this);

		// Now figure out how to display it...
		if (editor.isPaintable() && editor.supportsCustomEditor()) {
                    // These pop up dialogs, so we need a frame.
                    Component frame = this;
                    while ((frame != null) && (! (frame instanceof Frame))) {
                      frame = frame.getParent();
                    }
		    view = new PropertyCanvas((Frame)frame, editor);
		} 
                else if (editor.getTags() != null) {
		    view = new PropertySelector(editor);
		} else if (editor.getAsText() != null) {
		    String init = editor.getAsText();
		    view = new PropertyText(editor, true);
		} else {
		    System.err.println("Skipping non-displayable property " + name);
		    continue;
		}

	    } catch (Exception ex) {
		System.err.println("Skipping property " + name + " : " + ex);
		ex.printStackTrace();
		continue;
	    }
	    labels[i] = new Label(name, Label.RIGHT);
	    add(labels[i]);

	    views[i] = view;
	    add(views[i]);
	}
	if (getParent() != null) {
	    findSizes();
	    resize(width,height);
	    doLayout();
	}
	processEvents = true;
    }

    private void findSizes() {
	
	if (labelWidth != 0)
	    return;

	for (int i = 0; i < properties.length; i++) {
	    if (labels[i] == null || views[i] == null) {
		continue;
	    }
	    int w = labels[i].preferredSize().width;
	    if (w > labelWidth) {
		labelWidth = w;
	    }

	    w = views[i].preferredSize().width;
	    if (w > viewWidth) {
		viewWidth = w;
	    }
	    int h = views[i].preferredSize().height;
	    if (h < 30) {
		h = 30;
	    }

	    height += (h + hpad);
	}

	width = labelWidth + viewWidth + 10;
	if (width > 300) {
	    width = 300;
	}
    }


    public void layout() {

	if (labelWidth != 0)
	    return;
	findSizes();

	int y = insets().top + 10;

	if (views == null || labels == null) {
	    return;
	}

	for (int i = 0; i < properties.length; i++) {
	    if (labels[i] == null || views[i] == null) {
		continue;
	    }

	    labels[i].setBounds(0, y+5, labelWidth-8, 25);

	    int h = views[i].preferredSize().height;
	    if (h < 30) {
		h = 30;
	    }
	    int w = views[i].preferredSize().width;
	    if (w < 105) {
		w = 105;
	    }
	    if (w > viewWidth) {
		w = viewWidth;
	    }
	    views[i].setBounds(labelWidth, y, viewWidth, h);
	    y += (h + hpad);
	}

	y += 5 + insets().bottom;
    }

    synchronized void setCustomizer(Customizer c) {
	if (c != null) {
	    c.addPropertyChangeListener(this);
	}
    }

    public boolean handleEvent(Event evt) {
	if (evt.id == Event.ACTION_EVENT) {
	    Component parent = getParent();
	    if (parent != null) {
	        parent.postEvent(new Event(this, Event.MOUSE_DOWN, null));
	    }
	} else if (evt.id == Event.WINDOW_DESTROY) {
	    System.exit(0);
	}
	return (false);
    }

    // Ooops -- big problem: We only get events generated from within
    // the target when we are "pointed at" it.
    public synchronized void propertyChange(PropertyChangeEvent evt) {

	PropertyDescriptor property = null;
	Object value = null;


	int i = 0;

	if (!processEvents) {
	    return;
	}
	if (evt.getSource() == target) {
//System.out.println(" Event from target: " + target);
	    try {
	        property = new PropertyDescriptor(evt.getPropertyName(),
		    target.getClass());
		value = evt.getNewValue();

		Integer idx = 
		    (Integer) propertyMap.get(property.getDisplayName());
		if (idx == null)
		    return;

		i = idx.intValue();

		Object editorsValue = editors[i].getValue();
		if ((value == values[i]) || (value == editorsValue)
		   || ((value != null) && (editorsValue != null) && 
			value.equals(editorsValue))
		   || ((value != null) && (values[i] != null) && 
			value.equals(values[i])))
		    return;

		values[i] = value;
		editors[i].setValue(value);
	    	if (views[i] != null) {
		    views[i].repaint();
	    	}
	    }
	    catch (IntrospectionException e) {
		return;
	    }
	}
	else if (evt.getSource() instanceof PropertyEditor) {
//System.out.println(" Event from editor: " + evt.getSource());
	    PropertyEditor editor = (PropertyEditor) evt.getSource();
	    for (i = 0 ; i < editors.length; i++) {
	        if (editors[i] == editor) {
		    property = properties[i];
	            value = editor.getValue();
		    break;
		}
	    }

	    if (value == values[i])
		return;

	    Method setter = property.getWriteMethod();
	    Method getter = null;
	    // PLI -- trying to fix thrashing/recursion problem
	    // by NOT setting the property if it appears to be
	    // the same.
	    try {
		getter = property.getReadMethod();
		if (getter != null) {
		    Object args[] = {};
		    Object current = getter.invoke(target, args);
		    if (((current != null) && (value != null) && 
			current.equals(value)) || (current == value))
			return;
		}
	    } 
	    catch (Exception exe) {
		exe.printStackTrace();
	    }
		    
	    try {
	        Object args[] = { value };
	        args[0] = value;
	        setter.invoke(target, args);
		values[i] = value; // PLI
	    } catch (java.lang.reflect.InvocationTargetException ex) {
	      if (ex.getTargetException() instanceof PropertyVetoException) {
		  warning("Vetoed; reason is: " + 
			ex.getTargetException().getMessage());
	      }
	      else
	        error("InvocationTargetException while updating " + 
			property.getName(), 
			(Exception)ex.getTargetException());
	    } catch (Exception ex) {
	        error("Unexpected exception while updating " + 
			property.getName(), ex);
	    }
	}
	if (views[i] != null && views[i] instanceof PropertyCanvas) {
	    views[i].repaint();
        }
	
	if (property != null) {
	    if (box == null)
		box = BeanBox.findBeanBox(this);
	    if ((box != null) && (box instanceof QuarryBox)) 
		((QuarryBox)box).notifyParticipants(NuggetMessage.MODIFY,
                    new NuggetChangeInfo(target, property.getName(), value));
//System.out.println("--> NULL BEANBOX");

//System.out.println("--> Sending: " + property + " with value: " + value); 
	}

	// Now re-read all the properties and update the editors
	// for any other properties that have changed.
	for (i = 0; i < properties.length; i++) {
	    Object o;
	    try {
	        Method getter = properties[i].getReadMethod();
	        Object args[] = { };
	        o = getter.invoke(target, args);
	    } catch (Exception ex) {
		o = null;
		continue;
	    }
	    if ((o != null) && (o == values[i] || o.equals(values[i]))) {
	        // The property is equal to its old value.
		continue;
	    }
	    values[i] = o;
	    // Make sure we have an editor for this property...
	    if (editors[i] == null) {
		continue;
	    }
	    // The property has changed!  Update the editor.

// PLI: This is a problem -- it seems to cause infinite recursion _sometimes_
	    editors[i].setValue(o);

	    if (views[i] != null) {
		views[i].repaint();
	    }
	}

	// Make sure the target bean gets repainted.
	if (Beans.isInstanceOf(target, Component.class)) {
	    ((Component)(Beans.getInstanceOf(target, 
		Component.class))).repaint();
	}
    }


    private void warning(String s) {
// PLI:	new ErrorDialog(this, "Warning: " + s);

//System.err.println("Warning: " + s);

    }

    public Dimension getPreferredSize() {
	return new Dimension(width, height);
    }


    //----------------------------------------------------------------------
    // Log an error.

    private void error(String message, Exception ex) {
	String mess = message + ":\n" + ex;
	System.err.println(message);
	ex.printStackTrace();
	// Popup an ErrorDialog with the given error message.
//PLI:	new ErrorDialog(this, mess);

    }

    //----------------------------------------------------------------------
    private Object target;
    private PropertyDescriptor properties[];
    private PropertyEditor editors[];
    private Object values[];
    private Component views[];
    private Label labels[];

    private boolean processEvents;
    private static int hpad = 4;
    private int labelWidth = 100;
    private int viewWidth = 110;

    private int width = 100;
    private int height = 100;
}
