package sieve.quarry;

import java.awt.*;
import java.io.Serializable;
import java.beans.*;
import java.util.*;
import sieve.sluice.*;
import sieve.crew.*;

public class TableViewPanel extends Panel implements TableView,
    PropertyChangeListener, RemotePropertyChangeListener, 
    TableViewListener {
    TableView             tableView	    = null;
    TableView             sourceView	    = null;
    protected Font        f       = null;
    protected FontMetrics fm      = null;
    protected int         height  = 0;
    protected int         width   = 0;
    protected int         baseline= 0;

    protected String 	  label   = null;
    protected Label	  header  = null;

    protected Component	  customizer	= null;

    public TableViewPanel(String label_UNUSED, TableView tableView) {
	this.tableView = tableView;
	this.label = findLabel(tableView);
        setName(label);

	if (tableView instanceof Table)
	    addCustomizer(tableView);
   	else {
	    f = new Font("Helvetica", Font.PLAIN, 10);
            setFont(f);
            fm = getFontMetrics(f);
            baseline = fm.getMaxAscent() + 2;
            height = baseline + fm.getMaxDescent() + 2;
            width = fm.stringWidth(label) + 6;
            resize(width,height);
	}

	//	addContainerListener(new WrapperListener());
    }

    public void paint(Graphics g) {
        debug("paint");
	if (customizer == null) {
	    int width = fm.stringWidth(label) + 6;
            resize(width,height);
	    g.drawString(label, 3, baseline);
	}
	else
	    super.paint(g);
    }

    public TableView getTableView() {
    	return tableView;
    }

    // Make this look like a TableView...

    public TableView table() throws NoSourceException {
	return tableView.table();
    }

    public TableDescriptor getTableDescriptor() throws NoSourceException {
	return tableView.getTableDescriptor();
    }

    public String[] getFieldNames() throws NoSourceException {
	return tableView.getFieldNames();
    }

    public FieldDescriptor getFieldDescriptor(String field) throws 
        InterchangeException {
	return tableView.getFieldDescriptor(field);
    }

    /**
     * Returns number of records in the view.
     */
    public int getRecordCount() throws NoSourceException {
	return tableView.getRecordCount();
    }

    /**
     * Returns no more than <CODE>numberOfRecords</CODE> records starting
     * at record number <CODE>startingAt</CODE> from 
     * the fields specified in <CODE>fields</CODE>.
     */
    public Object[][] get(String[] fields, int startingAt, 
	int numberOfRecords) throws InterchangeException {
	return tableView.get(fields, startingAt, numberOfRecords);
    }

    /**
     * Returns all records starting from record number <CODE>startingAt</CODE>
     * from the fields specified in <CODE>fields</CODE>.
     */
    public Object[][] get(String[] fields, int startingAt) 
	throws InterchangeException { 
	return tableView.get(fields, startingAt); 
    } 

    /* -------------  Event-related methods ------------- */
    public void addTableViewListener(TableViewListener listener) {
	tableView.addTableViewListener(listener);
    }

    public void removeTableViewListener(TableViewListener listener) {
	tableView.removeTableViewListener(listener);
    }

    public void tableViewChanged(TableViewEvent event) {
	tableView.tableViewChanged(event);
    }

    /* -------------  Methods for link support ------------- */
    public void addSourceView(TableView sourceView) throws 
	TooManySourcesException {
        this.sourceView = sourceView;
	tableView.addSourceView(sourceView);
	addCustomizer(sourceView);
    }

    private String findLabel(TableView tv) {
	String name = tv.getClass().getName();
	try {
	    BeanInfo bi = Introspector.getBeanInfo(tv.getClass());
            BeanDescriptor bd = bi.getBeanDescriptor();
	    name = bd.getDisplayName();
        }
	catch (Exception e) {
	    e.printStackTrace();
	}
	return name;
    }

    public void setName(String name) {
	super.setName(name);
	label = name;
	if (header != null)
	    header.setText(label);
    }

    private void addCustomizer(TableView sourceView) {
	setLayout(new BorderLayout());
	try {
	    BeanInfo bi = Introspector.getBeanInfo(this.tableView.getClass());
            BeanDescriptor bd = bi.getBeanDescriptor();
            Class customizerClass = bd.getCustomizerClass();
	    header = new Label(label);
	    header.setAlignment(Label.CENTER);
            add("North", header);

	    if (customizerClass == null) {
		customizer = new PropertyPanel(this.tableView);
	    }
	    else {
		customizer = (Component)customizerClass.newInstance();

		/*
		 * The customizer must listen to the source. It may also
		 * choose to listen to the thing it is actually 
		 * customizing, though that is generally not a good idea
		 * as the handling of events generated by the customizer's
		 * actions can get tricky.
		 */
		if (customizer instanceof TableViewListener) {
		    sourceView.addTableViewListener(
			(TableViewListener)customizer);
		}
		((Customizer)customizer).setObject(this.tableView);
	        ((Customizer)customizer).addPropertyChangeListener(this);
	    }
	    add("Center", customizer);
	    invalidate();
	    Dimension size = customizer.getPreferredSize();
	    resize(size.width, size.height);
	    if (getParent() != null)
	        getParent().layout();
	}
	catch (Exception e) {
	    e.printStackTrace();
	}
    }

    private void removeCustomizer() {
	if (customizer != null) {
	    remove(customizer);
	    remove(header);
	    if (customizer instanceof TableViewListener) {
	        this.tableView.removeTableViewListener(
		    (TableViewListener)customizer);
	    }
	    customizer = null;
	}
    }

   public void removeSourceView(TableView tableView) {
     this.sourceView = null;
     this.tableView.removeSourceView(tableView);
     removeCustomizer();
   }

  //debugging
  public void print(Graphics g) {
    debug("print");
    super.print(g);
    paint(g);
  }
  /*
  public void printAll(Graphics g) {
    debug("printAll");
    super.printAll(g);
  }
  */

  private static final boolean debug = false;
  private void debug(String message){
    if (debug)
      System.err.println(getClass().getName()+": "+message);
  }

    public Dimension getPreferredSize() {
    	if (customizer != null)
	    return super.getPreferredSize();
	else
	    return new Dimension(width, height);
    }

    public synchronized void remotePropertyChange(PropertyChangeEvent evt) {
      if (customizer != null) {
        if (customizer instanceof PropertyChangeListener) 
          ((PropertyChangeListener)customizer).propertyChange(evt);
        else {
          removeCustomizer();
          if (sourceView != null)
            addCustomizer(sourceView);
        }
      } 
    }

    public void propertyChange(PropertyChangeEvent evt) {
      BeanBox box = BeanBox.findBeanBox(this);
      if ((box != null) && (box instanceof QuarryBox)) {
         ((QuarryBox)box).notifyParticipants(NuggetMessage.MODIFY,
           new NuggetChangeInfo(tableView, evt.getPropertyName(), 
           evt.getNewValue()));
      }
    }

}

