
package sieve.quarry;

/**
 * The ToolBox is a panel that shows icons and ID strings
 * for the available JavaBeans in the current BeanBoxFrame.
 */

import java.beans.*;
import java.awt.*;
import java.io.*;
import java.util.Vector;
import java.awt.event.*;
import java.util.*;
import java.lang.reflect.*;
import sieve.quarry.*;


// We currently keep all the contents of the ToolBox in a separate
// ToolBoxPanel object, to workaround win32 bug #4028421   KGH 1/28/97

// For now I'm adding the toolbox panel that we used for the LiNC demos.
// This needs to be upgraded in various ways. You should be able to load
// jar files into it. It should support double-clicking to select a module
// for placement. There should also be a way to cancel a selection.
// We should also consider if/how we want to support icons in the new panel.
public class ToolBox extends Panel {
    private ToolBoxPanel panel;
    private BeanBoxFrame bbf;

    ToolBox(int x, int y, BeanBoxFrame bbf) {
	//super("ToolBox");
	this.bbf = bbf;

	setBackground(Color.lightGray);
	//PLI setFont(new Font("Dialog", Font.PLAIN, 10));

	//panel = new ToolBoxPanel(this, bbf);
	panel = new ToolBoxPanel(null , bbf);

	setLayout(new BorderLayout());

	add("Center", new DemoToolPanel(bbf));

	//move(x, y);
	setSize(new Dimension(400, 200));
	//show();
    }

    Vector getLoadedJarInfo() {
	return panel.getLoadedJarInfo();
    }

    void addBeansInJar(String jarFile) {
	panel.addBeansInJar(jarFile);
    }

    public boolean handleEvent(Event evt) {
	if (evt.id == Event.WINDOW_DESTROY) {
	    System.exit(0);
	}
	return true;
    }

}


class DemoToolPanel extends Panel {

    ListPanel listPanel = null;
    BeanBoxFrame frame = null;

    public DemoToolPanel(BeanBoxFrame bbf) {
	this.frame = bbf;
	setLayout(new BorderLayout());
	//add("North", new Label("Tools", Label.CENTER));
	Button button = new Button(" ");
	add("South", button);
	listPanel = new ListPanel(this, button, frame);
	add("Center", listPanel);
	

	ToolSet simulations = new ToolSet("Simulations");
	simulations.addBeanClass("sieve.phys.lwblockandplane.BlockAndPlane");
	simulations.addBeanClass("sieve.phys.blockandplane.BlockAndPlane");

	ToolSet whiteboard = new ToolSet("Annotation Tools");
	whiteboard.addBeanClass("sieve.whiteboard.PostItNote");
	whiteboard.addBeanClass("sieve.whiteboard.Box");
	whiteboard.addBeanClass("sieve.whiteboard.Oval");
	whiteboard.addBeanClass("sieve.whiteboard.Label");
	whiteboard.addBeanClass("sieve.whiteboard.URLImage");


	ToolSet selectors = new ToolSet("Filters");
	selectors.addBeanClass("sieve.sluice.views.TableSelectionView");
	selectors.addBeanClass("sieve.sluice.views.FrequencyTableView");
	selectors.addBeanClass("sieve.sluice.views.CriteriaView");
	selectors.addBeanClass("sieve.sluice.views.FieldNameSelectionView");
	selectors.addBeanClass("sieve.sluice.views.ColorChangeView");
	selectors.addBeanClass("sieve.sluice.views.PolyInterpolatedView");

	ToolSet viewers = new ToolSet("Visualizations");
	viewers.addBeanClass("sieve.nuggets.LineGraph.LineGraph");
	viewers.addBeanClass("sieve.nuggets.TextTable");
	viewers.addBeanClass("sieve.nuggets.AttributeViewer");
	viewers.addBeanClass("sieve.nuggets.BarChart");
	viewers.addBeanClass("sieve.nuggets.Histogram");
	viewers.addBeanClass("sieve.nuggets.Graph");

	ToolSet other = new ToolSet("Other Tools");
	other.addBeanClass("sieve.sluice.views.CacheView");
	other.addBeanClass("sieve.sluice.weblog.TableImpl");

	ToolSet sosci = new ToolSet("SocSci Tools");
	sosci.addBeanClass("sieve.sluice.codebook.CodeBookTable");
	sosci.addBeanClass("sieve.sluice.gss.GSSTable");
	sosci.addBeanClass("sieve.nuggets.StretchableHistogram");

	ToolSet circuit = new ToolSet("Circuit Components");
	circuit.addBeanClass("sieve.phys.circuit.CircuitComponent");
	
	listPanel.addToolSet(sosci);
	listPanel.addToolSet(simulations);
	listPanel.addToolSet(circuit);
	listPanel.addToolSet(selectors);
	listPanel.addToolSet(viewers);
	listPanel.addToolSet(whiteboard);
	listPanel.addToolSet(other);

	//listPanel.selectToolSet("SocSci Tools");
	listPanel.selectToolSet("Simulations");
	listPanel.selectTool(0);
    }
    
}

class ListPanel extends Panel implements ActionListener, Runnable {
    DemoToolPanel toolPanel = null;
    Hashtable setMap = null;
    Hashtable setIdxMap = null;
    String lastSet = null;
    String currentTool = null;

    List sets = null;
    List tools = null;
    Button button = null;

    Thread life = null;
    Object pendingBean = null;
    BeanBoxFrame frame = null;

    public ListPanel(DemoToolPanel toolPanel, Button button, BeanBoxFrame bbf) {
	this.toolPanel = toolPanel;
	this.button = button;
	this.frame = bbf;
	button.addActionListener(this);
	setLayout(new GridLayout(1, 2, 2, 2));
	sets = new List();
	sets.addItemListener(new SetListListener(this));
	tools = new List();
	ToolListListener toolListListener = new ToolListListener(this);
	tools.addItemListener(toolListListener);
	tools.addActionListener(toolListListener);
	add(sets);
	add(tools);
	setMap = new Hashtable();
	setIdxMap = new Hashtable();

	life = new Thread(this);
	life.start();
    }

    public void addToolSet(ToolSet ts) {
	setMap.put(ts.getName(), ts);
	setIdxMap.put(ts.getName(), new Integer(sets.getItemCount()));
	sets.add(ts.getName());
    }

    public synchronized void selectToolSet(String setName) {
	ToolSet ts = (ToolSet)setMap.get(setName);

	if (setName.equals(lastSet))
	    return;

	if (ts != null ) {
	    lastSet = setName;
	    int idx = ((Integer)setIdxMap.get(setName)).intValue();
	    if (idx != sets.getSelectedIndex())
		sets.select(idx);
	    String[] toolNames = ts.getLabels();
	    tools.removeAll();
	    for (int t = 0; t < toolNames.length; t++)
		tools.addItem(toolNames[t]);
	    tools.select(0);
	    selectTool(0);
	}
    }

    public synchronized void selectToolSet(int idx) {
	selectToolSet(sets.getItem(idx));
    }

    public synchronized void selectTool(int idx) {
	currentTool = tools.getItem(idx);
	button.setLabel("Add New \"" + currentTool + "\"");
    }

    public void actionPerformed(ActionEvent e) {
	if (button.getLabel().startsWith("Click"))
	    return;
	button.setLabel("Click on workspace to add component");
	button.disable();
	sets.disable();
	tools.disable();

 
        // Change the cursor to indicate an "insert".
	setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
 
	// Find the current tool
	ToolSet ts = (ToolSet)setMap.get(lastSet);
	synchronized(this) {
	    pendingBean = ts.getBeanLabeled(currentTool);
	    notify();
	}
    }

    public synchronized void run() {
	// We run an internal thread to handle insertions, one at a time.
        for (;;) {
            Object bean;
            synchronized (this) {
                // Wait for a bean to be ready for insertion.
                while (pendingBean == null) {
                    try {
                        wait();
                    } catch (InterruptedException ix) {
                        System.err.println("ToolBox.run: unexpected interrupt");
                    }
                }
                bean = pendingBean;
		pendingBean = null;
            }

            // Find our the current BeanBox.
            BeanBox box = frame.getCurrentBeanBox();

            // do the insertion.
            box.doInsert(bean);
 
	    button.setLabel("Add New \"" + currentTool + "\"");
	    button.enable();
	    sets.enable();
	    tools.enable();
	    setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	}
    }
}


class SetListListener implements ItemListener {
    ListPanel listPanel = null;

    public SetListListener(ListPanel listPanel) {
	this.listPanel = listPanel;
    }

    public void itemStateChanged(ItemEvent e) {
	if (e.getStateChange() == ItemEvent.SELECTED) {
	    listPanel.selectToolSet(((Integer)e.getItem()).intValue());
	}
    }
}

class ToolListListener implements ItemListener, ActionListener {
    ListPanel listPanel = null;

    public ToolListListener(ListPanel listPanel) {
	this.listPanel = listPanel;
    }

    public void itemStateChanged(ItemEvent e) {
	if (e.getStateChange() == ItemEvent.SELECTED) {
	    listPanel.selectTool(((Integer)e.getItem()).intValue());
	}
    }

    // For double-clicks
    public void actionPerformed(ActionEvent e) {
	listPanel.actionPerformed(e);
    }
}

// A set of tools, e.g., visualizations, filters, or simulations
class ToolSet {
    Vector beanLabels = new Vector();
    Vector beanNames = new Vector();
    Vector beanIcons = new Vector();
    Vector beanJars = new Vector();
    Hashtable labelMap = new Hashtable();
    String name = null;

    public ToolSet(String name) {
	this.name = name;
    }

    public String getName() {
	return name;
    }

    public String[] getLabels() {
	String[] labels = new String[beanLabels.size()];
	beanLabels.copyInto(labels);
	return labels;
    }

    public void addBeanClass(String beanClassName) {
        Class beanClass;
        try {
            beanClass = Class.forName(beanClassName);
        } catch (Exception ex) {
            System.err.println("Toolbox: couldn't instantiate " +
                                                             beanClassName);
            return;
        }
        BeanInfo bi;
        try {
            bi = Introspector.getBeanInfo(beanClass);
        } catch (Exception ex) {
            System.err.println("Toolbox: couldn't find BeanInfo for " +
                                                                beanClassName);
            ex.printStackTrace();
            return;
        }

        String label = bi.getBeanDescriptor().getDisplayName();
        label = getUniqueName(beanLabels, label);
	beanLabels.addElement(label);

        Image img = bi.getIcon(BeanInfo.ICON_COLOR_16x16);
	labelMap.put(label, beanClassName);
        beanNames.addElement(beanClassName);
        beanIcons.addElement(img);
        //beanJars.addElement(null);
    }

    public synchronized Object getBeanLabeled(String label) {
	String beanName = (String)labelMap.get(label);
	if (beanName != null) {
            Object bean = null;
 
            if (beanName.equals("sieve.quarry.BeanBox")) {
		// should pass in BeanBoxFrame.atts -Bo
                bean = new BeanBox(new java.util.Hashtable());  
            } 
	    else {
                try {
                   Class cl = Class.forName(beanName);
                   bean = cl.newInstance();
	           return bean;
                } 
		catch (Exception exe) {
	 	    exe.printStackTrace();
                }
            }
	}
	return null;
    }

    String getUniqueName(Vector v, String s) {
        if (! v.contains(s)) {
            return s;
        }
        int count = 2;
        while (v.contains(s+"<"+count+">")) {
            count += 1;
        }
	return s+"<"+count+">";
    }
}

// -------------------- End of current code ---------------------

// Not currently used. This can be removed once all of its functionality
// exists in the new ToolBoxPanel.
class ToolBoxPanel extends Panel implements Runnable {

    ToolBoxPanel(Frame frame, BeanBoxFrame bbf) {
	this.frame = frame;
	this.bbf = bbf;
if (frame == null)
return;
	setLayout(null);
	setBackground(Color.lightGray);
	//PLI setFont(new Font("Dialog", Font.PLAIN, 10));


// PLI
addBeanClass("sieve.sluice.weblog.TableImpl");
addBeanClass("sieve.sluice.codebook.CodeBookTable");
addBeanClass("sieve.sluice.gss.GSSTable");
addBeanClass("sieve.nuggets.BarChart");
addBeanClass("sieve.nuggets.Histogram");
addBeanClass("sieve.nuggets.StretchableHistogram");
addBeanClass("sieve.nuggets.Graph");
addBeanClass("sieve.nuggets.ScatterPlot");
addBeanClass("sieve.sluice.views.CriteriaView");
addBeanClass("sieve.sluice.views.FieldNameSelectionView");
addBeanClass("sieve.sluice.views.FrequencyTableView");
addBeanClass("sieve.sluice.views.CacheView");
addBeanClass("sieve.sluice.views.ColorChangeView");
addBeanClass("sieve.sluice.views.TableSelectionView");
addBeanClass("sieve.whiteboard.PostItNote");
addBeanClass("sieve.whiteboard.Box");
addBeanClass("sieve.whiteboard.Oval");
addBeanClass("sieve.whiteboard.Label");
addBeanClass("sieve.nuggets.LineGraph.LineGraph");
addBeanClass("sieve.phys.lwblockandplane.BlockAndPlane");
addBeanClass("sieve.nuggets.TextTable");
addBeanClass("sieve.nuggets.AttributeViewer");

/*
	Vector jarNames = getJarNames();	
	for (int i = 0; i < jarNames.size(); i++) {
	    String name = (String)jarNames.elementAt(i);
	    addBeansInJar(name);
	}
*/

	insertThread = new Thread(this);
	insertThread.start();
    }

    // keep it with package scope for the inner class' sake
    void addWithUniqueName(Vector v, String s) {
	if (! v.contains(s)) {
	    debug("adding "+s);
	    v.addElement(s);
	    return;
	}
	int count = 2;
	while (v.contains(s+"<"+count+">")) {
	    count += 1;
	}
	debug("adding "+s+"<"+count+">");
	v.addElement(s+"<"+count+">");
	return;
    }

    private class Helper implements DoOnBean {
	public void action(JarInfo ji, BeanInfo bi, String beanName) {
	    String label;
	    Image img = null;

	    BeanDescriptor bd = bi.getBeanDescriptor();
	    debug("Helper.action; beanName: "+beanName);
	    debug("  beanName:  "+beanName);
	    debug("  getName(): "+bd.getName());
	    debug("  getBeanClass(): "+bd.getBeanClass());

	    Class beanClass = bd.getBeanClass();
	    if (beanName.equals(beanClass.getName())) {
		label = bi.getBeanDescriptor().getDisplayName();
		img = bi.getIcon(BeanInfo.ICON_COLOR_16x16);
	    } else {
	        label = beanName;
		int ix = beanName.lastIndexOf('.');
		if (ix >= 0) {
		    label = beanName.substring(ix+1);
		}
		img = null;
	    }
	    addWithUniqueName(beanLabels, label);
	    beanNames.addElement(beanName);
	    beanIcons.addElement(img);
	    beanJars.addElement(ji);
	}
	public void error(String msg) {
	    ToolBoxPanel.this.error(msg);
	}
	public void error(String msg, Exception ex) {
	    ToolBoxPanel.this.error(msg, ex);
	}
    }

    private Helper helper = new Helper();

    /**
     * Add all the beans in a Jar file into the ToolBox
     */
    synchronized void addBeansInJar(String jarFile) {
	JarLoader.loadJarDoOnBean(jarFile, helper);
	doLayout();
    }

    /**
     * Special action.
     *
     * Currently only used to register the BeanBox
     */
    synchronized void addBeanClass(String beanClassName) {
	Class beanClass;
	try {
	    beanClass = Class.forName(beanClassName);
	} catch (Exception ex) {
	    System.err.println("Toolbox: couldn't instantiate " +
							     beanClassName);
	    return;
	}
	BeanInfo bi;
	try {
	    bi = Introspector.getBeanInfo(beanClass);
	} catch (Exception ex) {
	    System.err.println("Toolbox: couldn't find BeanInfo for " + 
								beanClassName);
	    ex.printStackTrace();
	    return;
	}
	String label = bi.getBeanDescriptor().getDisplayName();
	Image img = bi.getIcon(BeanInfo.ICON_COLOR_16x16);
	addWithUniqueName(beanLabels, label);
	beanNames.addElement(beanClassName);
	beanIcons.addElement(img);
	beanJars.addElement(null);
	doLayout();
    }

    public void layout() {
	topPad = frame.insets().top;
	sidePad = frame.insets().left;

	int height = topPad + (rowHeight * beanLabels.size()) + 6 + frame.insets().bottom;
        if (height != size().height) {
	    resize(175, height);
	    frame.resize(175, height);
	}
    }

    public void paint(Graphics g) {
	for (int i = 0; i < beanLabels.size(); i++) {
	    String name = (String)beanLabels.elementAt(i);
	    g.drawString(name, sidePad + 21, topPad + (rowHeight*i) + rowHeight-4);
	    Image img = (Image)beanIcons.elementAt(i);
	    if (img != null) {
	        boolean status =
		    g.drawImage(img, sidePad+2, topPad + (rowHeight*i) + 2, 16, 16, this);
	    }
	}
    }

    public boolean handleEvent(Event evt) {
	if (evt.id == Event.MOUSE_DOWN) {

	    int row = (evt.y - topPad)/rowHeight;

	    Object bean;

	    String beanName = (String) beanNames.elementAt(row);
	    pendingBeanLabel = (String) beanLabels.elementAt(row);
	    if (beanName.equals("sieve.quarry.BeanBox")) {
	      bean = new BeanBox(new java.util.Hashtable());  // should pass in BeanBoxFrame.atts -Bo
	    } else {
		JarInfo ji = (JarInfo) beanJars.elementAt(row);
System.out.println("Got JarInfo: " + ji);
		try {
		    bean = ji.getInstance(beanName);
System.out.println("Got bean from JarInfo: " + bean);
		} catch (Exception ex) {
                    // PLI -- This should probably use the Beans instantiation
		    // method?
                    try {
System.out.println("Trying Class.forName");
                        Class cl = Class.forName(beanName);
                        bean = cl.newInstance();
		    } catch (Exception exe) {
		        error("instantion of a new bean failed", exe);
		        return true;
                    }
		}
	    }

	    // Wakeup the insert thread to insert the new bean.
	    synchronized (this) {
		pendingBean = bean;
	        notify();
	    }

	} else if (evt.id == Event.WINDOW_DESTROY) {
	    System.exit(0);
	}
	return true;
    }

    public void run() {
	// We run an internal thread to handle insertions, one at a time.
	for (;;) {
	    Object bean;
	    synchronized (this) {
		// Wait for a bean to be ready for insertion.
		while (pendingBean == null) {
		    try {
			wait();
		    } catch (InterruptedException ix) {
			System.err.println("ToolBox.run: unexpected interrupt");
		    }
		}
		bean = pendingBean;
	    }

	    // Figure our the current BeanBox.
	    BeanBox box = bbf.getCurrentBeanBox();

            // Change the cursor to indicate an "insert".
	    frame.setCursor(crosshairCursor);

	    // PLI
	    box.getFrame().showStatus("Click to add new \"" + 
		pendingBeanLabel + "\" module");

	    // do the insertion.
	    box.doInsert(bean);

	    // Clear any pending bean.  This will normally be the bean we
	    // just inserted, but we also want to clear any bogus insert
	    // that has been queued since we started.
	    pendingBean = null;

	    frame.setCursor(defaultCursor);
	}
    }

    /**
     * Provide information on all the loaded JAR files, regardless of origin.
     */
    public Vector getLoadedJarInfo() {
	return beanJars;
    }

    private static Vector getJarNames() {
	File cwd = new File(System.getProperty("user.dir"));
	File pwd = new File(cwd.getParent());
	File jars = new File(pwd, "jars");

	if (! jars.isDirectory()) {
	    System.err.println(jars+" is not a directory!!");
	}

	Vector result = new Vector();
	String names[];
	names = jars.list(new FileExtension(".jar"));
	for (int i=0; i<names.length; i++) {
	    result.addElement(jars.getPath() + File.separatorChar + names[i]);
	}
	names = jars.list(new FileExtension(".JAR"));
	for (int i=0; i<names.length; i++) {
	    result.addElement(jars.getPath() + File.separatorChar + names[i]);
	}

	return result;
    }

    public void error(String message, Exception ex) {
	String mess = message + ":\n" + ex;
	System.err.println(message);
	ex.printStackTrace();
	// Popup an ErrorDialog with the given error message.
	new ErrorDialog(frame, mess);

    }

    public void error(String message) {
	String mess = message;
	System.err.println(message);
	// Popup an ErrorDialog with the given error message.
	new ErrorDialog(frame, mess);

    }

    static  void debug(String msg) {
	if (debug) {
	    System.err.println("ToolBox:: "+msg);
	}
    }

    Vector beanLabels = new Vector();
    Vector beanNames = new Vector();
    Vector beanIcons = new Vector();
    Vector beanJars = new Vector();
    private int topPad = 0;
    private int sidePad = 0;
    private final static int rowHeight = 20;
    private Thread insertThread = null;
    private static boolean debug = false;
    private Object pendingBean;
    private String pendingBeanLabel;
    private Frame frame;
    private BeanBoxFrame bbf;

    private static Cursor crosshairCursor = Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR);
    private static Cursor defaultCursor = Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR);
}


// Auxiliary class

class FileExtension implements FilenameFilter {
    private String extension;

    public FileExtension(String ext) {
	this.extension = ext;
    }
    
    public boolean accept(File dir, String name) {
	return name.endsWith(extension);
    }
}
