package sieve.quarry;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.beans.*;
import java.lang.reflect.*;
import sieve.sluice.*;
import sieve.sluice.views.*;


public class WrapperApplet extends sieve.nuggets.TableViewApplet implements
    ActionListener {
    public static final String PARM_TAG             = "Tag";
    public static final String PARM_TABLEVIEW_CLASS = "TableViewClass";
    public static final String PARM_SOURCE          = "SourceView";
    public static final String PARM_BEHAVIOR        = "Behavior";
    public static final String BEHAVIOR_INVISIBLE   = "Invisible";
    public static final String BEHAVIOR_EMBEDDED    = "Embedded";
    public static final String BEHAVIOR_POP_UP      = "PopUp";
    public static final String PARM_POP_UP_BUTTON   = "ButtonName";

/* These aren't implemented yet...
    public static final String BEHAVIOR_EMBEDDED_WITH_EDIT    = 
	"EmbeddedWithEdit";
*/

    private boolean loaded = false;
    private Wrapper wrapper = null;
    private Component customizer = null;
    private String  behavior = null;
    private Frame   nuggetFrame = null;
    private String  tableViewClass = null;
    private String  tag = null;

    public synchronized void init() {
	tag = getParameter(PARM_TAG);
	tableViewClass = getParameter(PARM_TABLEVIEW_CLASS);
	if (tableViewClass != null) {
	    try {
		source = (TableView) 
		    Class.forName(tableViewClass).newInstance();
	    }
	    catch (Exception e) {
		e.printStackTrace();
		return;
	    }
	    behavior = getParameter(PARM_BEHAVIOR);
	    if (behavior == null)
		behavior = BEHAVIOR_EMBEDDED;

	    wrapper = new Wrapper(source);
	    source = (TableView)(wrapper.getBean());

	    if (behavior.equalsIgnoreCase(BEHAVIOR_EMBEDDED)) {
		Dimension size = getSize();
		wrapper.resize(new Dimension(size.width, size.height - 3));
	        add(wrapper);
	    }
	    else if (behavior.equalsIgnoreCase(BEHAVIOR_POP_UP)) {
	        String buttonName = getParameter(PARM_POP_UP_BUTTON);
		if (buttonName == null)
		    buttonName = "Launch Applet";
		Button launchButton = new Button(buttonName);
		launchButton.addActionListener(this);
	        nuggetFrame = new Frame();
	        nuggetFrame.add(wrapper);
	        nuggetFrame.resize(new Dimension(400, 400));
		add(launchButton);
	    }
	}
    }

    public void actionPerformed(ActionEvent e) {
	if (nuggetFrame != null) {
	    nuggetFrame.show();
	    nuggetFrame.toFront();
	}
    }

    public synchronized void start() {
	String sourceString = getParameter(PARM_SOURCE);
	if (sourceString != null) {
	    Applet app = getAppletContext().getApplet(sourceString);
	    int attempts = 0;
            if (app == null) {
                for (Enumeration e = getAppletContext().getApplets(); 
			e.hasMoreElements();) {
                    //System.out.println("----> " + 
		    //    ((WrapperApplet)e.nextElement()).tableViewClass);
		    Applet a = (Applet) e.nextElement();
		    if ((a instanceof WrapperApplet) && 
			sourceString.equals(((WrapperApplet)a).tag))
			app = (WrapperApplet)a;
                }
	    }
	    while ((app == null) && (attempts++ < 100)) {
		try {
System.out.println("Waiting for app. attempt: " + attempts);
		    wait(2000);
		}
		catch (InterruptedException e) {}
	        app = getAppletContext().getApplet(sourceString);
	    }
	    if ((app != null) && (app instanceof WrapperApplet)) {
		((WrapperApplet)app).waitToLoad();
		try {
		    source.addSourceView(((WrapperApplet)app).getTableView());
		    ((WrapperApplet)app).getTableView().addTableViewListener(source);
		     System.out.println("Attached "+ tableViewClass +
		        " to source applet: " + app + 
			" (" + sourceString + ")");
		}
		catch (InterchangeException e) {
		    e.printStackTrace();
		}
	    }
	    else 
		System.out.println("Bad source applet: " + app + 
		    " (" + sourceString +") for " + tableViewClass);
 	}
	else 
	    System.out.println("No source applet for " + tableViewClass);

    	Class tvClass;
    	Object target = source;
	if ((target instanceof TableViewPanel) && (wrapper != null))
		target = wrapper.getRawBean();
	tvClass = target.getClass();
        try {
            BeanInfo bi = Introspector.getBeanInfo(tvClass);
            Object args[] = new Object[1];
 
            PropertyDescriptor props[] = bi.getPropertyDescriptors();
            for (int i = 0; i < props.length; i++) {
                PropertyDescriptor pd = props[i];
	        String propName = pd.getName();
		String propVal = getParameter(propName.toLowerCase());
//System.out.println("PropName = " + propName + " value in parm: " + propVal + 
//" with type " + pd.getPropertyType());
                if (propVal != null) {
		    args[0] = null;
		    Class propType = pd.getPropertyType();
		    if (propType != null) {
			if (propType.equals( Class.forName("java.lang.Object")))
			    args[0] = propVal;
			else if (propType.equals(
			    Class.forName("java.lang.Boolean")) ||
			    propType.getName().equals("boolean"))
			    args[0] = new Boolean(propVal);
			else if (propType.equals(
			    Class.forName("java.lang.Integer")) ||
			    propType.getName().equals("int"))
			    args[0] = new Integer(propVal);
			else if (propType.equals(
			    Class.forName("java.lang.Long")) ||
			    propType.getName().equals("long"))
			    args[0] = new Long(propVal);
			else if (propType.equals(
			    Class.forName("java.lang.Double")) ||
			    propType.getName().equals("double"))
			    args[0] = new Double(propVal);
			else if (propType.equals(
			    Class.forName("java.lang.Float")) ||
			    propType.getName().equals("float"))
			    args[0] = new Float(propVal);
			else if (propType.equals(
			    Class.forName("java.lang.String")))
			    args[0] = propVal;
		    } 
		    if (args[0] != null) {
//System.out.println("Setting " + propName + " to: " + args[0]);
                        Method setter = pd.getWriteMethod();
                        setter.invoke(target, args);
		        if (source instanceof TableViewPanel) {
			    ((TableViewPanel)source).remotePropertyChange(
			        new PropertyChangeEvent(this, propName, null,
			        args[0]));
			}
		    }
                }
            }
        }
	//catch (java.io.FileNotFoundException fnfe) { }
	catch (Exception e) {
	    //e.printStackTrace();
	}
	loaded = true;
	notifyAll();
    }

    public synchronized void waitToLoad() {
	while (! loaded) {
	    try {
		wait();
	    }
	    catch (InterruptedException e) {
		e.printStackTrace();
	    }
	}
    }

    public synchronized TableView getTableView() {
	waitToLoad();
	return source;
    }

}

