package sieve.sluice;

import java.util.*;

/**
 * This is based roughly on java.beans.PropertyChangeSupport. See
 * beans spec for more information.
 */
public class TableViewEventSupport implements java.io.Serializable {
    protected	Vector	listeners	= null;

    public TableViewEventSupport() {
	listeners = new Vector();
    }

    public synchronized void 
	addTableViewListener(TableViewListener listener) {
	listeners.addElement(listener);
    }

    public synchronized void 
	removeTableViewListener(TableViewListener listener) {
	listeners.removeElement(listener);
    }

    /**
     * Propagates an event to all listeners. 
     */
    public void fireTableViewEvent(TableViewEvent event) {
	/*
	 * Freeze the state of the listeners before delivering the
	 * event.
	 */
	Vector	frozen = null;

	synchronized (this) {
	    frozen = (Vector) listeners.clone();
	}

	/*
	 * Start a separate thread to send the event to each
	 * listener. (Threads are cheap, right??) -- Unfortunately,
	 * all hell breaks loose when Sieve is being driven
	 * collaboratively if we do this ... :-(
	 */
	for (int i = 0; i < frozen.size(); i++) {
	    //TableViewEventThread thread = new TableViewEventThread(
	    //    ((TableViewListener)frozen.elementAt(i)), event);
	    ((TableViewListener)frozen.elementAt(i)).tableViewChanged(event);
	}
    }

}

class TableViewEventThread extends Thread {
    TableViewListener l = null;
    TableViewEvent         e = null;

    public TableViewEventThread(TableViewListener l, TableViewEvent e) {
	this.l = l;
	this.e = e;
	start();
    }

    public void run() {
	l.tableViewChanged(e);
    }
}


