package sieve.sluice.adapters;

import com.sun.java.swing.*;
import com.sun.java.swing.table.*;
import com.sun.java.swing.event.*;
import sieve.sluice.*;

/**
 * TableModelAdapter converts a TableView to a (Swing) TableModel.
 */
public class TableModelAdapter extends AbstractTableModel {
    private TableView tv = null;
    private boolean longLabels = false;

    /**
     * Constructs a new TableModelAdapter around the specified TableView.
     * Currently, any TableViewEvent will result in a generic TableModelEvent
     * being fired. This could probably be made more efficient.
     */
    public TableModelAdapter(TableView tv) {
	this.tv = tv;
	tv.addTableViewListener(new TVLAdapter());
    }

    private class TVLAdapter implements TableViewListener {
	public void tableViewChanged(TableViewEvent tve) {
	    // Need to do better than this...
	    fire();
	}
    }

    private void fire() {
	fireTableChanged(new TableModelEvent(this, -1));
    }

    /**
     * Returns the number of columns by finding the number of fields in 
     * in the TableView.
     */
    public int getColumnCount() {
	try {
	    return tv.getFieldNames().length;
	}
	catch(InterchangeException e) {
	    e.printStackTrace();
	}
	return 0;
    }

    /**
     * Returns the number of rows by finding the number of records in 
     * in the TableView.
     */
    public int getRowCount() {
	try {
	    return tv.getRecordCount();
	}
	catch(InterchangeException e) {
	    e.printStackTrace();
	}
	return 0;
    }

    /**
     * Returns an object from the TableView. This could be made more
     * efficient by retrieving and caching a block of values around the
     * requested element.
     */
    public Object getValueAt(int row, int col) {
	try {
	    String fld = getColumnName(col);
	    String[] flds = {fld};
	    return tv.get(flds, row, 1)[0][0];
	}
	catch(InterchangeException e) {
	    e.printStackTrace();
	}
	return null;
    }

    /**
     * Returns either the short name of the column or the longer "description"
     * of the columns, depending on the value of the longLabels property.
     */
    public String getColumnName(int column) {
	try {
	    String name = tv.getFieldNames()[column];
	    if (longLabels) {
		String longName = tv.getFieldDescriptor(name).getDescription();
	  	if (longName != null)
		    return longName;
	    }
	    return name;
	}
	catch(InterchangeException e) {
	    e.printStackTrace();
	}
	return null;
    }

    /**
     * If true, getColumnName will return the description (rather than the 
     * name) of the corresponding TableView field.
     */
    public void setLongLabels(boolean longLabels) {
	if (longLabels != this.longLabels) {
	    this.longLabels = longLabels;
	    fire();
	}
    }
    public boolean isLongLabels() {
	return longLabels;
    }
    public boolean getLongLabels() {
	return longLabels;
    }

}



