package sieve.sluice.codebook;

import java.net.*;
import java.util.*;
import java.applet.*;
import sieve.sluice.*;
import sieve.sluice.codebook.*;
import sieve.sluice.views.CacheView;

/**
 * CodebookFinder is a singleton object which loads an SGML table. It 
 * uses sieve.sluice.URLFinder to locate a valid URL based on a location
 * string.
 * <P>
 * CodebookFinder additionally supports caching of CodebookTable instances,
 * so that applets running in the same VM may share these. To prevent one
 * applet from affecting other applets by, for example, re-setting the
 * table's location, the thing returned by CodebookFinder is not actually
 * a CodebookTable, and should therefore not be cast as one.
 * <P>
 * CodebookFinder is implemented as a singleton object. Users of this class
 * may obtain a reference to it by calling:
 * <PRE>
 *     CodebookFinder finder = CodebookFinder.getCodebookFinder();
 * </PRE>
 *
 * <P>
 * Someday this should probably be generalized to support loading other
 * (non-codebook) tables. At that point it could be moved into the sieve.sluice
 * package.
 */

public class CodebookFinder {
    private static CodebookFinder instance = null;
    private Hashtable cache = null;

    private CodebookFinder() {
	cache = new Hashtable();
    }

    /**
     * Returns a reference to a CodebookFinder object.
     */
    public static CodebookFinder getCodebookFinder() {
	if (instance == null)
	    instance = new CodebookFinder();
	return instance;
    }

    /**
     * Uses sieve.sluice.URLFinder to try to locate a codebook table based
     * on a location string and a reference to the applet that we are running
     * in.
     */
    public TableView getCodebook(String location, Applet applet) {
	TableView view = (TableView) cache.get(location);
	if (view == null) {
	    URL url = null;
	    if (applet != null)
		url = URLFinder.getURLFinder().getURL(location, applet);
	    else
		url = URLFinder.getURLFinder().getURL(location);
	    if (url != null) {
	        try {
	            view = CodeBookTable.getTable(url);
		    // This needs alot more thought
		    int size = view.getRecordCount() * 
			view.getFieldNames().length;
		    if (size < 20000)
		        view = new CacheView(view);
		    cache.put(location, view);
	        }
	        catch(Exception e) { 
		    e.printStackTrace();
	        }
	    }
	}
	return view;
    }

    /**
     * Tries to find a codebook table based on a location string only. This
     * method should only be used when it is impossible to get a reference
     * to the applet that we are running as part of. Typically, this means
     * that this should only be used when running as an application.
     */
    public TableView getCodebook(String location) {
	return getCodebook(location, null);
    }

}

