package sieve.sluice.codebook;

import java.net.*;
import java.io.*;
import sieve.sluice.InterchangeException;

public class DataFile {

    // one of these two will be null
    private URL dataURL = null;
    private String dataFile = null;
    
    protected DataInputStream datastream = null;
    protected int currentLine;
    protected int numLines;
    protected long[] lineOffsets = null;


    /**
     * Can handle both local filenames and URLs
     */
    public DataFile(URL context, String filename) throws IOException {
	String tmp = context.toString();
	tmp = tmp.substring(0, tmp.lastIndexOf("/")) + "/" + filename;
//System.out.println("URL: " + context + " filename: " + filename + " tmp: " + tmp);
	dataURL = new URL(tmp);

	// This appears to choke on resource URLs...
	// dataURL = new URL(context, filename);
	
	parseFile(dataURL.openStream());
    }

    protected DataFile() {
        // For subclass
    }

    public int lineCount() { return numLines; }

    /**
     * Scans through the data file to store byte offsets for the
     * beginning of each new line.  These are stored in the lineOffsets
     * array for future use.
     */
    protected void parseFile(InputStream is) throws IOException {
        long[] tempOffsets = new long[100];
        int n = 0;      // Number in array
        
        BufferedInputStream buffstream = new BufferedInputStream(is);
        // Scan through and find the offsets for each new line
        long pos = 0;       // Offset in file
        tempOffsets[n++] = pos;     // For first position
        
        boolean EOLRead = true, done = false;
        while (! done) {
            pos++;
            switch (buffstream.read()) {
             case -1:
                // Take out last entry if file ended with newline
                if (EOLRead) n--;
                done = true;
                break;
             case '\r':
                buffstream.mark(1);
                if (buffstream.read() != '\n') buffstream.reset();
                else pos++;
                // no break here intentionally
             case '\n':
                if (n == tempOffsets.length) {   // Increase array length
                    long[] oldTemp = tempOffsets;
                    tempOffsets = new long[n * 2];
                    System.arraycopy(oldTemp, 0, tempOffsets, 0, n);
                }
                tempOffsets[n++] = pos;
                EOLRead = true;
                break;
             default:
                EOLRead = false;
            }
        }
        buffstream = null;

        // Put into lineOffsets
        lineOffsets = new long[n];
        System.arraycopy(tempOffsets, 0, lineOffsets, 0, n);

        // Set the number of lines
        numLines = n;
                
    }

    /**
     * Opens the URL for reading & sets the offset pointer
     */     
    public void open() throws IOException {
        datastream = (dataURL == null
                      ? new DataInputStream(new BufferedInputStream(
                        new FileInputStream(dataFile)))
                      : new DataInputStream(new BufferedInputStream(
                        dataURL.openStream()))
                     );
        currentLine = 0;
    }

    /**
     * Call this to "reset" the data file
     */
    public void close() { datastream = null; }


    /**
     * Allows skipping forward in the datastream to the specified line.
     * skipBytes is no faster than the underlying skip() method, so we
     * use the latter.
     */
    public void skipToLine(int line)
        throws IOException, InterchangeException {
        if (datastream == null)
            throw new InterchangeException("Data file not open");
        if (line > numLines || currentLine > numLines)
            throw new InterchangeException("No more lines in file.");
        long n = lineOffsets[line] - lineOffsets[currentLine];
        if (n < 0)
            throw new InterchangeException("Can't perform skip.");
        for (long i = 0 ; i < n; i += datastream.skip(n - i));
        currentLine = line;
    }


    public String readLine() throws InterchangeException, IOException {
        if (datastream == null)
            throw new InterchangeException("Data file not open.");
        if (currentLine > numLines)
            throw new InterchangeException("No more lines in file.");
        currentLine++;
        return datastream.readLine();
    }

}    
        
        




