package sieve.sluice.codebook;

import java.net.*;
import java.io.*;
import sieve.sluice.InterchangeException;


/**
 * Subclass of DataFile that assumes a local file
 * and uses a RandomAccessFile (much easier) rather than an http stream.
 */
public class RADataFile extends DataFile {

    private RandomAccessFile dataFile;  // hides superclass String dataFile

    /**
     * With this constructor we just open the file and leave it open.
     */
    public RADataFile(String contextFile, String filename) throws IOException {
        // Get directory info, etc. from the contextFile
        StringBuffer fname = 
            new StringBuffer(contextFile.substring(
            0, contextFile.lastIndexOf('/')+1));
        fname.append(filename);

        // Scans through file, finds offsets, & sets numLines
        // Much faster to do this with superclass version using a
        // BufferedInputStream than our RandomAccessFile
        parseFile(new FileInputStream(fname.toString()));

        // dataFile now stays open for the life of this object  
        dataFile = new RandomAccessFile(fname.toString(), "r");
    }

    protected void finalize() {
        // Make sure the file is closed
        try { dataFile.close(); } catch (IOException e) {}
    }

    /**
     * Opens the URL for reading & sets the offset pointer
     */     
    public void open() throws IOException {
        // Just reset the file pointer here
        dataFile.seek(0);
    }

    /**
     * Call this to "reset" the data file
     */
    public void close() { 
        // do nothing, we just leave the file open until it is garbage
        // collected
    }


    /**
     * Allows skipping forward in the datastream to the specified line.
     * skipBytes is no faster than the underlying skip() method, so we
     * use the latter.
     */
    public void skipToLine(int line)
        throws IOException, InterchangeException {
            if (line > numLines)
                throw new InterchangeException("Line doesn't exist.");
        dataFile.seek(lineOffsets[line]);
    }


    public String readLine() throws InterchangeException, IOException {
        // Right now, this is about 10 times slower than doing a readLine()
        // with a BufferedInputStream.  So it's not worth it to use this
        // class.  
        return dataFile.readLine();
    }

}    
        
        




