/*
 *
 * Written By:   Chengliang Yang
 * Date:         May 30 1997
 * Description:  Consumer for a channel that recieves messages from
 *		 channel and store them into a local table
 *
 */
 

package sieve.sluice.local;

import sieve.sluice.*;
import java.util.*;


/**
 * A local, in-memory table class extending TableImpl. It is used by 
 * the listener object to store all messages occurred in the channel
 * 
 * @author 
 * @version 
 */

public class SaveTable extends TableImpl implements TableEditor {

    public SaveTable() throws InterchangeException {
	super();
    }

   // append a record
   public void appendRecord(Object[] newRecord) throws InterchangeException {
	    if (newRecord.length != this.fields.size()) throw
		new InterchangeException("Non-compatible data");
	    int index = this.getRecordCount();	
	    for (int r = 0; r < newRecord.length; r++) {
		Vector fld = (Vector) this.data.elementAt(r);
		debug(r+"th element = " + newRecord[r]);
		fld.insertElementAt(newRecord[r], index); 
	    }

	}	

   // get an array of records
   public Object [] getRecord(int atRecord) throws InterchangeException{
		if ((atRecord < 0)||(atRecord > getRecordCount())) 
		    throw new NoSuchRecordException(atRecord);

		Object[] objs = new Object[fields.size()];
	
		for (int j = 0; j < fields.size(); j++){
			Vector fld = (Vector) data.elementAt(j);
			objs[j] = fld.elementAt(atRecord); 
		}
		return objs;
	}


    // print out a record
    public synchronized void printRecord(int atRecord) throws 
InterchangeException {

	if ((atRecord < 0)||(atRecord > getRecordCount())) 
	    throw new NoSuchRecordException(atRecord);
	
	for (int j = 0; j < fields.size(); j++){
		Vector fld = (Vector) data.elementAt(j);
		System.out.print(fld.elementAt(atRecord).toString() + " "); 
	}
	System.out.print("\n");
    }

    
   // find whether it is in the table
   public boolean isInTable(int type, Object obj){
	boolean retVal = false;
	Vector typeVector = (Vector)this.data.elementAt(2);
	Vector dataVector = (Vector)this.data.elementAt(3);

	try{
	for (int i=0; i<getRecordCount(); i++){
		if((Integer.parseInt(typeVector.elementAt(i).toString())==type) &&
			((dataVector.elementAt(i).equals(obj))))
		retVal = true;
	}
	}catch (NoSourceException e){}

	return retVal;
  }

    // print out all contents of the table, used for debug
    public synchronized void printAllRecord() throws 
InterchangeException {
	int numRecords = this.getRecordCount();

	debug("numRecords = " + numRecords);

	for (int j = 0; j < numRecords; j++){
		printRecord(j); 
	}
    }

  private static  final boolean debug = false;
  private void debug(String msg) {
    if (debug)
      System.out.println(getClass()+": "+msg);
  } 
}













