package sieve.sluice.local;

import sieve.sluice.*;
import sieve.sluice.event.*;
import java.util.*;
import java.net.*;

/**
 * A local, in-memory table implementation. 
 */
public class TableImpl extends Table implements TableView,
  TableViewListener {
    Vector 	data   = null; // A vector of vectors of Objects
    Vector 	fields = null; // A vector of Strings
    Hashtable 	idx    = null; // Table of LocalFieldDescriptors
    
  public TableImpl() {
    reset();
  }
    
  private void reset() {
    this.data = new Vector();
    this.fields = new Vector();
    this.idx = new Hashtable();
  }
    
    /* ---------------  Table methods --------------- */
  public synchronized void insert(Object[][] newData, int atRecord) throws
  InterchangeException {
    boolean isAppend = false;
    if (atRecord > getRecordCount()) 
      throw new NoSuchRecordException(atRecord);
    
    isAppend = (atRecord == getRecordCount());
    //System.out.println(getClass().getName()+" isAppend="+isAppend);

    if (newData.length != fields.size())
      throw new sieve.sluice.NoSuchFieldException("Fields in table:" + 
						  fields.size() + ", fields in data: " + newData.length);
    
    for (int f = 0; f < newData.length; f++) {
      Vector fld = (Vector)(data.elementAt(f));
      for (int r = 0; r < newData[0].length; r++) {
	fld.insertElementAt(newData[f][r], r + (int)atRecord); 
      }
    }
    
    if (isAppend)
      eventHandler.fireTableViewEvent(new AppendEvent(this, newData));
    else 
      // Need to be more specific than this...
      eventHandler.fireTableViewEvent(new TableViewEvent(this));
  }
    
  public synchronized void addField(FieldDescriptor fd)
    throws InterchangeException {
      
      if ((data.size() != 0) && (((Vector)(data.elementAt(0))).size() > 0))
	throw new InterchangeException("Cannot add field: Table has data");
      
      fields.addElement(fd.getName());
      data.addElement(new Vector(100));
      idx.put(fd.getName(), new LocalFieldDescriptor(fields.size() - 1, fd));
  }
    
  public synchronized void remove(int startingAt, int numToRemove) throws
    InterchangeException {
    if ((getRecordCount() <= startingAt) || 
	(getRecordCount() < startingAt + numToRemove))
      throw new NoSuchRecordException(data.size());
    
    int numFields = data.size();
    for (int f = 0; f < numFields; f++) {
      Vector v = (Vector) data.elementAt(f);
      for (int r = 0; r < numToRemove; r++) {
	v.removeElementAt(startingAt);
      }
    }
    
  }
    
  public synchronized void removeField(String fieldName) 
    throws InterchangeException {
      LocalFieldDescriptor fd = 
	(LocalFieldDescriptor) getFieldDescriptor(fieldName);
      int fldNum = fd.getFieldNumber(); 
      data.removeElementAt(fldNum);
      fields.removeElementAt(fldNum);
      idx.remove(fieldName);
      for (Enumeration e = idx.elements(); e.hasMoreElements();) {
	LocalFieldDescriptor f = (LocalFieldDescriptor)e.nextElement();
	int fNum = f.getFieldNumber();
	if (fNum > fldNum)
	  f.setFieldNumber(fNum - 1);
      }
  }
    
    
  public synchronized URL getLocationURL() {
	return null;
  }
    
  public synchronized void setLocationURL(URL location) {
  }
    
    /* -------------  TableView methods ------------- */
    
  public synchronized String[] getFieldNames() throws NoSourceException {
    String[] flds = new String[fields.size()];
    fields.copyInto(flds);
    return flds;
  }
    
  public synchronized FieldDescriptor getFieldDescriptor(String field) throws 
  InterchangeException {
    LocalFieldDescriptor fd = (LocalFieldDescriptor)(idx.get(field));
    if (fd == null)
      throw new sieve.sluice.NoSuchFieldException(field);
    return fd;
  }
    
  public synchronized int getRecordCount() throws NoSourceException {
    if (data.size() == 0)
      return 0;
    else
      return ((Vector)data.elementAt(0)).size();
  }
    
  public synchronized void tableViewChanged(TableViewEvent event) {
    eventHandler.fireTableViewEvent(event);
  }
    
  public synchronized Object[][] get(String[] fields, int startingAt,
				     int numberOfRecords) throws InterchangeException
  {
    
    if (startingAt >= getRecordCount())
      throw new NoSuchRecordException(startingAt);
    
    if (startingAt + numberOfRecords > getRecordCount())
      throw new NoSuchRecordException(getRecordCount());
    
    Object[][] result = new Object[fields.length][(int) numberOfRecords];
    
    for (int j = 0; j < fields.length; j++) {
      LocalFieldDescriptor fd = (LocalFieldDescriptor)
	(idx.get(fields[j]));
      if (fd == null)
	throw new sieve.sluice.NoSuchFieldException(fields[j]);
      
      int fieldNum = fd.getFieldNumber();
      
      for (int i = 0; i < numberOfRecords; i++) {
	result[j][i] = 
	  ((Vector)data.elementAt(fieldNum)).elementAt((int)
						       (i + startingAt));
      }
    }
    return result;
  }
    
  public synchronized Object[][] get(String[] fields, int startingAt)
    throws InterchangeException
  {
    return get(fields, startingAt, getRecordCount() - startingAt);
  }
    
  public synchronized boolean editRecord(String searchfield, Object searchValue, String editField, Object editValue)throws InterchangeException
  {
    String[] field = {searchfield};
    Object[][] results = get(field,0);
    int i = 0;
    boolean done = false;
    try{
      while((i < getRecordCount())&& (!done)) 
	{
	if(results[0][i].equals(searchValue))
	  {
	    LocalFieldDescriptor fd = (LocalFieldDescriptor)
	      (idx.get(editField));
	    if (fd == null)
	      throw new sieve.sluice.NoSuchFieldException(editField);
	    
	    int fieldNum = fd.getFieldNumber();
	    
	    field = getFieldNames();
	    System.out.println(field[fieldNum]);
	    System.out.println(i);
	    results = get(field,i,1);
	    
	    remove(i,1);
	    System.out.println(fieldNum);
	    System.out.println(results[0][0].toString());
	    results[fieldNum][0] = editValue;
	    insert(results,i); 
	    
	    done = true;
	  }
	else
	  i++;
	}
      i = 0;
    }catch(Exception e)
      {e.printStackTrace();}
    i = 0;
    return done;
  }  
}



class LocalFieldDescriptor extends FieldDescriptor {
  int fieldNumber = 0;
  
  public LocalFieldDescriptor(int fieldNumber, FieldDescriptor fd) {
    super(fd.getName());
    this.fieldNumber = fieldNumber;
    copy(fd);
  }
  
  int getFieldNumber() {
    return fieldNumber;
  }
  
  void setFieldNumber(int fieldNumber) {
    this.fieldNumber = fieldNumber;
  }
}



