package sieve.sluice.views;

import sieve.sluice.*;
import java.util.*;


// Among other improvements, this desparately needs to use Gamma's
// flyweight mechanism...
public class CacheView extends DefaultTableView implements TableView,
    TableViewListener {
    Object[][] 	cache = null;
    String[] 	fields = null;
    Hashtable 	idx = null;

    public CacheView() {
	super(null);
	this.source = null;
	this.cache = null;
    }

    public CacheView(TableView source) {
	super(null);
	try {
	    addSourceView(source);
	}
	catch (TooManySourcesException e) {
	    // Should never happen on construction....
	    e.printStackTrace();
	}
    }

    private void reset() {
        cache = null;
        fields = null;
	try {
	    fields = source.getFieldNames();
	    cache = source.get(fields, 0);
            //System.out.println("CacheView got array dim: " + cache.length + 
            //       " fields (expecting: " + fields.length + ") and " + 
            //         cache[0].length + " records");
	    idx = new Hashtable();
	    for (int i = 0; i < fields.length; i++) {
		idx.put(fields[i], new Integer(i));
	    }
	}
	catch(InterchangeException e) {
        e.printStackTrace(); 
        }
    }

    public void addSourceView(TableView newSource) throws 
	TooManySourcesException {
        if (this.source != null)
            throw new TooManySourcesException("Only one source allowed.");
        this.source = newSource;

	reset();

        eventHandler.fireTableViewEvent(new TableViewEvent(this));
    }

    public void removeSourceView(TableView source) {
  	if (source == this.source) {
            this.source = null;
	    cache = null;
	    fields = null;
            eventHandler.fireTableViewEvent(new TableViewEvent(this));
        }
    }

    public int getRecordCount() throws NoSourceException {
	if ((source == null) || (cache == null) || (cache.length == 0))
            throw new NoSourceException();
	return cache[0].length;
    }

    public void tableViewChanged(TableViewEvent event) {
	if (source != null) { 
	    reset();
	}
        eventHandler.fireTableViewEvent(event);
    }

    public Object[][] get(String[] fields, int startingAt,
        int numberOfRecords) throws InterchangeException {

        if ((source == null) || (cache == null) || (fields == null))
            throw new NoSourceException();

	if (startingAt >= getRecordCount())
            throw new NoSuchRecordException(startingAt);

	if (startingAt + numberOfRecords > getRecordCount())
            throw new NoSuchRecordException(getRecordCount());

	Object[][] result = new Object[fields.length][numberOfRecords];

	for (int j = 0; j < fields.length; j++) {
            Integer pos = (Integer)(idx.get(fields[j]));
            if (pos == null)
                throw new sieve.sluice.NoSuchFieldException(fields[j]);

            int fieldNum = pos.intValue();

	    for (int i = 0; i < numberOfRecords; i++) {
	        result[j][i] = cache[fieldNum][(i + startingAt)];
	    }
	}
	return result;
    }

    public Object[][] get(String[] fields, int startingAt)
        throws InterchangeException {
        return get(fields, startingAt, getRecordCount() - startingAt);
    }


}



