package sieve.sluice.views;

import java.beans.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

import sieve.sluice.*;

public class ColorChangeViewCustomizer extends Panel implements Customizer,
    TableViewListener, PropertyChangeListener, ItemListener, MouseListener {
    public static final String NOT_CONNECTED_MSG = "(Not Connected)";

    private ColorChangeView target;
    private FieldChoice	 fieldChoice;
    private Label 	 label;
    private PropertyEditor colorEditor = null;
    private Canvas 	colorCanvas = null;
    private Color   	currentColor = null;

    public ColorChangeViewCustomizer() {
	setLayout(new GridLayout(1, 2));
    }

    public void setObject(Object obj) {
	target = (ColorChangeView) obj;

	fieldChoice = new FieldChoice(target.source);
	fieldChoice.addItemListener(this);
	updateChoice();
	add(fieldChoice);

	try {
	    target.addTableViewListener(this);
            colorEditor = PropertyEditorManager.findEditor(Class.forName(
                "java.awt.Color"));
            colorEditor.addPropertyChangeListener(new ColorChangeAdapter(this));
  	    resetEditor();
	}
	catch (Exception e) {
	    e.printStackTrace();
	}

	colorCanvas = new Canvas();
	colorCanvas.addMouseListener(this);
	add(colorCanvas);
	if (currentColor == null)
	    colorCanvas.setBackground(Color.black);
	else
	    colorCanvas.setBackground(currentColor);
	colorCanvas.resize(30,fieldChoice.getPreferredSize().height);
    }

    protected void updateChoice() {
	fieldChoice.setTableView(target.source);
	fieldChoice.reset();
	String selection = fieldChoice.getSelectedItem();
	if ((target.getField() == null) && (selection != null)) {
	    try {
	        target.setField(selection);
	    }
	    catch(InterchangeException e) {}
	}
	if ((selection != null) && !(selection.equals(target.getField())))
            fieldChoice.select(target.getField());
	resize(fieldChoice.getPreferredSize().width * 2, 
	    fieldChoice.getPreferredSize().height);
    }

    public void resetEditor() {
      try {
	String selection = fieldChoice.getSelectedItem();
	if (selection != null)
          currentColor = target.getFieldDescriptor(selection).getColor();
        if ((colorEditor != null) && (currentColor != null))
          colorEditor.setValue(currentColor);
      }
      catch(Exception e) {
        e.printStackTrace();
      }
    }


    public void itemStateChanged(ItemEvent evt) {
	String txt = fieldChoice.getSelectedItem();
	if (! txt.equals(NOT_CONNECTED_MSG) ) {
	    try {
	        target.setField(txt);
                resetEditor();
	        support.firePropertyChange("field", null, txt);
	    }
	    catch (Exception e) {
		e.printStackTrace();
	    }
	}
    }

    public void mousePressed(MouseEvent evt){}
    public void mouseReleased(MouseEvent evt) {
	Component frame = this;
        while ((frame != null) && (! (frame instanceof Frame))) {
            frame = frame.getParent();
        }
        if (frame != null) {
            int x = frame.location().x - 30;
            int y = frame.location().y + 50;
	    new sieve.workbench.PropertyDialog((Frame)frame, colorEditor, x, y);
        }
    }
    public void mouseEntered(MouseEvent evt){}
    public void mouseExited(MouseEvent evt){}
    public void mouseClicked(MouseEvent evt){}

    //----------------------------------------------------------------------

    public void addPropertyChangeListener(PropertyChangeListener l) {
	support.addPropertyChangeListener(l);
    }

    public void removePropertyChangeListener(PropertyChangeListener l) {
	support.removePropertyChangeListener(l);
    }

    private PropertyChangeSupport support = new PropertyChangeSupport(this);

    //----------------------------------------------------------------------

    public void tableViewChanged(TableViewEvent event) {
	updateChoice();
        resetEditor();
    }

    boolean processingRemoteChange = false; // Ugly Hack
    public synchronized void propertyChange(PropertyChangeEvent evt) {
        processingRemoteChange = true;
        updateChoice();
        resetEditor();
        colorCanvas.setBackground(currentColor);
        processingRemoteChange = false;
    }

    public synchronized void colorReset() {
      if (processingRemoteChange) // HACK
        return;
      try {
/*
          FieldDescriptor fd = target.getFieldDescriptor(
	    fieldChoice.getSelectedItem()).copy();
          currentColor = (Color) colorEditor.getValue();
          fd.setColor(currentColor);
          target.setFieldDescriptor(fd);
*/
          currentColor = (Color) colorEditor.getValue();
	  if (currentColor != target.getColor()) {
              target.setColor(currentColor);
              if (colorCanvas != null)
            	  colorCanvas.setBackground(currentColor);
  	      support.firePropertyChange("color", null, currentColor);
	  }
      }
      catch (Exception e) {
        e.printStackTrace();
      }
    }



    private class ColorChangeAdapter implements PropertyChangeListener {
  	private ColorChangeViewCustomizer cust = null;

  	public ColorChangeAdapter(ColorChangeViewCustomizer cust) {
    	    this.cust = cust;
  	}

  	public void propertyChange(PropertyChangeEvent evt) {
    	    cust.updateChoice();
    	    cust.colorReset();
  	}
    }
}
