package sieve.sluice.views;

import java.beans.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

import sieve.sluice.views.predicates.*;
import sieve.sluice.*;

public class CriteriaViewCustomizer extends Panel implements Customizer,
    TableViewListener, PropertyChangeListener {
    public static final String NOT_CONNECTED_MSG = "(Not Connected)";

    private CriteriaView target;
    private FieldChoice	 fieldChoice;
    private PredicateChoice  testsChoice;
    private Choice	 valueChoice;
    private TextField	 valueField;
    private Checkbox	 negateBox;
    private Button 	 confirmButton;

    public CriteriaViewCustomizer() {
	setLayout(new GridLayout(3, 1));
    }

    public void setObject(Object obj) {
	target = (CriteriaView) obj;
	fieldChoice = new FieldChoice(target.source);
	fieldChoice.addItemListener(new ItemListener() {
	        public void itemStateChanged(ItemEvent evt) {
	            String txt = fieldChoice.getSelectedItem();
	            try {
	            	target.setField(txt);
	            	support.firePropertyChange("field", null, txt);
		    	updateTests();
	            }
	            catch (Exception e) {
		    	e.printStackTrace();
	            }
		}
	    }
	);
	testsChoice = new PredicateChoice();
	testsChoice.addItemListener(new ItemListener() {
	 	public void itemStateChanged(ItemEvent evt) {
	    	    String txt = testsChoice.getSelectedItem();
	    	    target.setPredicate(txt);
	    	    support.firePropertyChange("test", null, txt);
		}
	    }
	);
        valueField = new TextField(20);
	valueField.addKeyListener(new KeyAdapter() {
		public void keyPressed(KeyEvent evt) {
	    	    valueField.setForeground(Color.red);
		}
	    }
	);
	ActionListener confirmListener = new ActionListener() {
	    public void actionPerformed(ActionEvent evt) {
	        valueField.setForeground(Color.black);
	        String txt = valueField.getText();
	        Object val = sieve.util.Parser.parse(testsChoice.getType(), 
		    txt);
	    	target.setValue(val);
	    	support.firePropertyChange("value", null, val);
	    }
	};
	valueField.addActionListener(confirmListener);
	negateBox = new Checkbox("!");
	negateBox.setState(target.getNegatePredicate());
	negateBox.addItemListener(new ItemListener() {
		public void itemStateChanged(ItemEvent evt) {
	     	    target.setNegatePredicate(negateBox.getState());
	    	    support.firePropertyChange("negatePredicate", null, 
		    new Boolean(negateBox.getState()));
		}
	    }
	);
	confirmButton = new Button("set");
	confirmButton.addActionListener(confirmListener);
	updateFields();
	updateTests();
	updateValue();
	add(fieldChoice);
	Panel p1 = new Panel();
	p1.setLayout(new BorderLayout());
	p1.add("West", negateBox);
	p1.add(testsChoice);
	add(p1);
	Panel p2 = new Panel();
	p2.setLayout(new BorderLayout());
	p2.add(valueField);
	p2.add("East", confirmButton);
	add(p2);
	target.addTableViewListener(this);
	target.addPropertyChangeListener(this);
    }

    void updateFields() {
	fieldChoice.setTableView(target.source);
	fieldChoice.reset();
        String selection = fieldChoice.getSelectedItem();
        if ((target.getField() == null) && (selection != null)) {
	    try {
                target.setField(selection);
                target.setPredicate("IsEqualTo");
 	    }
	    catch(Exception e) {
		e.printStackTrace();
	    }
        }
        if ((selection != null) && !(selection.equals(target.getField())))
            fieldChoice.select(target.getField());
     }

     void updateTests() {
	String selection;
	try {
	    if (target.getTestAsString()) {
	        testsChoice.setType("java.lang.String");
	    }
	    else {
	        FieldDescriptor fd = 
		    target.getFieldDescriptor(target.getField());
	        testsChoice.setType(fd.getType());
	    }
	    selection = testsChoice.getSelectedItem();
	    if ((target.getPredicate() == null) && !(selection == null))
                target.setPredicate(selection);
            else if ((selection == null) || 
		!(selection.equals(target.getPredicate())))
                testsChoice.select(target.getPredicate());
	}
	catch (InterchangeException e) {
	    // Fail quietly...
	    // e.printStackTrace();
	}
    }

    void updateValue() {
	if ((valueField != null) && (target.getValue() != null))
	    valueField.setText(target.getValue().toString());
    }

    //----------------------------------------------------------------------

    public void addPropertyChangeListener(PropertyChangeListener l) {
	support.addPropertyChangeListener(l);
    }

    public void removePropertyChangeListener(PropertyChangeListener l) {
	support.removePropertyChangeListener(l);
    }

    private PropertyChangeSupport support = new PropertyChangeSupport(this);

    //----------------------------------------------------------------------

    public void tableViewChanged(TableViewEvent event) {
        updateFields();
        updateTests();
	invalidate();
	doLayout();
    }
 
    public void propertyChange(PropertyChangeEvent evt) {
        if (evt.getPropertyName().equals("field")) {
            fieldChoice.select((String)evt.getNewValue());
        }
        else if (evt.getPropertyName().equals("test")) {
            testsChoice.select((String)evt.getNewValue());
        }
        else if (evt.getPropertyName().equals("negatePredicate")) {
            negateBox.setState(((Boolean)evt.getNewValue()).booleanValue());
        }
	// This will need to be re-worked
        else if (evt.getPropertyName().equals("value")) {
            valueField.setText(evt.getNewValue().toString());
        }
    }

    
}
