package sieve.sluice.views;

import sieve.sluice.*;
import java.awt.*;
import java.util.*;

public class FieldChoice extends Choice {
    public final String NOT_CONNECTED_MSG = "(No Fields)";

    TableView 	tv 	= null;
    String	notConnectedMessage = NOT_CONNECTED_MSG;

    Font 	myFont	= getFont();
    Hashtable	map	= new Hashtable();

    public FieldChoice() { }

    public FieldChoice(TableView tv) {
	this.tv = tv;
	reset();
    }

    public FieldChoice(TableView tv, String notConnectedMessage) {
	this(tv);
	this.notConnectedMessage = notConnectedMessage;
    }

    public TableView getTableView() {
	return tv;
    }

    public void setTableView(TableView tv) {
	if (tv != this.tv) {
	    this.tv = tv;
 	    reset();
	}
    }

    public String getNotConnectedMessage() {
	return notConnectedMessage;
    }

    public void setNotConnectedMessage(String notConnectedMessage) {
	this.notConnectedMessage = notConnectedMessage;
    }

    public void reset() {
	String[] fieldNames = null;
	String[] newFields = null;

	try {
       	    if (tv != null) {
	        fieldNames = tv.getFieldNames();
		newFields = new String[fieldNames.length];
		map = new Hashtable();
		for (int i = 0; i < fieldNames.length; i++) {
		    FieldDescriptor fd = tv.getFieldDescriptor(fieldNames[i]);
		    newFields[i] = fd.getDescription();
		    if (newFields[i] == null)
		        newFields[i] = fieldNames[i];
		    map.put(newFields[i], fieldNames[i]);
		}
	    }
	}
	catch (InterchangeException e) {
	    // If something goes wrong we'll just show the
	    // notConnectedMessage
	}

	if ((newFields == null) || (newFields.length == 0)) {
	    removeAll();
	    add(notConnectedMessage);
	    disable();
	    // Enabling/Disabling makes the font get bigger...
	    setFont(myFont);
	}
	else {
	    String selection = super.getSelectedItem();
	    for (int i = 0; i < newFields.length; i++) {
		if ((getItemCount() <= i) || (!getItem(i).equals(newFields[i])))
		    insert(newFields[i], i);
	    }
	    int itemCount = getItemCount();
	    for (int j = newFields.length; j < itemCount; j++)
		//remove(newFields.length);
		remove(getItemCount() - 1);

	    if (selection != null)
	        select(selection);

	    enable();
	    // Enabling/Disabling makes the font get bigger...
	    setFont(myFont);
	}

    }

    public String getSelectedItem() {
     	String selection = super.getSelectedItem();
    	if ((selection != null) && (selection.equals(notConnectedMessage)))
	    selection = null;
	else if (selection != null)
	    selection = (String)map.get(selection);
	return selection;
    }
}



