package sieve.sluice.views;

import sieve.sluice.*;
import sieve.sluice.event.*;
import java.util.*;

public class FieldNameSelectionView extends SelectionView implements TableView,
    TableViewListener {
    protected FieldNameSelectionTest test = null;

    public FieldNameSelectionView() {
	super();
    }

    /**
     * Creates a view containing only the specified fields.
     */
    public FieldNameSelectionView(TableView source, String[] fields) {
        super(source);
        test = new FieldNameSelectionTest(fields);
        setSelectionTest(test);
    }

    /**
     * Creates a view containing only the specified field.
     */
    public FieldNameSelectionView(TableView source, String field) {
        super(source);
        test = new FieldNameSelectionTest(field);
        setSelectionTest(test);
    }

    public void setFields(String[] fields) {
	String[] oldFields = getFields();

	// This could be a bit more efficient. We want to ignore this
	// if the fields haven't actually changed.
	if (oldFields != null) {
	    if (oldFields.length == fields.length) {
		int i = 0;
	        for (i = 0; (i < fields.length) &&
		    oldFields[i].equals(fields[i]); i++);
		if (i == fields.length)
		    return;
	    }
	}
        test = new FieldNameSelectionTest(fields);
        setSelectionTest(test);
	changes.firePropertyChange("fields",  oldFields, fields);
    }

    public String[] getFields() {
	if (test != null)
	    return test.getFields();
        else
            return null;
    }

	/**
	 * returns all fields in "underlying" source table, not just the 
	 * selected ones
	 */
	public String[] getAvailableFieldNames() throws NoSourceException {
		return source.getFieldNames();
	}
 

    public void tableViewChanged(TableViewEvent event) {
	// No need to reset ourselves if this is just a descriptor change
	if (!(event instanceof FieldDescriptorChangeEvent) && (test != null))
	    setFields(getFields()); 
        super.tableViewChanged(event);
    }

}

class FieldNameSelectionTest implements SelectionTest {
    private Vector  flds    = null;
 
    public FieldNameSelectionTest(String[] fields) {
        flds = new Vector();
        for (int i = 0; i < fields.length; i++)
            flds.addElement(fields[i]);
    }
 
    public FieldNameSelectionTest(String field) {
        flds = new Vector();
        flds.addElement(field);
    }
 
    public boolean showField(TableView source, String field) {
        return flds.contains(field);
    }

    String[] getFields() {
        String[] fields = new String[flds.size()];
        flds.copyInto(fields);
        return fields;
    }
}

