package sieve.sluice.views;  

import sieve.sluice.*;
import java.util.Vector;

/**
 * NonEmptySampler is a Sampler filter for a SampleView. It maintains a list
 * (vector) of record numbers in the underlying source table for which the 
 * requested field values are not missing. CAUTION: right now fails if
 * fields[] has duplicates; will fix in CodebookTable class.
 *
 * @see sieve.sluice.views.SampleView
 * @see sieve.sluice.views.Sampler
 *  
 * @version	%I%, %G%
 * @author	Lynn W. Jones
 */


public class NonEmptySampler implements Sampler {

	private Vector validRecs;

	public NonEmptySampler(sieve.sluice.TableView src, String fields[], int maxRecs) {
    /**
	 * creates list of maxRecs records from table src, for which all fields in
	 * fields[] have "valid" data (non-missing entries)
	 */
		int i,j;
		int recCt = 0;
		int fldCt = fields.length;
		try { recCt = src.getRecordCount(); }
		catch(sieve.sluice.NoSourceException e) {e.printStackTrace();}
		Object[][] tab = null;
		try { tab = src.get(fields,0); }//get all starting from 0
		catch(sieve.sluice.InterchangeException e) {e.printStackTrace();}
		validRecs = new Vector(maxRecs);
		for(i=0;i<recCt;i++) {
			if (validRecs.size() == maxRecs)
				break;
			for (j=0;j<fldCt;j++){
				if(tab[j][i] == null)  //note order of indices
					break;
			}
			//if got through all fields, then this record OK; add it to vector
			if (j == fldCt)
				validRecs.addElement(new Integer(i));
		}
		validRecs.trimToSize();	//trim in case not as many as maxRecs valid records in table
	}

	public NonEmptySampler(sieve.sluice.TableView src, String fields[]) {
	/**
	* creates list of all records from table src, for which all fields in
	* fields[] have "valid" data (non-missing entries)
	*/
		int i,j;
		int recCt = 0;
		int fldCt = fields.length;
		Object[][] tab = new Object[recCt][fldCt];
		try { tab = src.get(fields,0); }//get all starting from 0
		catch(sieve.sluice.InterchangeException e) {e.printStackTrace();}
		try { recCt = src.getRecordCount(); }
		catch(sieve.sluice.NoSourceException e) {e.printStackTrace();}
		validRecs = new Vector(recCt);
		for(i=0;i<recCt;i++) {
			for (j=0;j<fldCt;j++){
				if(tab[j][i] == null) //note order of indices
					break;
			}
			//if got through all fields, then this record OK; add it to vector
			if (j == fldCt) 
				validRecs.addElement(new Integer(i));
		}
		validRecs.trimToSize();
	}

	public int size() {
	/**
	 * Returns the size of the sample.
	 */
		return validRecs.size();
	}

	public int getRecordNumber(int requestedRecord) throws sieve.sluice.NoSuchRecordException {
	/**
	 * Translates the requested record number into the record number in the underlying view. 
	 */
		Integer tmp;
		if ((requestedRecord >= 0) && (requestedRecord < validRecs.size()) ) {
			tmp = (Integer)(validRecs.elementAt(requestedRecord));
			return tmp.intValue();
		}
		else 
			throw new sieve.sluice.NoSuchRecordException(requestedRecord);
	}

	public boolean recordIsInSample(int record) {
	/**
	 * Returns true if the sample includes the record number in the underlying source view. 
	 */
		Integer tmp;
		for (int i=0;i<validRecs.size();i++) {
			tmp = (Integer)(validRecs.elementAt(i));
			if (tmp.intValue() == record)
				return true;
		}
		return false;
	}
}


