package sieve.sluice.views;

import sieve.sluice.*;
import java.util.*;

/**
 * A SampleView "shrinks" a TableView by sampling values from it using
 * a sampling mechanism. 
 */
public class SampleView extends DefaultTableView implements TableView {
    protected Sampler sampler = null;

    public SampleView(TableView source, Sampler sampler) {
	super(source);
	this.sampler = sampler;
    }

    public SampleView() {
	super(null);
    }

    protected Sampler getSampler() {
	return sampler;
    }

    protected void setSampler(Sampler sampler) {
	this.sampler = sampler;
    }

    /**
     * Returns number of records in the view.
     */
    public synchronized int getRecordCount() throws NoSourceException {
	if ((sampler == null) || (source == null))
	    throw new NoSourceException();
	return sampler.size();
    }

    /**
     * Returns no more than <CODE>numberOfRecords</CODE> records starting
     * at record number <CODE>startingAt</CODE> from 
     * the fields specified in <CODE>fields</CODE>.
     */
    public synchronized Object[][] get(String[] fields, int startingAt, 
	int numberOfRecords) throws InterchangeException {
	Object[][] results = new Object[fields.length][(int)numberOfRecords];

	for (int rec = (int)startingAt; rec < numberOfRecords + startingAt; 
		rec++) {
	    Object[][] record = source.get(fields, 
		sampler.getRecordNumber((int)rec), 1);
	    for (int fld = 0; fld < record.length; fld++)
		results[fld][(int)(rec - startingAt)] = record[fld][0];
	}

	return results;
    }

    /**
     * Returns all records starting from record number <CODE>startingAt</CODE>
     * from the fields specified in <CODE>fields</CODE>.
     */
    public synchronized Object[][] get(String[] fields, int startingAt) 
	throws InterchangeException {
	return get(fields, startingAt, sampler.size());
    }
}

