package sieve.sluice.views;

import sieve.sluice.*;
import java.util.*;

/**
 * A Sampler implements a sampling mechanism. It is used by SampleView
 * to translate a requested record number into a "real" record number
 * in the underlying source view.
 */ 
public interface Sampler {
    /**
     * Translates the requested record number into the record number in
     * the underlying view.
     */
    public int getRecordNumber(int requestedRecord) throws 
	NoSuchRecordException;

    /**
     * Returns the size of the sample. 
     */
    public int size();

    /**
     * Returns true if the sample includes the record number in the
     * underlying source view. This is used to filter out events which
     * indicate changes to records which are hidden by this sample.
     */
    public boolean recordIsInSample(int record);
}


