package sieve.sluice.views;

import sieve.sluice.*;
import sieve.sluice.NoSuchFieldException;
import java.util.*;

/**
 * A SelectionView hides one more fields based on a SelectionTest
 * object.
 */
public class SelectionView extends DefaultTableView implements TableView {
    private Vector		availableFields	= null;
    private SelectionTest	test	        = null;

    public SelectionView(TableView source, SelectionTest test) {
	super(source);
        setSelectionTest(test);
    }

    public SelectionView(TableView source) {
	super(source);
    }

    public SelectionView() {
        super(null);
    }

    public TableDescriptor getTableDescriptor() throws NoSourceException {
        TableDescriptor td = new TableDescriptor();
        td.copyFromSource(super.getTableDescriptor());
        // Need to move this to TableDescriptor class
        String fld = td.getLabelField();
        if ((fld != null) && !(availableFields.contains(fld)))
          td.setLabelField(null);
        fld = td.getIndependentVariableField();
        if ((fld != null) && !(availableFields.contains(fld)))
          td.setIndependentVariableField(null);
        return td;
    }

    public String[] getFieldNames() throws NoSourceException {
        if ((test != null) && (availableFields != null)) {
	    String[] fields = new String[availableFields.size()];
	    availableFields.copyInto(fields);
  	    return fields;
        }
        else if (source == null)
	    throw new NoSourceException();
	else {
	    return new String[0];
	}
    }

    public FieldDescriptor getFieldDescriptor(String field) throws 
        InterchangeException {
	    if (availableFields == null) 
	        throw new NoSourceException("Need to select fields");
            else if (availableFields.contains(field))
	        return super.getFieldDescriptor(field);
	    else
	        throw new NoSuchFieldException(field);
    }

    /**
     * Returns no more than <CODE>numberOfRecords</CODE> records starting
     * at record number <CODE>startingAt</CODE> from 
     * the fields specified in <CODE>fields</CODE>.
     */
    public Object[][] get(String[] fields, int startingAt, 
	int numberOfRecords) throws InterchangeException {
	if (availableFields == null) 
	    throw new NoSourceException("Need to select fields");
	else {
	    for (int i = 0; i < fields.length; i++) {
		if (!availableFields.contains(fields[i]))
	    	    throw new NoSuchFieldException(fields[i]);
	    }
	}
	return super.get(fields, startingAt, numberOfRecords);
    }

    /**
     * Returns all records starting from record number <CODE>startingAt</CODE>
     * from the fields specified in <CODE>fields</CODE>.
     */
    public Object[][] get(String[] fields, int startingAt) 
	throws InterchangeException {
	return get(fields, startingAt, getRecordCount());
    }


    public void addSourceView(TableView source) throws TooManySourcesException {
        super.addSourceView(source);
        resetAvailableFields();
    }

    public void setSelectionTest(SelectionTest test) {
        this.test = test;
        resetAvailableFields();
	eventHandler.fireTableViewEvent(new TableViewEvent(this));
    }

    protected void resetAvailableFields() {
        if ((test != null) && (source != null)) {
	    String[] allFields = null;
	    availableFields = new Vector();
            try {
	        allFields = source.getFieldNames();
            }
            catch (NoSourceException e) { 
                return;
            }

	    for (int i = 0; i < allFields.length; i++) {
	        if (test.showField(source, allFields[i])) {
		    availableFields.addElement(allFields[i]);
		}
	     }
        }
    }

}

