package sieve.sluice.views;

import sieve.sluice.*;
import java.util.*;

/**
 * A SystematicSampleView shrinks a table by randomly sampling the
 * first record and then choosing every k^th record after that.
 */
public class SystematicSampleView extends SampleView implements TableView {
    public SystematicSampleView(TableView source, int size) {
	super(source, new SystematicSampler(source, size));
    }
}

class SystematicSampler implements Sampler {
	private int		size	= 0;
	private	TableView	source	= null;

	private int		viewSize	= 0;
	private int		start		= 0;
	private int		step		= 0;

	private	Vector		records		= null;

	public SystematicSampler(TableView source, int size) {
		/* Need to handle bad sizes... */
		this.source = source;
		this.size = size;

		try {
		viewSize = source.getRecordCount();
       		}
		catch (Exception e) {
		    // WHAT TO DO HERE???
		    e.printStackTrace();
		}
		step = viewSize/size;
		start = (int)Math.round(Math.random() * viewSize);
	}

	public int getRecordNumber(int requestedRecord) 
		throws NoSuchRecordException {
		if ((requestedRecord < 0) || (requestedRecord >= size))
			throw new NoSuchRecordException(Long.toString(requestedRecord));
		return (start + (requestedRecord * step)) % viewSize;
	}

	public int size() {
		return size;
	}

	public boolean recordIsInSample(int record) {
		/*
		 * This is a pain, so don't do it until we have to...
		 */
		if (records == null) {
			for (int i = 0; i < size; i++)
				records.addElement(new Long((start + (i * step)) % viewSize));
		}
		return records.contains(new Long(record));
	}
}




