package sieve.sluice.views;

import java.beans.*;
import java.awt.*;
import java.awt.event.*;

import sieve.sluice.*;
import sieve.sluice.event.*;

public class TableSelectionViewCustomizer extends Panel implements 
    Customizer, TableViewListener, PropertyChangeListener, ItemListener {
    public static final String NOT_CONNECTED_MSG = "(Not Connected)";
    public static final String NO_TABLES_MSG = "(No Tables to Select)";

    private TableSelectionView 	target		= null;
    private List	   	tableList	= null;
    private String[] 	   	tableNames	= null;
    private TableView[]	  	tables		= null;

    public TableSelectionViewCustomizer() {
	setLayout(new BorderLayout());
    }

    public void setObject(Object obj) {
	target = (TableSelectionView) obj;
	tableList = new List(5, true);
	tableList.setMultipleMode(false);
	tableList.addItemListener(this);
	updateList();
	//	add("North", new Label("Table Selector"));
	add("Center", tableList);
	target.addPropertyChangeListener(this);
	target.addTableViewListener(this);
    }

    /**
     * generateTableNames is intended as a customization point. It should 
     * take the contents of its argument and generate an appropriate array of
     * strings representing those tables.
     */
    protected String[] generateTableNames(TableView[] tables) {
	String[] names = new String[tables.length];
	for (int t = 0; t < tables.length; t++) {
	    String name = null;
	    try {
	        TableDescriptor td = tables[t].getTableDescriptor();
	        name = td.getName();
	    }
	    catch(InterchangeException e) {} // Ignore

	    if (name == null)
		names[t] = "Table " + t;
	    else
		names[t] = name;
	}
	return names;
    }

    private void updateList() {
	String[]  newNames 		= null;
	TableView currentView 		= null;
	int	  currentViewIdx 	= -1;

	tables = null;

	try {
	    tables = target.getAvailableTableViews();
	    newNames = generateTableNames(tables);
	    currentView = target.getSelectedTableView();
        }
        catch (Exception e) {
	    // WHAT TO DO HERE?
        }

	if (newNames == null) {
	    tableList.removeAll();
 	    tableList.addItem(NOT_CONNECTED_MSG);
	    tableList.disable();
	}
	else {
	  if (newNames.length == 0) {
	    //tableList.disable();
	      tableList.addItem(NO_TABLES_MSG);
		tableList.enable();
	    } else {
		tableList.enable();
	    	for (int i=0; i < newNames.length; i++) {
		    if (i >= tableList.countItems())
	                tableList.addItem(newNames[i]);
		    else if (!newNames[i].equals(tableList.getItem(i)))
		        tableList.replaceItem(newNames[i], i);

		    if ((currentView != null) && (currentView == tables[i]))
			currentViewIdx = i;
	        }
	    }

	    for (int i = newNames.length; i < tableList.countItems(); i++)
		tableList.delItem(i);

	    tableNames = newNames; // Keep this?

	    int selectedIdx = tableList.getSelectedIndex();
	    if (selectedIdx != currentViewIdx) {
		tableList.deselect(selectedIdx);
		if (currentViewIdx >= 0)
		    tableList.select(currentViewIdx);
	    }
	}
	repaint();
    }

    public Dimension preferredSize() {
	return new Dimension(2 * tableList.getPreferredSize().width, 
	    tableList.getPreferredSize().height);
    }

    public void itemStateChanged(ItemEvent e) {
	if (e.getStateChange() != ItemEvent.SELECTED)
	    return;
	try {
	    int currentTableIdx = target.getSelectedTableViewIndex();
	    int selectedIdx = tableList.getSelectedIndex();
//System.out.println("Got action event: " + e);
//System.out.println("   current idx = " + currentTableIdx);
//System.out.println("   selected idx = " + selectedIdx);
	    if ((currentTableIdx != selectedIdx) && (selectedIdx >= 0)) {
	      try {
	        target.setSelectedTableViewIndex(selectedIdx);
		support.firePropertyChange("selectedTableViewIndex",
		    new Integer(currentTableIdx), new Integer(selectedIdx));
	      } catch (NoSuchRecordException ex) {
		// may ocurr until real tables exist
		// swallow this
	      }
	    }
	}
    	catch (InterchangeException ex) {
	    ex.printStackTrace();
    	}
    }

     public void tableViewChanged(TableViewEvent event) {
       //       System.out.println(getClass().getName()+" tableViewChanged="+event);
	updateList();
     }

     public void propertyChange(PropertyChangeEvent evt) {
       //     System.out.println("TableSelectionViewCustomizer.propertyChange "+evt);
	try {
	    if (evt.getPropertyName().equals("selectedTableIndex") &&
	    	(tableList.getSelectedIndex() != 
		target.getSelectedTableViewIndex())) {
		tableList.select(target.getSelectedTableViewIndex());
	    }
	}
    	catch (InterchangeException e) {
	    e.printStackTrace();
    	}
     }

    //----------------------------------------------------------------------
 
    public void addPropertyChangeListener(PropertyChangeListener l) {
        support.addPropertyChangeListener(l);
    }
 
    public void removePropertyChangeListener(PropertyChangeListener l) {
        support.removePropertyChangeListener(l);
    }
 
    private PropertyChangeSupport support = new PropertyChangeSupport(this);
 
    //----------------------------------------------------------------------


}

