package sieve.sluice.views;

import sieve.sluice.*;
import java.util.*;

/**
 * A TranslationView translates one or more fields to Strings based
 * on translation tables provided by the <CODE>addTranslation</CODE>
 * method. Currently we assume that the translations will be from
 * an arbitrary type to Strings, but we may want to relax this.
 *
 * @see LabelView
 */
public class TranslationView extends DefaultTableView implements TableView {
    protected Hashtable	fieldsToTranslate	= null;

    public TranslationView(TableView source) throws InterchangeException {
	super(source);
	fieldsToTranslate = new Hashtable();
    }

    public void addTranslation(String field, Hashtable translation) {
	fieldsToTranslate.put(field, translation);
    }

    public void removeTranslation(String field) {
	fieldsToTranslate.remove(field);
    }

    public FieldDescriptor getFieldDescriptor(String field) throws
	InterchangeException {
	Hashtable labels = null;
	FieldDescriptor fd = super.getFieldDescriptor(field).copy();
	if ((labels = ((Hashtable)fieldsToTranslate.get(field))) != null)
	    fd.setType("java.lang.String");
        return fd;
    }

    public Object[][] get(String[] fields, int startingAt,
        int numberOfRecords) throws InterchangeException {
	Hashtable labels = null;
	Object[][] data = super.get(fields, startingAt, numberOfRecords);

	for (int fld = 0; fld < fields.length; fld++) {
	    if ((labels = ((Hashtable)fieldsToTranslate.get(fields[fld]))) 
		!= null) {
	    	for (int rec = 0; rec < data[0].length; rec++ ) {
		    if (data[fld][rec] != null)
			data[fld][rec] = labels.get(data[fld][rec]);
		}
	    }
	}
	return data;
    }

    public Object[][] get(String[] fields, int startingAt)
        throws InterchangeException {
	return get(fields, startingAt, getRecordCount());
    }

}


