package sieve.sluice.views.tests;

import java.beans.*;
import java.lang.reflect.*;
import java.util.*;

/**
 * Need to think about:
 * - Tests vs. binary relations...
 * - Provide support for binary logical ops?
 */
public class ObjectTest {
    Object first = null;
    Object second = null;
    
    public ObjectTest() {}

    public void setFirst(Object first) {
	this.first = first;
    }

    public void setSecond(Object second) {
	this.second = second;
    }

    public boolean getEquals() {
	if ((first != null) && (second != null))
	    return first.equals(second);
	else 
	    return false;
    }

    public static boolean testObjects(Object first, String test, 
	Object second) {
	Boolean result = new Boolean(false);
	ObjectTest testObj = getObjectTest(first.getClass());
	testObj.setFirst(first);
	testObj.setSecond(second);
	try {
	    Method testMethod = getTestMethod(testObj.getClass(), test);
	    Object[] args = {};
	    result = (Boolean)(testMethod.invoke(testObj, args));
	}
	catch(Exception e) {
            e.printStackTrace();
	}
	return result.booleanValue();
    }

    public static Method getTestMethodForObject(Object obj, String test) {
	ObjectTest testObj = getObjectTest(obj.getClass());
	return getTestMethod(testObj.getClass(), test);
    }

    public static Method getTestMethodForClassName(String classname, 
	String test) throws ClassNotFoundException {
	ObjectTest testObj = getObjectTest(Class.forName(classname));
	return getTestMethod(testObj.getClass(), test);
    }

    public static Method getTestMethod(Class aClass, String test) {
	Method method = null;
	if (! test.toUpperCase().equals(test)) {
	    StringBuffer buf = new StringBuffer(test);

	    // De-capitalize the first character (Not doing this because
	    // we're not using Beans introspector here...
            // if (Character.isUpperCase(buf.charAt(0))) {
            //     buf.setCharAt(0, Character.toLowerCase(buf.charAt(0)));
            // }

            // Remove spaces 
            int len = test.length();
            for (int ch = 1; ch < len; ch++) {
                if (Character.isSpace(buf.charAt(ch))) {
	   	    for (int j = ch; j < len - 1; j++)
                        buf.setCharAt(j, buf.charAt(j + 1));
                    ch--;
                    len--;
		}
            }
	    buf.setLength(len);
	    test = buf.toString();
        }
	try {
	    Class[] args = {};
	    method = aClass.getMethod("get" + test, args);
	}
	catch (Exception e) {
	    System.err.println("Failed to find method get" + test + 
		" for class " + aClass.getName());
	    e.printStackTrace();
	}
	return method;
    }

    public static ObjectTest getObjectTest(Class aClass) {
	ObjectTest test = null;
//System.out.println("Looking test class for: " + aClass.getName());
	while ((test == null) && 
	    (!aClass.getName().equals("java.lang.Object"))) {
	    try {
		String name = aClass.getName();
		name = "sieve.sluice.views.tests." +
		    name.substring(name.lastIndexOf('.') + 1) + "Test";
//System.out.println("Looking for: " + name);
	        Class testClass = Class.forName(name);
                test = (ObjectTest) testClass.newInstance();
            }
	    catch (Exception e) {
		aClass = aClass.getSuperclass();
	    }
	}
	if (test == null)
	    test = new ObjectTest();
	return test;
    }

    /**
     * Returns a list of tests available for the specified class. We
     * will need a way to register new ones of these.
     */
    public static String[] getTests(Class aClass) {
        String[] tests = null;
	ObjectTest test = getObjectTest(aClass);
        try {
  	    BeanInfo bi = Introspector.getBeanInfo(test.getClass());
	    PropertyDescriptor props[] = bi.getPropertyDescriptors();
	    Vector boolProps = new Vector();

	    // Look for properties that return Boolean
	    for (int i = 0; i < props.length; i++) {
	        if (props[i].getPropertyType().getName().equals(
		    "boolean"))
		    boolProps.addElement(props[i].getName());
	    }

	    // Clean up the property names and put them in an array
            tests = new String[boolProps.size()];
	    for (int i = 0; i < tests.length; i++) {
		String name = (String)boolProps.elementAt(i);
		// Leave all-caps properties alone
		if (! name.toUpperCase().equals(name)) {
		    StringBuffer buf = new StringBuffer(name);
		    // Capitalize the first character
                    if (Character.isLowerCase(buf.charAt(0))) {
                        buf.setCharAt(0, Character.toUpperCase(buf.charAt(0)));
                    }
                    // Add Spaces before any other capital letters.
                    int len = name.length();
                    for (int ch = 1; ch < len; ch++) {
                        if (Character.isUpperCase(buf.charAt(ch))) {
                            buf.insert(ch, ' ');
                            ch++;
                            len++;
                        }
                    }
		    name = buf.toString();
		}
		tests[i] = name;
	    }
	}
        catch (Exception e) {
	    e.printStackTrace();
	}
        return tests; 
    }

    public static String[] getTests(String className) {
	String[] tests = null;
	try {
	    tests = getTests(Class.forName(className));
	}
	catch (Exception e) {
	    e.printStackTrace();
        }
	return tests;
    }

    public static String[] getTests(Object anObject) {
	return getTests(anObject.getClass());
    }

}

