package sieve.util;

import com.sun.java.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.util.*;
import java.applet.Applet;

/**
 * A small panel with the checkboxes, to help control layout.
 * The panel itself is an ItemListener, notified when the state of the
 * radio buttons changes. It takes ActionListeners and notifies of the 
 * buttonGroup selection change with ActionEvents.
 *  
 * @version	%I%, %G%
 * @author	Lynn W. Jones
 */

public class SwingButtonGroupBox extends JPanel implements ItemListener {
//a small panel with the checkboxes, to help control layout
	public  ButtonGroup		grp = null;
	private JRadioButton	initTrue;
	private Vector			radioListeners;

	public SwingButtonGroupBox(int axis, int num, String names[], int whichTrue) {
    /**
	 * axis == BoxLayout.Y_AXIS or BoxLayout.X_AXIS, for direction of layout
	 * num == number of checkboxes to be created
	 * names[] == the labels for each checkbox
	 * whichTrue == index of label for initially selected box (starts with 0, not 1)
	 */
		super(true);
		setLayout(new BoxLayout(this,axis));
		setOpaque(true);
		radioListeners = new Vector();	//where to keep the added ActionListeners
		JRadioButton tempBtn;
		grp = new ButtonGroup(); 
		for (int i=0;i<num;i++) {
			if (i == whichTrue) {
				tempBtn = new JRadioButton(names[i],true);
				initTrue = tempBtn;	//keep a handle on this one for resetting
			}
			else 
				tempBtn = new JRadioButton(names[i],false);
			tempBtn.setActionCommand(names[i]);
			tempBtn.addItemListener(this);
			add(tempBtn);
			grp.add(tempBtn);
		}
	}

	public void addNotify() {
		super.addNotify();
	}

	public void addActionListener(ActionListener l) {
	/**
	 * Add a listener to notify when we get an item event from the radio buttons. 
	 */
		radioListeners.addElement(l);
	}

	public void removeActionListener(ActionListener l) {
		radioListeners.removeElement(l);
	}

	public void itemStateChanged(ItemEvent e) {
	/**
	 * notify listeners that there is a new selection in the radio group
	 */
		if(e.getStateChange() == ItemEvent.SELECTED) {
			//we get 2 events, one from each button changed. Only broadcast
			//the new selection event.
			String command = (grp.getSelection()).getActionCommand();
			int num = radioListeners.size();
			Vector temp = (Vector)(radioListeners.clone());	//clone to prevent changes from elsewhere (removeListener)
			for (int i=0; i<num; i++)						//  from affecting this iteration
				((ActionListener)(temp.elementAt(i))).actionPerformed(new ActionEvent(this,0,command));
		}
	}

	public String getCurrentCommand() {
	/**
	 * returns command of currently selected radioButton
	 */
		return grp.getSelection().getActionCommand();
	}

	public void setBackground(Color bg) {
	/**
	 * overridden to explicitly set button background same as panel
	 */
		Enumeration e;
		super.setBackground(bg);	
		if (grp != null) {			//need this test since setBackground also called
			e = grp.getElements();	//...in construction and will cause null
				if (e != null) {	//...pointer error (I think this is why...)
				while(e.hasMoreElements()) 
					((Component)(e.nextElement())).setBackground(bg);
			}
		}
	}

	public Enumeration getElements() {
	/**
	 * returns the radio buttons. Useful for setting text or background colors...
	 */
		return grp.getElements();
	}
	
	public void restart() {
	/**
	 * resets checkbox to the initial true which was set in constructor
	 */
		grp.setSelected(initTrue.getModel(), true);
	}
			
}


