package sieve.whiteboard;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import sieve.workbench.*;
 
/**
 * A Box component. These are created by the BoxTool.
 * @see BoxTool
 */
public class Box extends RectangularComponent implements WorkbenchComponent, 
    MouseMotionListener, MouseListener {
    private boolean filled = false;

    static {
	ToolFactory.getToolFactory().registerTool(Box.class, BoxTool.class);
    }

    public Box(Point start, Point end, Workbench wb) {
	super(start, end, wb);
    }

    public boolean getFilled() {
        return filled;
    }
    public boolean isFilled() {
        return filled;
    }
    public void setFilled(boolean filled) {
        this.filled = filled;
        repaint();
    }

    public void paint(Graphics g) {
        g.setColor(getColor());
        if (filled)
            g.fillRect(Math.min(start.x, end.x),
                    Math.min(start.y, end.y),
                    Math.abs(end.x - start.x),
                    Math.abs(end.y - start.y));
        else
            g.drawRect(Math.min(start.x, end.x),
                    Math.min(start.y, end.y),
                    Math.abs(end.x - start.x),
                    Math.abs(end.y - start.y));
	super.paint(g);
    }

    public boolean contains(int x, int y) {
	if (!constructed)
	    return false;
        if ((isSelected() && (startHandle.contains(x, y) || 
	    endHandle.contains(x, y) ||
            startHandle2.contains(x, y) || endHandle2.contains(x, y))))
            return true;
 
        Rectangle bounds = new Rectangle(Math.min(start.x, end.x) -1,
                Math.min(start.y, end.y) - 1, Math.abs(end.x - start.x) + 2,
                Math.abs(end.y - start.y) + 2);
	if (filled)
	    return bounds.contains(x, y);
	else {
            Rectangle innerBounds = new Rectangle(Math.min(start.x, end.x) + 4,
                Math.min(start.y, end.y) + 4, Math.abs(end.x - start.x) - 8,
                Math.abs(end.y - start.y) - 8);
            return bounds.contains(x, y) && !innerBounds.contains(x, y);
	}
    }

}



