package sieve.whiteboard;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import sieve.workbench.*;
 
/**
 * A Oval component. These are created by the OvalTool.
 * @see OvalTool
 */
public class Oval extends RectangularComponent implements 
    WorkbenchComponent, MouseMotionListener, MouseListener {
    private boolean filled = false;

    static {
	ToolFactory.getToolFactory().registerTool(Oval.class, OvalTool.class);
    }

    public Oval(Point start, Point end, Workbench wb) {
	super(start, end, wb);
    }

    public boolean getFilled() {
        return filled;
    }
    public boolean isFilled() {
        return filled;
    }
    public void setFilled(boolean filled) {
        this.filled = filled;
        repaint();
    }

    public void paint(Graphics g) {
        g.setColor(getColor());
	if (filled)
            g.fillOval( Math.min(start.x, end.x), 
	     	    Math.min(start.y, end.y), 
		    Math.abs(end.x - start.x), 
		    Math.abs(end.y - start.y));
	else
            g.drawOval( Math.min(start.x, end.x), 
	     	    Math.min(start.y, end.y), 
		    Math.abs(end.x - start.x), 
		    Math.abs(end.y - start.y));
	super.paint(g);
    }

    public boolean contains(int x, int y) {
	if (!constructed)
	    return false;
        if ((isSelected() && (startHandle.contains(x, y) ||
            endHandle.contains(x, y) ||
            startHandle2.contains(x, y) || endHandle2.contains(x, y))))
            return true;
 
        Rectangle bounds = new Rectangle(Math.min(start.x, end.x) -1,
                Math.min(start.y, end.y) - 1, Math.abs(end.x - start.x) + 2,
                Math.abs(end.y - start.y) + 2);
	int tx = x - (bounds.x + bounds.width/2);
	int ty = y - (bounds.y + bounds.height/2);

	double r = ((double)(tx * tx)/(double)(bounds.width/2 * bounds.width/2)
	    + (double)(ty * ty)/(double)(bounds.height/2 * bounds.height/2));

        if (filled)
            return (r < 1.1);
        else 
            return Math.abs(r - 1.0) < 0.1;
    }

}

