package sieve.whiteboard.text;

import java.awt.*;
import java.awt.event.*;
import sieve.workbench.*;
import sieve.workbench.event.*;

/**
 * SharedTextWrapperTool is the tool used to create SharedTextWrapper objects.
 * Its primary function is to wrap the SharedTextWrapper in our WrapperWrapper
 * class, rather than in Sieve's DefaultWrapper class.
 */
public class SharedTextWrapperTool extends SimpleTool implements Tool {
    Workbench wb = null;
    Class     cl = null;

    public void init(Workbench wb, Class cl) {
	this.wb = wb;
	this.cl = cl;
    }

    public void mouseReleased(MouseEvent e) { 
	try {
	    Cursor c = wb.getCursor();
	    wb.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
	    DefaultWrapper wrapper = 
		new WrapperWrapper(wb, new SharedTextWrapper());
	    wb.add(wrapper);
	    wrapper.setLocation(e.getPoint());
	    wb.setCursor(c);
	    wb.componentAdded(wrapper, this);
	}
	catch (Exception ex) {
	    ex.printStackTrace();
	}
	
    }
}

/**
 * WrapperWrapper wraps a SharedTextWrapper. It extends DefaultWrapper and
 * will therefore propagate any property changes in the usual way. It also
 * registers itself as a SharedDocumentListener, and therefore receives
 * commands describing any changes made to the editor. Such commands are
 * then wrapped in a SharedTextEditEvent and then forwarded to the Workbench,
 * which will (if we're being used collaboratively) then send them to all
 * other sessions.
 * <P>
 * This can't be inner because we have to serialize it :-(
 */
class WrapperWrapper extends DefaultWrapper implements SharedDocumentListener {

    private transient SharedTextWrapper stw = null;
    private transient Workbench workbench = null;

    public WrapperWrapper(Workbench wb, SharedTextWrapper model) {
	super(wb, model);
	this.stw = model;
	this.workbench = wb;
	((SharedDocument)stw.getSharedTextArea().getDocument()
		).addSharedDocumentListener(this);
    }

    public void processCommand(String command) {
	workbench.componentEdited(this, new SharedTextEditEvent(this, command));
    }
}

// This can't be inner because we have to serialize it :-(
class SharedTextEditEvent extends EditEvent {
    private String command = null;

    public SharedTextEditEvent(Object src, String command) {
	super(src);
	this.command = command;
    }

    public String getCommand() {
	return command;
    }

    /**
     * Passes the command to the SharedTextWrapper.
     */
    public void applyChange(Object target) {
	if (target instanceof SharedTextWrapper) 
	    ((SharedTextWrapper)target).setTextCommand(command);
    }

    /**
     * Always returns false. This forces all of these messages to
     * be replayed.
     */
    public boolean isObsoletedBy(EditEvent newEvent) {
	return false;
    }

    /**
     * Returns true if the commands match. This should probably
     * be improved a bit, for the unlikely event that identical
     * changes occur simultaneously to two different text areas.
     */
    public boolean isEquivalentTo(EditEvent newEvent) {
	return (newEvent instanceof SharedTextEditEvent) &&
	    command.equals(((SharedTextEditEvent)newEvent).getCommand());
    }

}




