package sieve.workbench;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.applet.*;
import java.util.*;
import java.net.InetAddress;
import sieve.workbench.event.*;
import sieve.workbench.collab.*;
import vt.cs.widgets.*;

import com.sun.java.swing.*;

/**
 * A presentable client for the Sieve workbench.
 */
public class Client extends JComponent {
    private static String SESSION_NAME = "Demo Client Session";

    private Workbench wb      = null;
    private CollaborationSupport cs = null;
	
	//the SingleUserSupport performs like the CollaborationSupport,
	//"capturing" all event messages so that the workspace can be
	//saved and restored locally as it can now from the server
	private SingleUserSupport sus = null;

    private WorkbenchClearer clearer = null;
    private ToolBox   toolBox = null;
    private JFrame    toolBoxFrame = null;
    private JFrame    editor = null;
    private MenuBar  menuBar = null;
    private RadarPane radarPane = null;

    private static Frame     f = null;
    private static String    host = null;
    private static JFrame    pbf = null;
    private static JLabel    pbl = null;
    private static JProgressBar pb = null;
    private static int	     pbCount = 0;

    public Client() { }

    private void initialize(String host, String session,
	String channel, String user, Color color) {
	setLayout(new BorderLayout());
	wb = new Workbench();
	clearer = new WorkbenchClearer();
	wb.addWorkbenchListener(clearer);

	incrProgressBar("Loading Tools...");
	toolBoxFrame = createToolBox(wb);
	toolBoxFrame.setLocation(50, 50);
	editor = createPropertyEditor(wb);
	editor.setLocation(50, 60 + toolBoxFrame.getSize().height);

	menuBar = createMenuBar(wb, toolBoxFrame, toolBox, clearer, editor);

	incrProgressBar("Connecting to Server...");
	cs = new CollaborationSupport(wb, host, session,
            channel, user, color);
	radarPane = cs.getRadarPane();
        radarPane.setScaler(new WorkbenchScaler(wb));
	add("Center", radarPane);
	wb.setSize(new Dimension(1500, 1500));
	//((JFrame) getTopLevelAncestor()).setJMenuBar(menuBar);
	// add("North", menuBar);
	incrProgressBar("Restoring Workbench State...");
	cs.restoreWorkbench();
	//if give dialog to select saved session at startup need to do 
	//sus.restoreWorkbench();

		//Lynn added:
	sus = new SingleUserSupport(wb); 

    }

    public MenuBar getMenuBar() {
	return menuBar;
    }

    protected JFrame createToolBox(Workbench wb) {
	JFrame tb = new JFrame("Tools");
	tb.getContentPane().setLayout(new BorderLayout());
	toolBox = new ToolBox(200, 200, wb);
	loadJarFiles(toolBox);
        tb.getContentPane().add(toolBox);
	tb.pack();
	tb.setSize(new Dimension(200, 150));
	tb.setVisible(true);
	return tb;
    }

    // This should be moved to toolBox
    protected void loadJarFiles(ToolBox toolBox) {
        File cwd = new File(System.getProperty("user.dir"));
        File jars = new File(cwd, "jars");
 
        if (! jars.isDirectory()) {
            System.err.println(jars+" is not a directory!!");
        }
        Vector result = new Vector();
        String names[];
        names = jars.list(new JarFileFilter());
	if (names != null) {
          for (int i=0; i<names.length; i++) 
            toolBox.addJarFile(jars.getPath() + File.separatorChar + names[i]);
        }

    }

    protected JFrame createPropertyEditor(Workbench wb) {
	EditorWindow pe = new EditorWindow("Editor");
	wb.setWorkbenchComponentEditor(pe);
	pe.pack();
	pe.setSize(new Dimension(200, 200));
	pe.setVisible(true);
	return pe;
    }
    private class EditorWindow extends JFrame implements 
	WorkbenchComponentEditor {
	private DefaultPropertyEditor ed = null; 
	public EditorWindow(String name) {
	    super(name);
	    ed = new DefaultPropertyEditor(wb);
	    getContentPane().setLayout(new BorderLayout());
	    getContentPane().add(ed);
	    pack();
	}
	public void setTarget(WorkbenchComponent wbc) {
	    ed.setTarget(wbc);
	    if( wbc == null)
	        setTitle("(No component selected)");
	    else
	        setTitle("Editing: " + ed.getTargetName());
	}
    }

    protected MenuBar createMenuBar(final Workbench wb, 
	final JFrame toolBoxFrame, final ToolBox toolBox, 
	final WorkbenchClearer clearer, JFrame editor) {
	MenuBar menuBar = new MenuBar();

	// Set up the file menu
	Menu file = new Menu("File");
	// file.getPopupMenu().setLightWeightPopupEnabled(false);

	MenuItem loadJar = new MenuItem("Load Tools...");
	// Add a listener that opens a file dialog to load a jar file
	loadJar.addActionListener(
	    new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    FileDialog fd = new FileDialog(toolBoxFrame,
			"Load Jar File", FileDialog.LOAD);
		    fd.setFilenameFilter(new JarFileFilter());
		    fd.show();
		    String name = fd.getFile();
		    if (name != null) {
			toolBox.addJarFile(fd.getDirectory() + 
			    java.io.File.separator + name);
			// Just in case new link types were in this JAR
			LinkFactory.getLinkFactory().clearCache();
		    }
		}
	    }
	);
	// Local jar loading is disabled in collaborative mode. We need to
	// think about how this should work. Only allowing loading from
	// jars on the net? Ship the file to everyone and store a copy on
	// the server? ---> see below
	loadJar.setEnabled(host == null);
	file.add(loadJar);

	MenuItem loadLocalSession = new MenuItem("Load Snapshot...");
	loadLocalSession.addActionListener(
		new ActionListener() {
		public void actionPerformed(ActionEvent et) {
			FileDialog fd = new FileDialog(toolBoxFrame,"Load Session Snapshot", FileDialog.LOAD);
		    //fd.setFilenameFilter(/*make a filter here for .ser files: new JarFileFilter()*/);
		    fd.show();
		    String name = fd.getFile();
		    if (name != null) {
				//singleUserSupport should read the file and replay the messages
				sus.loadSnapshot(name);
		    }
		}
		}
	);
	loadLocalSession.setEnabled(true);
	file.add(loadLocalSession);

	MenuItem saveLocalSession = new MenuItem("Save Snapshot...");
	saveLocalSession.addActionListener(
		new ActionListener() {
		public void actionPerformed(ActionEvent et) {
			FileDialog fd = new FileDialog(toolBoxFrame,"Save Session Snapshot", FileDialog.SAVE);
		    //fd.setFilenameFilter(/*make a filter here for .ser files: new JarFileFilter()*/);
		    fd.show();
		    String name = fd.getFile();
		    if (name != null) {
				//singleUserSupport should read the file and replay the messages
				sus.saveSnapshot(name);
		    }
		}
		}
	);
	saveLocalSession.setEnabled(true);
	file.add(saveLocalSession);

	
	MenuItem exit = new MenuItem("Exit");
	// Add a listener which (for now) just exits. This will need to
	// shut down our collaboration support eventually.
	exit.addActionListener( 
	    new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    wb.setCursor(
			Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		    cs.shutdown();
		    System.exit(0);
		}
	    }
	);
	file.add(exit);
	menuBar.add(file);

	// Set up the edit menu
	Menu edit = new Menu("Edit");
	// edit.getPopupMenu().setLightWeightPopupEnabled(false);
	final MenuItem clear = new MenuItem("Clear");
	clear.setEnabled(false);
	// Enable "Clear" only when something is selected
	wb.addWorkbenchListener(
	    new WorkbenchListenerAdapter() {
		public void componentSelected(WorkbenchComponentEvent e) {
		    clear.setEnabled(true);
		}
		public void componentDeselected(WorkbenchComponentEvent e) {
		    clear.setEnabled(false);
		}
	    }
	);
	// Let clear remove the current selection
	clear.addActionListener(
	    new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    WorkbenchComponent wbc = wb.getCurrentSelection();
            	    if (wbc != null)
                	wbc.remove();
		}
	    }
	);
	edit.add(clear);

	// Make a "Clear all" menu item which invokes the clearer's
	// clear() method.
	MenuItem clearAll = new MenuItem("Clear all");
	clearAll.addActionListener(
	    new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
	    	    int confirm = JOptionPane.showConfirmDialog(wb, 
			"Remove all components?",
			"Confirm Clear", JOptionPane.OK_CANCEL_OPTION);
		    // confirm will be 0 if the user clicks "OK"
		    if (confirm == 0)
		        clearer.clear();
		}
	    }
	);
	// Temporarily disabling this in collaborative mode. We need to
	// make the clearer smart enough to not blow away 
	// participant objects.
	// clearAll.setEnabled(host == null);
	edit.add(clearAll);

	MenuItem participant = new MenuItem("Edit Participant Info...");
	participant.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
		    ParticipantDialog.showParticipantDialog(f, wb);
                }
            }
        );
	participant.setEnabled(host != null);
	edit.addSeparator();
	edit.add(participant);


	menuBar.add(edit);

	// Setup the Options menu
	Menu options = new Menu("Options");
	// options.getPopupMenu().setLightWeightPopupEnabled(false);
	WindowMenuItem showToolBox = 
	    new WindowMenuItem("Show Tools", toolBoxFrame);
	options.add(showToolBox);
	WindowMenuItem showEditor = new WindowMenuItem("Show Editor", editor);
	options.add(showEditor);
	CheckboxMenuItem showBorder = new CheckboxMenuItem(
	    "Show Object Borders");
	// showBorder.setSelected(DefaultWrapper.getShowBorders());
	showBorder.setState(DefaultWrapper.getShowBorders());
	showBorder.addItemListener(
	    new ItemListener() {
	   	public void itemStateChanged(ItemEvent evt) {
		    DefaultWrapper.setShowBorders(
		    	evt.getStateChange() == ItemEvent.SELECTED);
		    wb.repaint();
		}
	    }
	);
	options.add(showBorder);

	menuBar.add(options);
	return menuBar;
    }

    private static void incrProgressBar(String message) {
	pb.setValue(++pbCount);
	pbl.setText(message);
	pbf.show();
    }

    public static void main(String[] args) {
	// See if they just need help
	if (getArg("-help", args) != null) {
	    System.err.println("Usage: java sieve.workbench [-host hostname] "+
		" [-user username] [-chan channelname]");
	    System.exit(0);
	}

	// Set up a simple progress bar
	pbf = new JFrame("Sieve");
	JPanel pbp = new JPanel();
	pbp.setBorder(new 
	    com.sun.java.swing.border.EmptyBorder(20, 20, 20, 20));
	pbf.setContentPane(pbp);
	pbf.getContentPane().setLayout(
                new BoxLayout(pbf.getContentPane(), BoxLayout.Y_AXIS));
	Dimension d = new Dimension(400, 20);
	pbl = new JLabel("Loading, please wait...");
	pbl.setMaximumSize(d);
	pbf.getContentPane().add(pbl);
	pbf.getContentPane().add(Box.createRigidArea(new Dimension(1,10)));
	pb = new JProgressBar();
	pbl.setLabelFor(pb);
	pb.setMinimum(0);
	pb.setMaximum(4);
	pb.setValue(0);
	pbf.getContentPane().add(pb);
	pbf.getContentPane().add(Box.createRigidArea(new Dimension(1,20)));
	pbf.setSize(400, 110);
	Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        pbf.setLocation(screenSize.width/2 - 400/2,
              screenSize.height/2 - 100/2);
	pbf.show();

	// Re-direct standard error to a log file
	try {
	    System.out.println(
		"NOTE: Errors are being saved in \"sievelog.txt\"");
	    FileOutputStream fos = new FileOutputStream("sievelog.txt", true);
	    PrintStream ps = new PrintStream(fos, true);
	    ps.print("** Sieve session started " + new Date() + ". Args: ");
	    for (int i = 0; i < args.length; i++) 
		ps.print(args[i] + " ");	
	    ps.println();
	    ps.println();
	    System.setErr(ps);
	    // System.setOut(ps);
	}
	catch(Exception e) {
	    e.printStackTrace();
	}

	// Reset the local hostname property. This is to get around problems
	// with getLocalHost().getName() in Win95, Solaris 2.4, and probably
	// others.
        try {
            String myAddress = InetAddress.getLocalHost().getHostAddress();
            Properties props = System.getProperties();
            props.put("java.rmi.server.hostname", myAddress);
            System.setProperties(props);
        }
        catch(Exception e) {
            e.printStackTrace();
        }

	// "-hc" is a debug flag. It forces the client to load the
	// hard-coded classes in the Toolbox.
	if ((getArg("-hc", args) != null) || (getArg("-hardcoded", args) != null)) {
	     ToolBox.setLoadHardCodedToolsets(true);
	}

	f = new Frame("Sieve Workbench (Single User Mode)");
	Client cl = new Client();

	// Find a user name
	String user = getArg("-user", args);
	if (user == null)
	    user = System.getProperty("user.name", "local user");
	if (user.equals("unknown"))
	    user = "local user";

	host = getArg("-host", args);
	if ((host != null) && ! WorkbenchChannelManager.isServerUp(host)) {
	    // or WARNING_MESSAGE
	    JOptionPane.showMessageDialog(f, 
		"Sieve cannot connect to the server on host " + host + ".", 
		"Cannot connect to server", 
		JOptionPane.INFORMATION_MESSAGE);
	    host = null;
	}
	
	String channel = getArg("-chan", args);
	if (host != null) {
	    if (channel == null) {
		WorkbenchChannelManager mgr = 
		    WorkbenchChannelManager.getManager(host, SESSION_NAME);
		JComboBox cb = new JComboBox();
      		cb.setEditable(true);
		String[] channels = mgr.getAvailableChannels();
		for (int c = 0; c < channels.length; c++)
		    cb.addItem(channels[c]);
		Object[] message = {"Choose or enter a channel:", cb};
		String[] options = {"OK"};
		JOptionPane.showOptionDialog(f, message, 
		     "Choose Sieve Channel",
		     JOptionPane.DEFAULT_OPTION, 
		     JOptionPane.QUESTION_MESSAGE,
		     null, options, options[0]);
		
		channel = cb.getSelectedItem().toString();
		if ((channel == null) || (channel.length() == 0))
	            channel = "Default Channel";
	    }
	    f.setTitle("Sieve Workbench: " + channel);
	}
	else {
	    System.out.println("Starting Sieve in single-user mode. Use "+
	     "the \"-host <hostname>\" \nflag to connect to a collaborative "+
	     "session. Other optional parameters \nare \"-user <username>\" "+
	     "and \"-chan <channelname>\".");
	}

	cl.initialize(host, SESSION_NAME, channel, user, Color.black);
	// f.setJMenuBar(cl.getMenuBar());
	// f.getContentPane().add(cl);
	f.setMenuBar(cl.getMenuBar());
	f.add(cl);
	f.setSize(new Dimension(400, 500));
	f.setVisible(true);
	f.setLocation(260, 50);
	pbf.hide();
	pbf.dispose();
    }

    private static String getArg(String parameter, String args[]) {
        int length = args.length;
        for (int i = 0; i < length; i++) {
            if (args[i].equals(parameter)) {
                if (++i < length) 
                    return(args[i]);
		else
		    return "";
            }
        }
        return null;
    }

    // private class WindowMenuItem extends JCheckBoxMenuItem implements
    private class WindowMenuItem extends CheckboxMenuItem implements
	ItemListener, WindowListener {
	private Rectangle bounds = null;
	private Window win = null;

	public WindowMenuItem(String name, Window win) {
	    super(name);
	    this.win = win;
	    win.addWindowListener(this); 
	    // this.setSelected(win.isVisible());
	    this.setState(win.isVisible());
	    this.addItemListener(this);
	}

	public void windowClosing(WindowEvent evt) {
	    win.setVisible(false);
	    // setSelected(false);
	    setState(false);
	}

	public void itemStateChanged(ItemEvent evt) {
	    boolean show =
		evt.getStateChange() == ItemEvent.SELECTED;
	    if (show && (bounds != null))
		win.setBounds(bounds);
	    else
		bounds = win.getBounds();
	    win.setVisible(show);
	}
	public void windowActivated(WindowEvent evt) {}
	public void windowDeactivated(WindowEvent evt) {}
	public void windowClosed(WindowEvent evt) {}
	public void windowDeiconified(WindowEvent evt) {}
	public void windowIconified(WindowEvent evt) {}
	public void windowOpened(WindowEvent evt) {}
    }

    private class JarFileFilter implements FilenameFilter {
	public boolean accept(File dir, String name) {
	    return name.endsWith("jar");
	}
    }

    /**
     * WorkbenchClearer listens to the workbench and tracks the current
     * set of existing links and components. When the <CODE>clear</CODE>
     * method is called, all links and components are removed from
     * the workbench.
     */
    private class WorkbenchClearer extends WorkbenchListenerAdapter {
	private Vector links = null;
	private Vector components = null;

	public WorkbenchClearer() {
	    links = new Vector();
	    components = new Vector();
	}

        public void linkAdded(LinkEvent e) {
	    links.addElement(e.getLink());
        }
        public void linkRemoved(LinkEvent e) {
	    links.removeElement(e.getLink());
  	}
     	public void componentAdded(WorkbenchComponentEvent e) {
	    components.addElement(e.getWorkbenchComponent());
	}
     	public void componentRemoved(WorkbenchComponentEvent e) {
	    WorkbenchComponent wbc = e.getWorkbenchComponent();
	    components.removeElement(wbc);
	    // While we're at it, remove any links to or from this guy IF
	    // it wasn't some sort of app-specific component
	    if (wbc instanceof DefaultWrapper) {
		synchronized(links) {
		    for (int l = links.size(); l > 0;) {
			l--;
			Link link = (Link)links.elementAt(l);
			if ((link.getSourceComponent() == wbc) ||
			   (link.getDestComponent() == wbc)) {
			   link.remove();
			}
		    }
		}
	    }
	}
	public void clear() {
	    Vector linkClone = (Vector) links.clone();
	    Vector compClone = (Vector) components.clone();
	    for (Enumeration l = linkClone.elements(); l.hasMoreElements();)
	        ((Link)l.nextElement()).remove();
	    for (Enumeration c = compClone.elements(); c.hasMoreElements();) {
		WorkbenchComponent wbc = (WorkbenchComponent)c.nextElement();
		if (! (wbc instanceof WorkbenchParticipant))
	            wbc.remove();
	    }

	    // What the heck, as long as we're here...
	    System.gc();
	}

    }
}


