package sieve.workbench;

import java.util.*;
import java.beans.*;

// Registration must support multiple entries for each (src, dest) pair
// Also support registration of LinkCompatibilityPolicy objects?

public class LinkFactory {
    private static LinkFactory instance = null;

    private Hashtable map = null;
    private Hashtable cache = null;

    private LinkFactory() {
	map = new Hashtable();
	cache = new Hashtable();
    }
    
    public static LinkFactory getLinkFactory() {
	if (instance == null)
	    instance = new LinkFactory();
	return instance;
    }

    public void registerLink(Class sourceClass, Class destClass, 
	Class linkClass) {
	cache.clear();
	Hashtable ht = (Hashtable)map.get(sourceClass);
	if (ht == null) {
	    ht = new Hashtable();
	    map.put(sourceClass, ht);
	}
	Vector v = (Vector)ht.get(destClass);
	if (v == null) {
	    v = new Vector();
	    ht.put(destClass, v);
	}
	v.addElement(linkClass);
    }

    public void clearCache() {
	cache.clear();
    }

    private String getShortName(Class cl) {
	String name = cl.getName();
	name = name.substring(name.lastIndexOf(".") + 1, name.length());
	return name;
    }
    private String getPackageName(Class cl) {
	String name = cl.getName();
	name = name.substring(0, name.lastIndexOf("."));
	return name;
    }

    private void addRegisteredLinks(Class src, Class dest, Vector v) {
	Hashtable ht = (Hashtable) map.get(src);
	if (ht != null) {
	    Vector regLinks = (Vector)ht.get(dest);
	    if (regLinks != null) {
		for (Enumeration e = regLinks.elements(); e.hasMoreElements();)
		    v.addElement(e.nextElement());
	    }
	}
    }

    private void checkClass(Class cl, Vector foundClasses, String name,
	Vector checkedClasses) {
	if (!checkedClasses.contains(name)) {
	    try {
		checkedClasses.addElement(name);
	        Object obj = Beans.instantiate(cl.getClassLoader(), name);
	        cl = obj.getClass();
	        if (Link.class.isAssignableFrom(cl))
		    foundClasses.addElement(cl);
	    }
	    catch(Exception e) { }
	}
    }

    public Class[] getLinkTypes(WorkbenchComponent srcComponent, 
	WorkbenchComponent destComponent) {
	return getLinkTypes(srcComponent.getModel().getClass(),
	    destComponent.getModel().getClass());
    }

    public Class[] getLinkTypes(Class srcClass, Class destClass) { 
	// This should possibly also include classloader info, or be
	// a record containing the two classes rather than a string.
	String key = srcClass.getName() + "-->" + destClass.getName();

	Class[] classes = (Class[])cache.get(key);
	if (classes != null)
	    return classes;

	Vector options = new Vector();

	// Keep track of the names we've looked at
	Vector checkedClasses = new Vector();

	while (srcClass != null) {
	    String srcFullName = srcClass.getName();
	    String srcPackage = getPackageName(srcClass);
	    String srcName = getShortName(srcClass);
	    Class dest = destClass;
	     
	    while (dest != null) {

		// First see if anything is explicitly registered
		addRegisteredLinks(srcClass, dest, options);

		// Check everything in the source class's package
		if (! srcFullName.startsWith("java."))
		    checkClass(srcClass, options,
		        srcFullName + getShortName(dest) + "Link",
			checkedClasses);

		Class[] destInts = dest.getInterfaces();
		for (int i = 0; i < destInts.length; i++) {
		    addRegisteredLinks(srcClass, destInts[i], options);
		    if (! srcFullName.startsWith("java."))
		        checkClass(srcClass, options,
		            srcFullName + getShortName(destInts[i]) + "Link",
			    checkedClasses);
		}

		Class[] srcInts = srcClass.getInterfaces();
		for (int i = 0; i < srcInts.length; i++) {

		    String srcIntName = srcInts[i].getName();
		    boolean isJavaInterface = srcIntName.startsWith("java.");

		    addRegisteredLinks(srcInts[i], dest, options);
		    if (! isJavaInterface)
		        checkClass(srcInts[i], options,
		            srcIntName + getShortName(dest) + "Link",
			    checkedClasses);

		    for (int j = 0; j < destInts.length; j++) {
		        addRegisteredLinks(srcInts[i], destInts[j], options);
		    	if (! isJavaInterface)
		            checkClass(srcInts[i], options, srcIntName + 
				getShortName(destInts[j]) + "Link",
				checkedClasses);
		    }
		}
		dest = dest.getSuperclass();    
	    }
	    srcClass = srcClass.getSuperclass();
	}

	// Always make our no-op "simple" link the last option
	options.addElement(SimpleLink.class);

	classes = new Class[options.size()];
	options.copyInto(classes);
	cache.put(key, classes);

	checkedClasses = null;
	options = null;
	return classes;
    }

}

