package sieve.workbench;
/**
 * 
 */

import java.awt.*;
import java.awt.event.*;

public interface WorkbenchComponent {

    /**
     * Returns a uniqueID for this component.
     */
    public UniqueID getUniqueID();

    /**
     * Sets the uniqueID for this component.
     */
    public void setUniqueID(UniqueID id);

    /**
     * Sets a source for the preferred color for this component.
     * Components may use this information in any appropriate ways. For
     * components which display handles or borders, this color should
     * be used for those items.
     */
    public void setColorSource(ColorSource cs);

    /**
     * Returns the editable/introspectable object that is currently
     * selected. For components which are wrappers, this should return
     * the wrapped object. For aggregates components, this should return
     * the currently selected subcomponent. Other components may simply
     * return a reference to themselves.
     * <I>(The "model" in MVC. This was formerly <CODE>getObject</CODE>)</I>
     */
    public Object getModel();

    /**
     * Returns the visual representation of the component. This may or
     * may not be the same as the editable/introspectable object.
     * <I>(The "view" in MVC. This was formerly <CODE>getComponent</CODE>)</I>
     */
    public Component getView();

    /**
     * Returns a component which can be used to edit the component.
     * If this method returns null, then introspection will be used to
     * generate an editor.
     * <I>(The "controller" in MVC. Shouldn't this just use the Customizer
     * mechanism?)</I>
     */
//    public Component getEditor();
    

    /**
     * Sets the component to be selected or deselected
     */
    public void setSelected(boolean selected);

    /**
     * Returns true if the component is currently selected.
     */
    public boolean isSelected();


    /**
     * Removes the component from the workbench.
     */
    public void remove();

}



