package sieve.workbench;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;
import sieve.workbench.event.*;
import sieve.workbench.collab.*;

import vt.cs.widgets.*;

/**
 * WorkbenchScaler objects know how to generate scaled representations of
 * the workbench contents. (eg, for use by RadarPane)
 */
public class WorkbenchScaler extends Scaler {
    private static Hashtable scalers = new Hashtable();

    public static void addScaler(Class cl, Scaler sc) {
	if (sc == null)
	    scalers.remove(cl);
	else
	    scalers.put(cl, sc);
    }

    public WorkbenchScaler(Container c) {
	super(c);
    }

    public void paintComponent(Graphics g, Component c,
	Rectangle scaleBounds, Rectangle fullBounds) {
	Class superClass = c.getClass();
	Scaler sc = null;
	while ((sc == null) && (superClass != null)) {
	    sc = (Scaler)scalers.get(superClass);
	    superClass = superClass.getSuperclass();
	}

	if (sc != null) 
	    sc.paintComponent(g, c, scaleBounds, fullBounds);
	else if (c instanceof SimpleLinkComponent) {
	    SimpleLinkComponent l = (SimpleLinkComponent ) c;
	    Component source = l.getSourceComponent();
	    Component dest = l.getDestComponent();
	    if (source != null && dest != null) {
	 	Rectangle sourceBounds = source.getBounds();
	 	Rectangle destBounds = dest.getBounds();
                Point sourcePoint = scalePoint(
                    new Point(sourceBounds.x+(sourceBounds.width/2),
                        sourceBounds.y+(sourceBounds.height/2)),
                    new Dimension(scaleBounds.width, scaleBounds.height),
                    new Dimension(fullBounds.width, fullBounds.height));
                Point destPoint = scalePoint(
                    new Point(destBounds.x+destBounds.width/2,
                        destBounds.y+destBounds.height/2),
                    new Dimension(scaleBounds.width, scaleBounds.height),
                    new Dimension(fullBounds.width, fullBounds.height));
                g.setColor(Color.black);
                g.drawLine(sourcePoint.x, sourcePoint.y,
                    destPoint.x, destPoint.y);
	    }
	}
	else if (c instanceof Telepointer) {
	    // Ignore telepointers....
	}
	else 
	    super.paintComponent(g, c, scaleBounds, fullBounds);	
    }
}

