package sieve.workbench.collab;

import sieve.workbench.*;
import sieve.workbench.event.*;
import java.awt.event.*;

/**
 * This class is used by Controller to feed MouseEvents to a 
 * MouseEvent/MouseMotionEventListener, specifically a Tool object.
 * It allows a sequence of MouseEvents to be "re-played".
 */
public class MouseEventDelegator {
    MouseListener ml = null;
    MouseMotionListener mml = null;
    Workbench wb = null;

    public MouseEventDelegator(MouseListener ml, 
    	MouseMotionListener mml, Workbench wb) {
	this.ml = ml;
	this.mml = mml;
	this.wb = wb;
    }

    public void fireEvent(MouseEvent e) {
	switch(e.getID()) {
	    case MouseEvent.MOUSE_CLICKED:
		ml.mouseClicked(e);
		break;
	    case MouseEvent.MOUSE_PRESSED:
		ml.mousePressed(e);
		break;
	    case MouseEvent.MOUSE_RELEASED:
		ml.mouseReleased(e);
		break;
	    case MouseEvent.MOUSE_ENTERED:
		ml.mouseEntered(e);
		break;
	    case MouseEvent.MOUSE_EXITED:
		ml.mouseExited(e);
		break;
	    case MouseEvent.MOUSE_MOVED:
		mml.mouseMoved(e);
		break;
	    case MouseEvent.MOUSE_DRAGGED:
		mml.mouseDragged(e);
		break;
	    default:
		;
	}
    }

}



