package sieve.workbench.collab;

import sieve.workbench.*;
import java.awt.*;
import java.util.*;
import vt.cs.widgets.*; 

public class ParticipantTool extends SimpleTool implements Tool {
    private Workbench wb = null;

    public ParticipantTool() {}

    public Cursor getPreferredCursor() {
	return null;
    }

    /**
     * The ParticipantTool should never show up in a ToolBox, so if
     * we get an init method we have to be being constructed remotely
     * (ie, in Controller.doAddComponent).
     */
    public void init(Workbench wb, Class cl) {
	this.wb = wb;
	WorkbenchParticipant p = new WorkbenchParticipant(wb, false);
	AwarenessData ad = AwarenessData.getAwarenessData(wb);
	MURadarPane muRadarPane = ad.getMURadarPane();
	if (muRadarPane != null)
	    muRadarPane.addParticipant(p, p);
	wb.componentAdded(p, this);
    }


    public static WorkbenchParticipant getLocalParticipant(Workbench wb) {
	AwarenessData ad = AwarenessData.getAwarenessData(wb);

	WorkbenchParticipant p = ad.getLocalParticipant();
	if (p != null)
	    return p;
	
        p = new WorkbenchParticipant(wb, true);
	ComponentRegistry compRegistry = 
	    ComponentRegistry.getComponentRegistry(wb);

	// This is a little ugly -- the workbench won't send toolUsed()
	// unless the component was added by the current tool.
	ParticipantTool pt = new ParticipantTool();
	Tool t = wb.getCurrentTool();
	wb.setCurrentTool(pt);
	wb.componentAdded(p, pt);
	wb.setCurrentTool(t);

	ad.setLocalParticipant(p);

	return p;
    }

    public static void setMURadarPane(Workbench wb, MURadarPane muRadarPane) {
	AwarenessData ad = AwarenessData.getAwarenessData(wb);
	ad.setMURadarPane(muRadarPane);
	if (muRadarPane != null) {
	    muRadarPane.addLocalViewPort(ad.getLocalParticipant());
	}
    }

    public static void shutdown(Workbench wb) {
	AwarenessData ad = AwarenessData.getAwarenessData(wb);
	WorkbenchParticipant p = ad.getLocalParticipant();
	p.remove();
    }

    public static void removeParticipant(Workbench wb, WorkbenchParticipant p) {
	AwarenessData ad = AwarenessData.getAwarenessData(wb);
	MURadarPane muRadarPane = ad.getMURadarPane();
	if (muRadarPane != null)
	    muRadarPane.removeParticipant(p);
	wb.remove(p.getView());
	wb.componentRemoved(p);
    }

}

class AwarenessData {
    private static Hashtable instances = null;

    private Workbench wb = null;
    private MURadarPane muRadarPane = null;
    private WorkbenchParticipant localParticipant = null;
    private boolean showLabels = true;
    private boolean showTelepointers = true;

    static {
	instances = new Hashtable();
    }

    public static AwarenessData getAwarenessData(Workbench wb) {
	AwarenessData ad = (AwarenessData) instances.get(wb);
	if (ad == null) {
	    ad = new AwarenessData(wb);
	    instances.put(wb, ad);
	}
	return ad;
    }

    protected AwarenessData(Workbench wb) {
	this.wb = wb;
	// This should listen for WB component removal...
	this.muRadarPane = muRadarPane;
    }

    public MURadarPane getMURadarPane() {
	return muRadarPane;
    }

    public void setMURadarPane(MURadarPane muRadarPane) {
	this.muRadarPane = muRadarPane;
    }

    public void setLocalParticipant(WorkbenchParticipant p) {
	this.localParticipant = p;
    }
    public WorkbenchParticipant getLocalParticipant() {
	return localParticipant;
    }

}
