package sieve.workbench.collab;

import SharedData.*;
import sieve.workbench.collab.msg.Message;
import java.rmi.*;
import java.rmi.server.*;
import java.rmi.registry.*;
import java.util.*;
import java.io.*;
import java.net.*;

/**
 * Simple JSDA-based server for use by the Workbench.
 */
public class Server extends UnicastRemoteObject implements ServerInterface {
    public static final String NAME = "Sieve Workbench Server";
    public static final int REGISTRY_PORT = 14999;
    // public static final int REGISTRY_PORT = 4561;
    private static Registry rmiRegistry = null;


    public static final int DEFAULT_PORT = 4999;
    public static final String DEFAULT_HOST = "simon.cs.vt.edu";

    private String directory = null;
    private String 	hostname = null;
    private int 	hostport = REGISTRY_PORT;
    private String 	sessionType = null;

    // This table will contain SessionRecord instances.
    private Hashtable	sessions = null;

    private ServerListenerSupport listeners = null;
    
    public static void  main(String args[]) {
	String     sessionType 	= null;
	String     hostname    	= null;
	int        hostport    	= REGISTRY_PORT;
	String     directory 	= null;

	System.setSecurityManager(new java.rmi.RMISecurityManager());

        hostname    = getHost(args);
	hostport    = getPort(args);
	sessionType = getType(args);

        directory = getArg("-d", args);

	try {
	    Server server = new Server(hostname, hostport, sessionType, 
		directory);
	}
	catch (Exception e) {
	    e.printStackTrace();
	}

        // Reset the local hostname property. This is to get around problems
        // with getLocalHost().getName() in Win95, Solaris 2.4, and probably
        // others.
        try {
            String myAddress = InetAddress.getLocalHost().getHostAddress();
            Properties props = System.getProperties();
            props.put("java.rmi.server.hostname", myAddress);
            System.setProperties(props);
        }
        catch(Exception e) {
            e.printStackTrace();
        }

    }

    public Server(String hostname, int hostport) throws RemoteException {
	// SOCK this(hostname, hostport, "socket", null);
	this(hostname, hostport, "rmi", null);
    }

    public Server(String hostname, int hostport, String sessionType,
	String directory) 
	throws RemoteException {
	this.hostname = hostname;
   	this.hostport = hostport;
	this.sessionType = sessionType;
	this.directory = directory;
	sessions = new Hashtable();
	listeners = new ServerListenerSupport(this);

	try {
	    if (rmiRegistry == null) {
	        rmiRegistry = LocateRegistry.createRegistry(REGISTRY_PORT);
	        System.out.println("Constructed RMI registry: " + 
		    rmiRegistry);
	    }
	    rmiRegistry.bind(NAME, this);
	}
	catch (Exception re) {
	    re.printStackTrace();
	}
    }

    /**
     * Adds a session. JSDA has problems with session names that include
     * non-alphanumeric characters such as ":". 
     */
    public String addSession(String sessionName) 
	throws RemoteException {
	String url = "coll://" + hostname + ":" + hostport + "/" + 
	    sessionType + "/Session/" + sessionName; 

	// We don't want two clients to try to create the same session
 	// add the same time.
	synchronized(sessions) {
	    SessionRecord sr  = (SessionRecord) sessions.get(sessionName);
	    if (sr == null) {
	    	Session session = null;
	      
	    	try {
	            /* Create a session, [re]bind it and create a channel. */
System.out.println("Adding session with URL: " + url);
	            session = SessionFactory.createSession(url);
		    sessions.put(sessionName, new SessionRecord(session,
			sessionName, directory));
		    listeners.sessionAdded(sessionName);
	    	}
	    	catch (SharedDataException e) {
                    System.out.println(
			"Server: main: shared data exception: " + e);
	            e.printStackTrace();
            	}
	    }
	}
	return url;
    }
    public synchronized String[] getSessions() throws RemoteException {
	String[] names = new String[sessions.size()];
	Enumeration e = sessions.keys(); 
	for(int i = 0; i < names.length; i++)
	    names[i] = (String) e.nextElement();
	return names;
    }

    public synchronized void addChannel(String session, String channel) 
	throws RemoteException {
	addSession(session);
	SessionRecord sr = (SessionRecord) sessions.get(session);
	if (sr.getRecorder(channel) == null) {
	    sr.addChannel(channel);
	    listeners.channelAdded(session, channel);
	}
    }

    public synchronized void removeChannel(String session, String channel) 
        throws RemoteException {
    }

    public synchronized String[] getChannels(String session) 
	throws RemoteException {
	SessionRecord sr = (SessionRecord) sessions.get(session);
	if (sr == null)
	    return new String[0];
	else
	    return sr.getChannelNames();
    }

    public synchronized Message[] getStoredMessages(String session, 
	String channel) throws RemoteException {
	SessionRecord sr = (SessionRecord) sessions.get(session);
	if (sr != null) {
	    ChannelRecorder cr = sr.getRecorder(channel);
	    if (cr != null)
		return cr.getMessages();
	}
	return null;
    }
    /**
     * Adds a listener for session and channel creation/deletion
     */
    public void addServerListener(ServerListener l) throws RemoteException {
	listeners.addServerListener(l);
    }
 
    /**
     * Removes a listener for session and channel creation/deletion
     */
    public void removeServerListener(ServerListener l) throws RemoteException {
	listeners.removeServerListener(l);
    }

    private static String getHost(String args[]) {
        String returnString = getArg("-server", args);
        if (returnString != null)
      	    return returnString;
    	else
      	    return DEFAULT_HOST;
    }

    private static int getPort(String args[]) {
     	String returnString = getArg("-port", args);
    	if (returnString != null)
      	    return(Integer.parseInt(returnString));
        else
      	    return REGISTRY_PORT;
      	    // SOCK return DEFAULT_PORT;
    }

    private static String getType(String args[]) {
    	String returnString = getArg("-type", args);
    	if (returnString != null)
      	    return returnString;
    	else
      	    return "rmi";
      	    // SOCK return "socket";
    }


    private static String getArg(String parameter, String args[]) {
    	int length = args.length;
    	for (int i = 0; i < length; i++) {
      	    if (args[i].equals(parameter)) {
		if (++i < length) {
	  	    return(args[i]);
		}
      	    }    
    	}
    	return null;
    }

/*
    // This is *really* useless, since we don't know when channels are
    // created...
    class ServerSessionObserver implements SessionObserver {
	public void sessionChanged(SessionEvent e) {
	    String  clientName = e.getClientName();
            Session session    = e.getSession();
            int     type       = e.getType();
            if (type == SessionEvent.JOINED) {
                // deal with session join...
            }
	}
	public String getName() {
	    return "Workbench Session Listener";
	}
    }
*/
}

// SOCK: Making these non-inner...

    class SessionRecord implements Client, Serializable {
	private Session session = null;
	private String name = null;
	private Hashtable channels = null;
	private Vector channelNames = null;
	private File directory = null;
	private Saver saver = null;


	public SessionRecord(Session session, String name, String directory) {
	    this.session = session;
	    this.name = name;
	    channelNames = new Vector();
	    channels = new Hashtable();
	    if (directory != null) {
	        try {
	            this.directory = new File(directory, name);
	            this.directory.mkdirs();
		    String[] files = this.directory.list();
		    for (int i = 0; i < files.length; i++)
		    	channelNames.addElement(files[i]);
	        }
	        catch (Exception e) {
		    e.printStackTrace();
	        }
		saver = new Saver(this, this.directory);
	    }
	}

      // ----------- Client Interface methods ---------

      public String authenticate(AuthenticationInfo info) {
	return null;
      }

      public void clientChanged(ClientEvent event) { }
      
      public String getName() {
	return name;
      }
      // ------------------------------------------------ 


        public Session getSession() {
	    return session;
	}

	public ChannelRecorder getRecorder(String channel) {
	    ChannelRecorder recorder = (ChannelRecorder) channels.get(channel);
	    if ((recorder == null) && (directory != null)) {
		try {
		    File file = new File(directory, channel);
		    if (file.exists()) {
		        ObjectInputStream fin = new ObjectInputStream(
			    new BufferedInputStream(
			    new FileInputStream(file)));
			recorder = (ChannelRecorder)fin.readObject();
			channels.put(channel, recorder);
			try {
	            	    getSession().createChannel(this, channel, true, 
				true);
	            	    ServerChannelObserver observer = 
				new ServerChannelObserver(getSession(), 
				channel, this);
	 		}
			catch (SharedDataException e) {
		    	    e.printStackTrace();
			}
		    }
		}
		catch (Exception e) {
		     e.printStackTrace();
		}
	    }
	    return recorder;
	}

    	public void addChannel(String name) {
	    if (channels.get(name) == null) {
	        channelNames.addElement(name);
	        // ChannelRecorder recorder = new FullReplayRecorder(name);
	        ChannelRecorder recorder = new MinimalReplayRecorder(name);
	        channels.put(name, recorder);
		try {
	            getSession().createChannel(this, name, true, true);
	            ServerChannelObserver observer = new ServerChannelObserver(
		    	getSession(), name, this);
	 	}
		catch (SharedDataException e) {
		    e.printStackTrace();
		}
	        scheduleSave(name);
	    }
	}
        public synchronized String[] getChannelNames() {
	    String[] names = new String[channelNames.size()];
	    channelNames.copyInto(names);
	    return names;
	}

	public void scheduleSave(String channel) {
	    if (saver != null) {
	    	ChannelRecorder recorder = 
		    (ChannelRecorder) channels.get(channel);
		if (recorder != null)
		    saver.addRecorder(recorder);
	    }
	}

	private class Saver extends Thread implements Serializable {
	    private SessionRecord sr = null;
	    private Vector queue = null;
	    private File directory = null;

	    public Saver(SessionRecord sr, File directory) {
		this.sr = sr;
		this.directory = directory;
		queue = new Vector();
		setDaemon(true);
		start();
	    }
	    public void addRecorder(ChannelRecorder r) {
		synchronized(queue) {
		    if (!queue.contains(r))
		   	queue.addElement(r);
		}
	    }
	    public synchronized void run() {
		while (true) {
		    while (queue.isEmpty()) {
			try {
			    wait(300 * 1000);
			}
			catch(InterruptedException e) {}
		    }
		    Vector queueClone = null;
		    synchronized(queue) {
			queueClone = (Vector)queue.clone();
			queue.removeAllElements();
		    }
		    for (Enumeration e = queueClone.elements(); 
			e.hasMoreElements();) {
			ChannelRecorder recorder = 
			    (ChannelRecorder) e.nextElement();
			try {
		    	    File file = new File(directory, recorder.getName());
		    	    ObjectOutputStream fout = new ObjectOutputStream(
			        new FileOutputStream(file));
		    	    fout.writeObject(recorder);
			}
			catch (IOException ex) {
		     	    ex.printStackTrace();
			}
		    }
		}
	    }
	}

    }

    class ServerChannelObserver implements ChannelObserver, Client,
	ChannelConsumer  {
	private SessionRecord sessionRec = null;
	private Channel channel = null;
	private String name = null;
 	private String channelName = null;

	public ServerChannelObserver(Session session, 
	    String channelName, SessionRecord sessionRec) {
	    this.sessionRec = sessionRec;
	    this.name = "ServerChannelObserver " + new Date();
	    this.channelName = channelName;

	    try {
		session.join(this);
	        channel = Channel.join(session, channelName, this);
		channel.addObserver(this);
		channel.addConsumer(this, this);
	    }
	    catch (SharedDataException e) {
		System.out.println("Got SharedDataException: " + 
		    e.typeToString(e.getType()));
		e.printStackTrace();
	    }
	}

	public synchronized void channelChanged(ChannelEvent e) { }

	public String getName() {
	    return name;
	}
	public String authenticate(AuthenticationInfo info) {
	    return "";
	}
	public void clientChanged(ClientEvent event) {}

	public void dataReceived(Data data) {
            Message m = Message.getMessage(data);
	    ChannelRecorder recorder = sessionRec.getRecorder(channelName);
	    //int msgs = recorder.getMessageCount();
             m.deliver(recorder);
	    //if (recorder.getMessageCount() != msgs)
	        sessionRec.scheduleSave(channelName);
        }
    }


