package sieve.workbench.collab;

import java.rmi.*;

/**
 * Interface implemented by objects which want to be notified of 
 * interesting things that happen on a (remote) server. Normally the
 * arguments to these methods would be a subclass of java.util.EventObject.
 * I have not done that here in an attempt to reduce the amount of
 * stuff that has to be serialized.
 */
public interface ServerListener extends Remote {

    /**
     * Called when a new session is added on the server.
     */
    public void sessionAdded(String sessionName) throws RemoteException;

    /**
     * Called when a new channel is added in a session.
     */
    public void channelAdded(String sessionName, String channelName)
	throws RemoteException;

    /**
     * Called when a session is removed.
     */
    public void sessionRemoved(String sessionName)
	throws RemoteException;

    /**
     * Called when a channel is removed.
     */
    public void channelRemoved(String sessionName, String channelName)
	throws RemoteException;
}


