/*
 * %W% %E%  James Bo Begole
 *
 * Copyright (c) 1997 James Begole,  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * James Begole MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. James Begole 
 * SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */
 
package sieve.workbench.collab;

import java.awt.*;

/**
 * A Telepointer
 *
 * @version 	$Revision: 1.1 $
 * @author 	James Begole 
 */


public class Telepointer extends Component {
    /**
     * Location of the telepointer
     */
    Point location = null;

    /**
     * Participant associated with this telepointer.
     */
    Participant participant = null;

   /**
    * Flag to show participant's name or not.
    */
   boolean showName = false;
	 
    /**
     * The Font to draw Participant name, if shown.
     */
    Font font = new Font("serif", Font.PLAIN, 12);
	  
    /**
     * The FontMetrics of the specified font.
     */
    FontMetrics fontMetrics = Toolkit.getDefaultToolkit().getFontMetrics(font);

    /**
     * The outside cursor polygon.  All instances use this shape.
     */
    static Polygon outside = null;
    static int outsidePoints = 13;
    static int[] outsideX = new int[outsidePoints];
    static int[] outsideY = new int[outsidePoints];
    {
	/*
	 * points = (0,0),(0,1),(9,18),(10,18),(9,11),(18,20),(20,20),
	 *          (20,17),(12,9),(18,10),(19,9),(1,0),(0,0)
	 */	
		int i = 0;
		outsideX[i++] = 0;
		outsideX[i++] = 0;
		outsideX[i++] = 9;
		outsideX[i++] = 10;
		outsideX[i++] = 9;
		outsideX[i++] = 18;
		outsideX[i++] = 20;
		outsideX[i++] = 20;
		outsideX[i++] = 12;
		outsideX[i++] = 18;
		outsideX[i++] = 19;
		outsideX[i++] = 1;
		outsideX[i++] = 0;
		i=0;
		outsideY[i++] = 0;
		outsideY[i++] = 1;
		outsideY[i++] = 18;
		outsideY[i++] = 18;
		outsideY[i++] = 11;
		outsideY[i++] = 20;
		outsideY[i++] = 20;
		outsideY[i++] = 17;
		outsideY[i++] = 9;
		outsideY[i++] = 10;
		outsideY[i++] = 9;
		outsideY[i++] = 0;
		outsideY[i++] = 0;
		outside = new Polygon(outsideX, outsideY, outsidePoints);
	}
	
	/**
	* The inside cursor polygon.  All instances use this shape.
	*/
	static Polygon inside = null;
	static int insidePoints = 9;
	static int[] insideX = new int[insidePoints];
	static int[] insideY = new int[insidePoints];
	{
	/*
	 * points = (0,0),(9,17),(8,9),(18,19),(19,19),(19,17),
	 *          (10,8),(18,9),(0,0)
	 */
		int i = 0;
		insideX[i++] = 0;
		insideX[i++] = 9;
		insideX[i++] = 8;
		insideX[i++] = 18;
		insideX[i++] = 19;
		insideX[i++] = 19;
		insideX[i++] = 10;
		insideX[i++] = 18;
		insideX[i++] = 0;
		i=0;
		insideY[i++] = 0;
		insideY[i++] = 17;
		insideY[i++] = 9;
		insideY[i++] = 19;
		insideY[i++] = 19;
		insideY[i++] = 17;
		insideY[i++] = 8;
		insideY[i++] = 9;
		insideY[i++] = 0;
		inside = new Polygon(insideX, insideY, insidePoints);
	}


    /**
     * Creates a telepointer.
     * @param participant The participant represented by this telepointer.
     */
    public Telepointer(Participant participant) {
	setSize(getMinimumSize());
	this.participant = participant;
    }

    public Color getColor() {
	return participant.getColor();
    }

    public void update(Graphics g) {
        paint(g);
    }

    public void paint(Graphics g) {
    	if (!isVisible())
      	    return;

     	g.setColor(Color.white);
    	g.fillPolygon(outside);
    	g.setColor(getColor());
    	g.fillPolygon(inside);
    	if (showName) {
	    Rectangle bounds = outside.getBounds();
	    g.drawString(participant.getName(), bounds.width+1, 
		bounds.height-(fontMetrics.getMaxDescent()+1));
    	}
    }
    public void print(Graphics g) { 
	// Do nothing...
    }
    
    public boolean contains(int x, int y) {
  	return false;
    }
    
    public void setShowName(boolean value) {
    	showName = value;
        setSize(getMinimumSize());
    }
    
    public boolean getShowName() {
    	return showName;
    }
    
    public void setFont(Font font) {
    	this.font = font;
	fontMetrics = Toolkit.getDefaultToolkit().getFontMetrics(font);
        setSize(getMinimumSize());
    }
    
    public Font getFont() {
    	return font;
    }
    
    public Dimension getMinimumSize() {
    	Rectangle bounds = outside.getBounds();
    	int extraWidth = showName ? 
    		fontMetrics.stringWidth(participant.getName())+15 :
    		0;
    	return new Dimension(bounds.width + extraWidth, 
		bounds.height);
    }
    
    public Dimension getPreferredSize() {
    	return getMinimumSize();
    }

    public void setLocation(Point location) {
        Dimension minSize = getMinimumSize();
	this.location = location;
        if (location != null)
      	    setBounds(location.x, location.y, minSize.width, minSize.height);
    	else
      	    setSize(minSize);
    }
    
}

