package sieve.workbench.collab;

import sieve.workbench.*;
import vt.cs.widgets.*;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.util.Observable; // RadarPane needs this :-(
import java.util.Hashtable;

import com.sun.java.swing.SwingUtilities;
import com.sun.java.swing.JLayeredPane;

public class WorkbenchParticipant extends Observable implements 
    Participant, WorkbenchComponent, RadarViewPort, MouseMotionListener, 
    MURadarPaneParent, ContainerListener {

    private UniqueID id = null;
    private Workbench wb = null;
    private boolean isLocal = false;
    private transient Telepointer telepointer = null;

    private String name = "(unknown)";
    private Color color = Color.black;
    private Rectangle bounds = new Rectangle(0, 0, 0, 0);
    private Point location = new Point(0, 0);

    static {
	ToolFactory.getToolFactory().registerTool(WorkbenchParticipant.class,
	    ParticipantTool.class);
    }

    public WorkbenchParticipant(Workbench wb, boolean isLocal) {
	telepointer = new Telepointer(this);
	telepointer.setShowName(true);
	id = new UniqueID(this);
	this.wb = wb;
	this.isLocal = isLocal;
	if (isLocal) {
	    wb.addMouseMotionListener(this);
	    wb.addContainerListener(this);
	}
	else
	    wb.add(telepointer, JLayeredPane.PALETTE_LAYER);
    }

    public boolean isLocal() {
	return isLocal;
    }

    // ----------- WorkbenchComponent Methods -----------
    public Component getView() {
	return telepointer;
    }
    public Object getModel() {
	return this;
    }
    public void remove() {
	wb.remove(telepointer);
	wb.componentRemoved(this);
    }
    public UniqueID getUniqueID() {
	return id;
    }
    public void setUniqueID(UniqueID id) {
	this.id = id;
    }
    public boolean isSelected() {
	return false;
    }
    public void setSelected(boolean selected) {}

    public void setColorSource(ColorSource s) {}
    // --------------------------------------------------

    // ----------- Notify workbench of property changes -----------
    public void setColor(Color color) {
	Color orig = this.color;
	this.color = color;
	telepointer.repaint();
	notifyObservers("Color"); // :-(
	if (isLocal)
	    wb.componentEdited(this, new PropertyChangeEvent(this,
		"color", orig, color));
    }
    public Color getColor() {
	return color;
    }

    public String getName() {
	return name;
    }
    public void setName(String name) {
	String orig = this.name;
	this.name = name;
	telepointer.repaint();
	notifyObservers("Name"); // :-(
	if (isLocal)
	    wb.componentEdited(this, new PropertyChangeEvent(this,
		"name", orig, name));
    }

    public void setBounds(Rectangle bounds) {
	Rectangle orig = this.bounds;
	this.bounds = bounds;
	notifyObservers("Bounds"); // :-(
	if (isLocal)
	    wb.componentEdited(this, new PropertyChangeEvent(this,
		"bounds", orig, bounds));
    }

    // ------------------------------------------------------------

    // --------------- MouseMotionListener  Methods ---------------
    public void setLocation(Point location) {
	Point orig = this.location;
	this.location = location;
	telepointer.setLocation(location);
	telepointer.repaint();
	notifyObservers("Location"); // :-(
	if (isLocal)
	    wb.componentEdited(this, new PropertyChangeEvent(this,
		"location", orig, location));
	else
	    wb.repaint();
    }

    public Point getLocation() {
	return location;
    }

    public void mouseMoved(MouseEvent e) {
	setLocation(SwingUtilities.convertPoint(e.getComponent(), 
	    e.getPoint(), wb));
    }
	
    public void mouseDragged(MouseEvent e) {
	setLocation(SwingUtilities.convertPoint(e.getComponent(), 
	    e.getPoint(), wb));
    }
    // ------------------------------------------------------------

    // ------------------------------------------------------------
    private static Hashtable eventModelCache = new Hashtable();
    /**
     * This was lifted from BeanBox. We need to check for classs which
     * use the "old" (1.02) AWT event model and *NOT* register ourself
     * as a listener for those. If we do, then the old-style events
     * aren't delivered.
     */ 
    private synchronized boolean useNewEventModel(Object child) {
        // Check our cache first.
        Boolean b = (Boolean) eventModelCache.get(child.getClass());
        if (b != null) 
            return b.booleanValue();
 
        // We check whether the bean has any public methods that take
        // old AWT event objects.  If so, we assume it wants the
        // old event model, otherwise we assume it wants the new.
        boolean useNew = true;
        try {
            Class clz = child.getClass();
            while (useNew && clz != null) {
                java.lang.reflect.Method methods[] = clz.getDeclaredMethods();
                for (int i = 0; i < methods.length; i++) {
                    java.lang.reflect.Method m = methods[i];
                    int mods = m.getModifiers();
                    // Skip non=public methods.
                    if (!java.lang.reflect.Modifier.isPublic(mods)) {
                        continue;
                    }
                    Class params[] = m.getParameterTypes();
                    if (params.length > 0 && params[0] == java.awt.Event.class){
                        // First arg is java.awt.Event.  We assume this is an
                        // old-style AWT event handler method.
                        useNew = false;
                        break;
                    }
                }
                clz = clz.getSuperclass();
                if (clz.getName().indexOf("java.") == 0) {
                    // We've reached a java.* class, so we're done.
                    break;
                }
            }
        } catch (Exception ex) {}
        eventModelCache.put(child.getClass(), new Boolean(useNew));
        return useNew;
    }
    // ------------------------------------------------------------

    // ---------------- ContainerListener Methods -----------------
    public void componentAdded(ContainerEvent e) {
	addTelepointerListener(e.getChild());
    }
    private void addTelepointerListener(Component comp) {
	if (useNewEventModel(comp)) {
	    comp.addMouseMotionListener(this);
	    if (comp instanceof Container) {
	    	Container c = (Container) comp;
	    	c.addContainerListener(this);
	    	Component[] comps = c.getComponents();
	    	for (int i = 0; i < comps.length; i++) 
	            addTelepointerListener(comps[i]);
	    }
  	}
    }
    private void removeTelepointerListener(Component comp) {
	if (useNewEventModel(comp)) {
	    comp.removeMouseMotionListener(this);
	    if (comp instanceof Container) {
	    	Container c = (Container) comp;
	    	c.removeContainerListener(this);
	    	Component[] comps = c.getComponents();
	    	for (int i = 0; i < comps.length; i++) 
	            removeTelepointerListener(comps[i]);
	     }
	}
    }
    public void componentRemoved(ContainerEvent e) {
	removeTelepointerListener(e.getChild());
    }   
    // ------------------------------------------------------------

    // ------------------- RadarViewPort Methods ------------------
    public String getLabel() {
	return getName();
    }
    public Rectangle getBounds() {
	return bounds; // Fix this!
    }
    public Point getCursorPosition() {
	return location;
    }
    // ------------------------------------------------------------

    // ------------------- MURadarParent Methods ------------------
    public void updateBounds(Rectangle bounds) {
	setBounds(bounds);
    }
    // ------------------------------------------------------------

}

