package sieve.workbench.collab.msg;

import sieve.workbench.UniqueID;
import sieve.workbench.collab.WorkbenchController;
import java.awt.event.MouseEvent;
import java.io.Serializable;
import java.beans.Beans;

public class AddComponentMessage extends Message implements Serializable {
    private UniqueID id = null;

    // We'd rather just pass a Class object here, but then _all_ of our
    // classes have to be marked Serializable :-(
    private String className = null;
    private MouseEvent[] evts = null;

    public AddComponentMessage(UniqueID objID, String className, 
	MouseEvent[] evts) {
	this.id = objID;
	this.className = className;

	this.evts = new MouseEvent[evts.length];
	for (int i = 0; i < evts.length; i++) {
	    this.evts[i] = evts[i];
	}
    }

    public AddComponentMessage(UniqueID objID, Class cl, MouseEvent[] evts) {
	this(objID, cl.getName(), evts);
    }

    public void deliver(WorkbenchController controller) {
	controller.doAddComponent(className, evts, id);

/* We don't do this here because we don't want the WorkbenchController
   i/f to know anything about classes. If it does, then all the classes
   have to be serializable, and everybody has to have the .class files.
   This causes a problem on the server.

	Class cl = null;
	// This is a problem. We try Class.forName first to see if the
	// system classloader can find this class. Otherwise our only
	// other option is to try to Jar loader. There are probably
	// cases where neither of these options will work. Hmmmmmm....
	try {
	    cl = Class.forName(className);
	}
	catch(Exception e) {
	    try {
		cl = sieve.workbench.jar.SimpleClassLoader.ourLoader.loadClass(
		    className);
	    }
	    catch(Exception ex) {
	        ex.printStackTrace();
	    }
	}
	if (cl != null)
	    controller.doAddComponent(cl, evts, id);
*/
    }
}

