package sieve.workbench.collab.msg;

import java.util.*;
import java.io.Serializable;

import sieve.workbench.UniqueID;
import sieve.workbench.collab.WorkbenchController;
import sieve.workbench.event.EditEvent;
import SharedData.Data;

public class EditMessage extends Message implements Serializable {
    /**
     * This hash table will store unacknowledged edit messages. Each edit
     * message is sent to all sessions, including the session that originates
     * it. On each deliver request, an edit message:
     * <OL>
     * <LI> Checks to see if it originated from the current session. If it
     * did, then it is removed from the unacknowledged list and is not
     * delivered.
     * <LI> Otherwise, it a check is made to see if its event is obsoleted
     * by any event in an unacknowledged message. If so, the message is
     * not delivered.
     * </OL>
     * The unacked table is a table of Vectors of messages, with one such
     * Vector per unique id (ie, one per WorkBenchComponent)
     */
    private static Hashtable unacked = new Hashtable();
    private static long myID = System.currentTimeMillis(); // Hmmm....
    private static boolean isClient = false;
    private static int count = 0;

    private UniqueID id = null;
    private EditEvent ee = null;
    private long msgID = myID;
    private int seqNumber = 0;

    public EditMessage(UniqueID objID, EditEvent e) {
	this.id = objID;
	this.ee = e;
	this.seqNumber = count++;
	if (isClient) {
            getVector().addElement(this);
	}
    }

    private Vector getVector() {
	Vector v = (Vector)unacked.get(id);
	if (v == null) {
	    v = new Vector(2);
	    unacked.put(id, v);
	}
	return v;
    }

    public EditEvent getEditEvent() {
	return ee;
    }

    public void deliver(WorkbenchController controller) {
	boolean ignore = false;
	synchronized (unacked) {
	    if (isClient) {
	        // If this message came from this session, ignore it and removed
	        // it from the unack'd list.
	        if (myID == msgID) {
	            ignore = true;
		    boolean found = false;
		    Vector tmp = (Vector)getVector().clone();
	            for (Enumeration e = tmp.elements(); 
		        e.hasMoreElements() && ! found;) {
		        EditMessage em = (EditMessage)e.nextElement();
			if (em.seqNumber <= seqNumber) {
			    getVector().removeElement(em);
			}
		    }
	        }
	        // Otherwise, see if any of the unacknowledged messages 
	        // obsolete this one.
	        else {
	            for (Enumeration e = getVector().elements(); 
		        e.hasMoreElements() && ! ignore;) {
		        EditMessage em = (EditMessage)e.nextElement();
		        if (ee.isObsoletedBy(em.getEditEvent())) {
			    ignore = true;
			}
		    }
	        }
	    }
	}

	if (! ignore)
	    controller.doEditComponent(id, ee);
    }

    /**
     * Returns true to force this message to be sent to all sessions.
     */
    public boolean sendToAll() {
	return true;
    }

    public Data toData() {
	isClient = true;
	return super.toData();
    }

    public String toString() {
	return super.toString() + "[" + ee.toString() + "]";
    }

}

