package sieve.workbench.collab.msg;

import sieve.workbench.UniqueID;
import sieve.workbench.collab.WorkbenchController;
import java.io.Serializable;

public class MakeLinkMessage extends Message implements Serializable {

    // Passing the class name here is a little less-than-ideal. See
    // the notes on this in AddComponentMessage and Controller.
    private String linkClassName = null; 

    private UniqueID srcID = null;
    private UniqueID destID = null;
    private UniqueID linkID = null;
    private Serializable state = null;

    public MakeLinkMessage(Class linkClass, UniqueID srcID, UniqueID destID, 
	UniqueID linkID, Serializable state) {
	this(linkClass.getName(), srcID, destID, linkID, state);
    }

    public MakeLinkMessage(String linkClassName, UniqueID srcID, 
	UniqueID destID, UniqueID linkID, Serializable state) {
	this.linkClassName = linkClassName;
	this.srcID = srcID;
	this.destID = destID;
	this.linkID = linkID;
	this.state = state;
    }

    public void deliver(WorkbenchController controller) {

/* We don't do this here because we don't want the WorkbenchController
   i/f to know anything about classes. If it does, then all the classes
   have to be serializable, and everybody has to have the .class files.
   This causes a problem on the server.

        Class cl = null;
        // This is a problem. We try Class.forName first to see if the
        // system classloader can find this class. Otherwise our only
        // other option is to try to Jar loader. There are probably
        // cases where neither of these options will work. Hmmmmmm....
        try {
            cl = Class.forName(linkClassName);
        }
        catch(Exception e) {
            try {
                cl = sieve.workbench.jar.SimpleClassLoader.ourLoader.loadClass(
                     linkClassName);
            }
            catch(Exception ex) {
                ex.printStackTrace();
            }
        }
	controller.doMakeLink(cl, srcID, destID, linkID, state);
*/
	controller.doMakeLink(linkClassName, srcID, destID, linkID, state);
    }
}


