/*
 *
 * Written By:   Philip Isenhour
 * Date:         July 1997
 * Description:  Abstraction of a Sieve workbench message.
 *
 * $Header: /home/nie/cvsroot/sieve/sieve/workbench/collab/msg/Message.java,v 1.11 1998/01/03 05:54:45 isenhour Exp $
 *
 * Copyright (c) 1996 Philip Isenhour
 * Copyright (c) 1996 Virginia Polytechnic University and State University
 */

package sieve.workbench.collab.msg;

import java.io.*;
import SharedData.*;
import sieve.workbench.collab.*;

/**
 * Abstraction of a Sieve workbench message. The design of this class
 * should allow these to simply be pulled of the wire and then have their 
 * deliver() method called with any WorkbenchController object.
 *
 * @version $Id: Message.java,v 1.11 1998/01/03 05:54:45 isenhour Exp $
 */
public abstract class Message implements Serializable {

    /**
     * Delivers a message to the specified WorkbenchController 
     * (e.g., sieve.workbench.collab.Controller).
     */
    public abstract void deliver(WorkbenchController controller);

    /**
     * Returns true if this message should be sent to all sessions,
     * including the one that originated it. By default this returns
     * false, indicating that the message should only be sent to sessions
     * other than the one that originated it.
     */
    public boolean sendToAll() {
	return false;
    } 

    /**
     * Utility routine for serializing an arbitrary object into a byte
     * array.
     */
    public static byte[] toByteArray(Object obj) {
    	ByteArrayOutputStream byteArrayStream = null;
    	ObjectOutputStream objectStream = null;
    	byte[] bytes = null;

    	try {
      	    byteArrayStream = new ByteArrayOutputStream();
      	    objectStream = new ObjectOutputStream(byteArrayStream);

      	    objectStream.writeObject(obj);
      	    bytes = byteArrayStream.toByteArray();
        }
    	catch(Exception e) {
      	    e.printStackTrace();
        }
    	return bytes;
    }

    /**
     * Converts this message to a JSDA Data object.
     */
    public Data toData() {
        return new Data(toByteArray(this));
    }

    /**
     * Deserializes a message object from the contents of a byte array.
     */
    public static Message getMessage(byte[] data) {
    	ByteArrayInputStream byteArrayStream = null;
    	ObjectInputStream objectStream = null;
    	Message msg = null;
 
    	try {
      	    byteArrayStream = new ByteArrayInputStream(data);
            objectStream = new ObjectInputStream(byteArrayStream);
 
            msg = (Message)objectStream.readObject();
    	}
    	catch(Exception e) {
      	    e.printStackTrace();
    	}
	return msg;
    }

    /**
     * Extracts a Message object from a JSDA Data object.
     */
    public static Message getMessage(Data data) {
        return getMessage(data.getData());
    }

}

