package sieve.workbench.event;

import java.io.Serializable;
import java.beans.*;
import java.lang.reflect.*;

/**
 * A wrapper around PropertyChangeEvent which implements the two methods
 * required by EditEvent.
 */
public class PropertyEditEvent extends EditEvent implements Serializable {
    private transient PropertyChangeEvent pce = null;

    // DOH! There is currently *NO* IndexedPropertyChangeEvent... :-(
    // private boolean isIndexed = false;
    // private int index = 0;

    private String propertyName = null;
    private Object newValue = null;

    public PropertyEditEvent(PropertyChangeEvent pce) {
	super(pce.getSource());
	this.pce = pce;
	this.propertyName = pce.getPropertyName();
	this.newValue = pce.getNewValue();
    }

    public PropertyChangeEvent getPropertyChangeEvent() {
	// This will happen only if we've been de-serialized...
	if (pce == null)
	    pce = new PropertyChangeEvent(this, propertyName, null, newValue);
	return pce;
    }

    public String getPropertyName() {
	return propertyName;
    }

    public Object getNewValue() {
	return newValue;
    }
    
    /**
     * Applies the change represented by this event to another object.
     * Currently we fail quietly if the target object isn't of the right
     * type or if the change cannot be applied. 
     */
    public void applyChange(Object target) {
	try {
	    PropertyDescriptor pd =
                new PropertyDescriptor(propertyName, target.getClass());
	    Object[] args = { newValue };
            pd.getWriteMethod().invoke(target, args);
	}
	catch(InvocationTargetException ex) {
            ex.printStackTrace();
            System.err.println("Embedded exception:");
            ex.getTargetException().printStackTrace();
        }
        catch(Exception exe) {
            exe.printStackTrace();
        }
    }

    /**
     * Returns true if the new event obsoletes this event. In this case,
     * we return true if the property names match.
     */
    public boolean isObsoletedBy(EditEvent newEvent) {
	return ((newEvent instanceof PropertyEditEvent) &&
	    (((PropertyEditEvent)newEvent).getPropertyName().equals(
	    propertyName)));
    }
 
    /**
     * We return true if the property name and values match.
     */
    public boolean isEquivalentTo(EditEvent newEvent) {
	if (newEvent instanceof PropertyEditEvent) {
	    PropertyEditEvent pe = (PropertyEditEvent)newEvent;
	    return (pe.getPropertyName().equals(propertyName)) && 
		((pe.getNewValue() == newValue) || 
		    ((pe.getNewValue() != null) && (newValue != null) &&
		    (newValue.equals(pe.getNewValue()))));
	}
	return false;
    }

    public String toString() {
	return super.toString() + "[" + propertyName + " -> " + newValue + "]";
    }
}


