package sieve.workbench.event;
 
import sieve.workbench.*;

/**
 * WorkbenchListeners are notified of interesting events on the workbench.
 */
public interface WorkbenchListener extends java.util.EventListener {
    /**
     * Called when a component is added to the workbench. Note that this
     * notification is not sent until the component has been completely
     * constructed.
     */
    public void componentAdded(WorkbenchComponentEvent e);

    /**
     * Called when a component is selected. 
     */
    public void componentSelected(WorkbenchComponentEvent e);

    /**
     * Called when a component is deselected. 
     */
    public void componentDeselected(WorkbenchComponentEvent e);

    /**
     * Called when a component is removed from the workbench. 
     */
    public void componentRemoved(WorkbenchComponentEvent e);

    /**
     * Called when the bounds of a component are changed.
     */
    public void componentBoundsChanged(WorkbenchComponentEvent e);

    /**
     * Called when a component has been edited. (This should probably just
     * use PropertyChangeEvent)
     */
    public void componentEdited(WorkbenchComponentEvent e);

    /**
     * Called when the z-order of a component is changed.
     */
    public void componentZOrderChanged(WorkbenchComponentEvent e);

    /**
     * Called when a link is started.
     */
    public void linkStarted(LinkEvent e);

    /**
     * Called when a link is added.
     */
    public void linkAdded(LinkEvent e);

    /**
     * Called when a link is removed.
     */
    public void linkRemoved(LinkEvent e);

}

